@@ -1,4 +1,4 @@
-/* $Id: pdflib.java,v 1.13.2.2 2001/05/11 12:38:08 york Exp $
+/* $Id: pdflib.java,v 1.13.2.21 2002/06/15 20:41:53 rjs Exp $
  *
  * PDFlib Java class
  */
@@ -12,7 +12,7 @@ import java.io.*;
     please refer to the PDFlib API reference manual which is available
     in the PDF file PDFlib-manual.pdf in the PDFlib distribution.
     @author Thomas Merz
-    @version 4.0.1
+    @version 4.0.3
 */
 
 public final class pdflib {
@@ -26,11 +26,12 @@ public final class pdflib {
 	    System.loadLibrary("pdf_java");
 	} catch (UnsatisfiedLinkError e) {
 	    System.err.println(
-	    "Cannot load the PDFlib shared library/DLL for Java.\n" +
-	    "Make sure that LD_LIBRARY_PATH (Unix) or PATH (Win32)\n" +
-	    "points to the directory with the PDFlib shared library/DLL.\n" +
-	    e);
-	    System.exit(1);
+	"Cannot load the PDFlib shared library/DLL for Java.\n" +
+	"Make sure to properly install the native PDFlib library.\n\n" +
+	"For your information, the current value of java.library.path is:\n" +
+	 System.getProperty("java.library.path") + "\n");
+
+	    throw e;
 	}
 	PDF_boot();
     }
@@ -47,6 +48,18 @@ public final class pdflib {
 	p = PDF_new();
     }
 
+    /** Explicitly delete a PDF object. This may be called by the finalizer,
+        too, but it is safe to issue multiple calls of PDF_delete().
+     */
+    public final void delete() throws
+    OutOfMemoryError, IOException, IllegalArgumentException,
+    IndexOutOfBoundsException, ClassCastException, ArithmeticException,
+    RuntimeException, InternalError, UnknownError
+    {
+	PDF_delete(p);
+	p = (long) 0;
+    }
+
     /** Open a new PDF file associated with p, using the supplied file name.
         @return -1 on error, 1 on success.
      */
@@ -467,8 +480,7 @@ public final class pdflib {
 	PDF_closepath_fill_stroke(p);
     }
 
-    /** End the current path.
-	@deprecated Use one of the stroke, fill, or clip functions instead.
+    /** End the current path without filling or stroking it.
     */
     public final void endpath() throws
     OutOfMemoryError, IOException, IllegalArgumentException,
@@ -487,60 +499,72 @@ public final class pdflib {
 	PDF_clip(p);
     }
 
-    /** Set the current fill color to a gray value between 0 and 1 inclusive. */
+    /** Set the current fill color to a gray value between 0 and 1 inclusive.
+	@deprecated Use setcolor("fill", "gray", g, 0, 0, 0) instead.
+    */
     public final void setgray_fill(float g) throws
     OutOfMemoryError, IOException, IllegalArgumentException,
     IndexOutOfBoundsException, ClassCastException, ArithmeticException,
     RuntimeException, InternalError, UnknownError
     {
-	PDF_setgray_fill(p, g);
+	PDF_setcolor(p, "fill", "gray", g, 0, 0, 0);
     }
 
     /** Set the current stroke color to a gray value between 0 and 1
         inclusive.
-      */
+	@deprecated Use setcolor("stroke", "gray", g, 0, 0, 0) instead.
+    */
     public final void setgray_stroke(float g) throws
     OutOfMemoryError, IOException, IllegalArgumentException,
     IndexOutOfBoundsException, ClassCastException, ArithmeticException,
     RuntimeException, InternalError, UnknownError
     {
-	PDF_setgray_stroke(p, g);
+	PDF_setcolor(p, "stroke", "gray", g, 0, 0, 0);
     }
 
-    /** Set the current fill and stroke color. */
+    /** Set the current fill and stroke color.
+        inclusive.
+	@deprecated Use setcolor("both", "gray", g, 0, 0, 0) instead.
+    */
     public final void setgray(float g) throws
     OutOfMemoryError, IOException, IllegalArgumentException,
     IndexOutOfBoundsException, ClassCastException, ArithmeticException,
     RuntimeException, InternalError, UnknownError
     {
-	PDF_setgray(p, g);
+	PDF_setcolor(p, "both", "gray", g, 0, 0, 0);
     }
 
-    /** Set the current fill color to the supplied RGB values. */
+    /** Set the current fill color to the supplied RGB values.
+	@deprecated Use setcolor("fill", "rgb", red, green, blue, 0) instead.
+    */
     public final void setrgbcolor_fill(float red, float green, float blue) throws
     OutOfMemoryError, IOException, IllegalArgumentException,
     IndexOutOfBoundsException, ClassCastException, ArithmeticException,
     RuntimeException, InternalError, UnknownError
     {
-	PDF_setrgbcolor_fill(p, red, green, blue);
+	PDF_setcolor(p, "fill", "rgb", red, green, blue, 0);
     }
 
-    /** Set the current stroke color to the supplied RGB values. */
+    /** Set the current stroke color to the supplied RGB values.
+	@deprecated Use setcolor("stroke", "rgb", red, green, blue, 0) instead.
+    */
     public final void setrgbcolor_stroke(float red, float green, float blue) throws
     OutOfMemoryError, IOException, IllegalArgumentException,
     IndexOutOfBoundsException, ClassCastException, ArithmeticException,
     RuntimeException, InternalError, UnknownError
     {
-	PDF_setrgbcolor_stroke(p, red, green, blue);
+	PDF_setcolor(p, "stroke", "rgb", red, green, blue, 0);
     }
 
-    /** Set the current fill and stroke color to the supplied RGB values. */
+    /** Set the current fill and stroke color to the supplied RGB values.
+	@deprecated Use setcolor("both", "rgb", red, green, blue, 0) instead.
+    */
     public final void setrgbcolor(float red, float green, float blue) throws
     OutOfMemoryError, IOException, IllegalArgumentException,
     IndexOutOfBoundsException, ClassCastException, ArithmeticException,
     RuntimeException, InternalError, UnknownError
     {
-	PDF_setrgbcolor(p, red, green, blue);
+	PDF_setcolor(p, "both", "rgb", red, green, blue, 0);
     }
 
     /** Place an image at the lower left corner (x, y), and scale it. */
@@ -886,6 +910,7 @@ public final class pdflib {
         RuntimeException, InternalError, UnknownError
     {
 	PDF_delete(p);
+	p = (long) 0;
     }
 
     private final static void classFinalize() {
@@ -1131,46 +1156,6 @@ public final class pdflib {
         IndexOutOfBoundsException, ClassCastException, ArithmeticException,
         RuntimeException, InternalError, UnknownError;
 
-    private final static native void PDF_setgray_fill(long jarg0, float jarg1) throws
-        OutOfMemoryError, IOException, IllegalArgumentException,
-        IndexOutOfBoundsException, ClassCastException, ArithmeticException,
-        RuntimeException, InternalError, UnknownError;
-
-    private final static native void PDF_setgray_stroke(long jarg0, float jarg1) throws
-        OutOfMemoryError, IOException, IllegalArgumentException,
-        IndexOutOfBoundsException, ClassCastException, ArithmeticException,
-        RuntimeException, InternalError, UnknownError;
-
-    private final static native void PDF_setgray(long jarg0, float jarg1) throws
-        OutOfMemoryError, IOException, IllegalArgumentException,
-        IndexOutOfBoundsException, ClassCastException, ArithmeticException,
-        RuntimeException, InternalError, UnknownError;
-
-    private final static native void PDF_setrgbcolor_fill(long jarg0, float jarg1, float jarg2, float jarg3) throws
-        OutOfMemoryError, IOException, IllegalArgumentException,
-        IndexOutOfBoundsException, ClassCastException, ArithmeticException,
-        RuntimeException, InternalError, UnknownError;
-
-    private final static native void PDF_setrgbcolor_stroke(long jarg0, float jarg1, float jarg2, float jarg3) throws
-        OutOfMemoryError, IOException, IllegalArgumentException,
-        IndexOutOfBoundsException, ClassCastException, ArithmeticException,
-        RuntimeException, InternalError, UnknownError;
-
-    private final static native void PDF_setrgbcolor(long jarg0, float jarg1, float jarg2, float jarg3) throws
-        OutOfMemoryError, IOException, IllegalArgumentException,
-        IndexOutOfBoundsException, ClassCastException, ArithmeticException,
-        RuntimeException, InternalError, UnknownError;
-
-    private final static native int PDF_get_image_width(long jarg0, int jarg1) throws
-        OutOfMemoryError, IOException, IllegalArgumentException,
-        IndexOutOfBoundsException, ClassCastException, ArithmeticException,
-        RuntimeException, InternalError, UnknownError;
-
-    private final static native int PDF_get_image_height(long jarg0, int jarg1) throws
-        OutOfMemoryError, IOException, IllegalArgumentException,
-        IndexOutOfBoundsException, ClassCastException, ArithmeticException,
-        RuntimeException, InternalError, UnknownError;
-
     private final static native void PDF_place_image(long jarg0, int jarg1, float jarg2, float jarg3, float jarg4) throws
         OutOfMemoryError, IOException, IllegalArgumentException,
         IndexOutOfBoundsException, ClassCastException, ArithmeticException,
