@@ -1,7 +1,7 @@
 /*---------------------------------------------------------------------------*
  |              PDFlib - A library for generating PDF on the fly             |
  +---------------------------------------------------------------------------+
- | Copyright (c) 1997-2001 PDFlib GmbH and Thomas Merz. All rights reserved. |
+ | Copyright (c) 1997-2002 PDFlib GmbH and Thomas Merz. All rights reserved. |
  +---------------------------------------------------------------------------+
  |    This software is NOT in the public domain.  It can be used under two   |
  |    substantially different licensing terms:                               |
@@ -27,7 +27,7 @@
  |    however, will receive warranty and support statements in writing.      |
  *---------------------------------------------------------------------------*/
 
-/* $Id: pdflib_java.c,v 1.15.10.2 2001/05/11 12:38:08 york Exp $
+/* $Id: pdflib_java.c,v 1.15.10.19 2002/06/19 10:20:02 tm Exp $
  *
  * JNI wrapper code for the PDFlib Java binding
  *
@@ -134,6 +134,23 @@ NULL,
 #define try	ex = (pdf_wrapper_data *) (PDF_get_opaque(p));	\
 		if (setjmp(ex->env) == 0)
 
+#define PDF_ISNULL(j)	(j == (PDF_ENV) 0)
+
+
+#define PDF_JAVA_SANITY_CHECK_VOID(j) 				\
+    if (PDF_ISNULL(j)) {					\
+	jthrow(jenv, PDF_RuntimeError,				\
+	    "Must not call any PDFlib method after delete()");	\
+	return;							\
+    }
+
+#define PDF_JAVA_SANITY_CHECK(j) 				\
+    if (PDF_ISNULL(j)) {					\
+	jthrow(jenv, PDF_RuntimeError,				\
+	    "Must not call any PDFlib method after delete()");	\
+	return _jresult;					\
+    }
+
 #ifndef PDFLIB_EBCDIC
 
 #define jthrow(jenv, type, msg) (*jenv)->ThrowNew(jenv, 		\
@@ -223,9 +240,6 @@ GetStringPDFChars(PDF *p, JNIEnv *jenv, jstring string)
 
     (*jenv)->ReleaseStringChars(jenv, string, unistring);
 
-#ifdef PDFLIB_EBCDIC
-    __atoe(result);  /* convert to EBCDIC */
-#endif
 
     return result;
 }
@@ -387,14 +401,13 @@ GetStringUnicodePDFChars(PDF *p, JNIEnv *jenv, jstring string, int *lenP)
 	     */
 	    result =
 		(unsigned char *) JNU_GetString_NativeChars(jenv, ex, string);
+	    if (lenP)
+		*lenP = strlen((const char *) result);
 	}
     }
 
     (*jenv)->ReleaseStringChars(jenv, string, unistring);
 
-#ifdef PDFLIB_EBCDIC
-    __atoe(result);  /* convert to EBCDIC */
-#endif
 
     return (char *) result;
 }
@@ -423,13 +436,16 @@ JNIEXPORT jlong JNICALL
 Java_com_pdflib_pdflib_PDF_1new(JNIEnv *jenv, jclass jcls)
 {
     jlong _jresult;
-    PDF *p;
+    PDF * PDF_VOLATILE p = (PDF *) 0;
     EXC_STUFF;
     char jdkname[64];
     jclass stringClass;
 
 #ifdef AS400
-    int aux = 0;
+/* This must NOT be 0 since we use 0 as guard value after PDF_delete()! */
+
+#define PDF_JAVA_AS400_FIRSTINDEX	1
+    int aux = PDF_JAVA_AS400_FIRSTINDEX;
     JavaI2L(aux, _jresult);
 #else
     _jresult = int2ll(0);
@@ -443,9 +459,13 @@ Java_com_pdflib_pdflib_PDF_1new(JNIEnv *jenv, jclass jcls)
 
     ex->nativeunicode = 0;
 
-    p = (PDF *)PDF_new2(pdf_errorhandler, NULL, NULL, NULL, (void *) ex);
+    /* Can't use try() yet since the PDFlib core isn't initialized yet,
+     * and PDF_get_opaque() wouldn't work. Do a manual setjmp() instead.
+     */
+    if (setjmp(ex->env) == 0) {
+
+	p = (PDF *)PDF_new2(pdf_errorhandler, NULL, NULL, NULL, (void *) ex);
 
-    if (p) {
 	ex->jdkversion = (*jenv)->GetVersion(jenv);
 
 	if (((*jenv)->ExceptionOccurred(jenv)) != NULL_INT) {
@@ -470,7 +490,12 @@ Java_com_pdflib_pdflib_PDF_1new(JNIEnv *jenv, jclass jcls)
 	    (*jenv)->ExceptionDescribe(jenv);
 	    jthrow(jenv, PDF_SystemError,
 		"Couldn't initialize PDFlib (FindClass)");
-	    return int2ll(0);
+#ifdef AS400
+	    JavaI2L(0, _jresult);
+#else
+	    _jresult = int2ll(0);
+#endif
+	    return _jresult;
 	}
 
 	ex->MID_String_getBytes =
@@ -483,9 +508,7 @@ Java_com_pdflib_pdflib_PDF_1new(JNIEnv *jenv, jclass jcls)
 	    return _jresult;
 	}
 
-    } else {
-	jthrow(jenv, PDF_MemoryError, "Couldn't initialize PDFlib (PDF_new2)");
-    }
+    } catch;
 
 
 #ifdef AS400
@@ -503,6 +526,13 @@ Java_com_pdflib_pdflib_PDF_1delete(JNIEnv *jenv, jclass jcls, PDF_ENV jarg0)
     PDF *p;
     EXC_STUFF;
 
+    /* Different sanity check here since PDF_delete() may be called multiply,
+     * e.g., once from the finalizer and once explicitly
+     */
+
+    if (PDF_ISNULL(jarg0))
+	return;	
+
     p = PDF_ENV2PTR(jarg0);
 
 
@@ -514,12 +544,14 @@ JNIEXPORT jint JNICALL
 Java_com_pdflib_pdflib_PDF_1open_1file(JNIEnv *jenv, jclass jcls,
     PDF_ENV jarg0, jstring jarg1)
 {
-    jint _jresult;
+    jint _jresult = -1;
     int PDF_VOLATILE _result = -1;
     PDF *p;
     EXC_STUFF;
     char * PDF_VOLATILE _arg1;
 
+    PDF_JAVA_SANITY_CHECK(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
 
     try {
@@ -538,6 +570,8 @@ Java_com_pdflib_pdflib_PDF_1close(JNIEnv *jenv, jclass jcls, PDF_ENV jarg0)
     PDF *p;
     EXC_STUFF;
 
+    PDF_JAVA_SANITY_CHECK_VOID(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
 
     try {     PDF_close(p);
@@ -545,31 +579,34 @@ Java_com_pdflib_pdflib_PDF_1close(JNIEnv *jenv, jclass jcls, PDF_ENV jarg0)
 }
 
 JNIEXPORT jbyteArray JNICALL
-Java_com_pdflib_pdflib_PDF_1get_1buffer(JNIEnv *jenv, jclass jcls, PDF_ENV jarg0)
+Java_com_pdflib_pdflib_PDF_1get_1buffer(JNIEnv *jenv, jclass jcls,
+    PDF_ENV jarg0)
 {
-    jbyteArray PDF_VOLATILE jbuffer;
+    jbyteArray PDF_VOLATILE _jresult = 0;
     const unsigned char *buffer;
     PDF *p;
     EXC_STUFF;
     long size;
 
-    p = *(PDF**) &jarg0;
+    PDF_JAVA_SANITY_CHECK(jarg0);
+
+    p = PDF_ENV2PTR(jarg0);
 
     try {
 	buffer = (const unsigned char *) PDF_get_buffer(p, &size);
-	jbuffer = (*jenv)->NewByteArray(jenv, (jsize) size);
+	_jresult = (*jenv)->NewByteArray(jenv, (jsize) size);
 
-	if (jbuffer == (jbyteArray) NULL) {
+	if (_jresult == (jbyteArray) NULL) {
 	    pdf_errorhandler(p, PDF_MemoryError,
 		"Couldn't allocate PDF output buffer");
 	} else {
 
-	    (*jenv)->SetByteArrayRegion(jenv, jbuffer,
+	    (*jenv)->SetByteArrayRegion(jenv, _jresult,
 			0, (jsize) size, (jbyte *) buffer);
 	}
     } catch;
 
-    return jbuffer;
+    return _jresult;
 }
 
 JNIEXPORT void JNICALL
@@ -581,6 +618,8 @@ Java_com_pdflib_pdflib_PDF_1begin_1page(JNIEnv *jenv, jclass jcls,
     float _arg1;
     float _arg2;
 
+    PDF_JAVA_SANITY_CHECK_VOID(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
     _arg1 = flt2nat(jarg1);
     _arg2 = flt2nat(jarg2);
@@ -596,6 +635,8 @@ Java_com_pdflib_pdflib_PDF_1end_1page(JNIEnv *jenv, jclass jcls, PDF_ENV jarg0)
     PDF *p;
     EXC_STUFF;
 
+    PDF_JAVA_SANITY_CHECK_VOID(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
 
     try {     PDF_end_page(p);
@@ -611,6 +652,8 @@ Java_com_pdflib_pdflib_PDF_1set_1parameter(JNIEnv *jenv, jclass jcls,
     char * PDF_VOLATILE _arg1;
     char * PDF_VOLATILE _arg2;
 
+    PDF_JAVA_SANITY_CHECK_VOID(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
 
     try {
@@ -644,6 +687,8 @@ Java_com_pdflib_pdflib_PDF_1get_1parameter(JNIEnv *jenv, jclass jcls,
     PDF *p;
     EXC_STUFF;
 
+    PDF_JAVA_SANITY_CHECK(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
     key = (char *) GetStringPDFChars(p, jenv, jkey);
     mod = flt2nat(jmod);
@@ -653,6 +698,8 @@ Java_com_pdflib_pdflib_PDF_1get_1parameter(JNIEnv *jenv, jclass jcls,
 
     if(_result)
         _jresult = (jstring)(*jenv)->NewStringUTF(jenv, _result);
+
+    ReleaseStringPDFChars(jenv, jarg1, key);
     return _jresult;
 }
 
@@ -665,6 +712,8 @@ Java_com_pdflib_pdflib_PDF_1set_1value(JNIEnv *jenv, jclass jcls,
     char * _arg1;
     float _arg2;
 
+    PDF_JAVA_SANITY_CHECK_VOID(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
     _arg1 = (char *) GetStringPDFChars(p, jenv, jarg1);
     _arg2 = flt2nat(jarg2);
@@ -686,6 +735,8 @@ Java_com_pdflib_pdflib_PDF_1get_1value(JNIEnv *jenv, jclass jcls,
     char * _arg1;
     float _arg2;
 
+    PDF_JAVA_SANITY_CHECK(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
     _arg1 = (char *) GetStringPDFChars(p, jenv, jarg1);
     _arg2 = flt2nat(jarg2);
@@ -693,12 +744,12 @@ Java_com_pdflib_pdflib_PDF_1get_1value(JNIEnv *jenv, jclass jcls,
     try {     _result = (float )PDF_get_value(p, _arg1, _arg2);
     } catch;
 
-    ReleaseStringPDFChars(jenv, jarg1, _arg1);
 #ifndef OS390
     _jresult = (jfloat) _result;
 #else
     _jresult = tofloat(_result);
 #endif
+    ReleaseStringPDFChars(jenv, jarg1, _arg1);
     return _jresult;
 }
 
@@ -706,7 +757,7 @@ JNIEXPORT jint JNICALL
 Java_com_pdflib_pdflib_PDF_1findfont(JNIEnv *jenv, jclass jcls,
     PDF_ENV jarg0, jstring jarg1, jstring jarg2, jint jarg3)
 {
-    jint _jresult;
+    jint _jresult = -1;
     int PDF_VOLATILE _result = -1;
     PDF *p;
     EXC_STUFF;
@@ -714,6 +765,8 @@ Java_com_pdflib_pdflib_PDF_1findfont(JNIEnv *jenv, jclass jcls,
     char * PDF_VOLATILE _arg2;
     int _arg3;
 
+    PDF_JAVA_SANITY_CHECK(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
 
     try {
@@ -740,6 +793,8 @@ Java_com_pdflib_pdflib_PDF_1setfont(JNIEnv *jenv, jclass jcls,
     int _arg1;
     float _arg2;
 
+    PDF_JAVA_SANITY_CHECK_VOID(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
     _arg1 = (int )jarg1;
     _arg2 = flt2nat(jarg2);
@@ -757,6 +812,8 @@ Java_com_pdflib_pdflib_PDF_1show(JNIEnv *jenv, jclass jcls,
     char * PDF_VOLATILE _arg1;
     int len;
 
+    PDF_JAVA_SANITY_CHECK_VOID(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
 
     try {
@@ -779,6 +836,8 @@ Java_com_pdflib_pdflib_PDF_1show_1xy(JNIEnv *jenv, jclass jcls,
     float _arg3;
     int len;
 
+    PDF_JAVA_SANITY_CHECK_VOID(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
 
     try {
@@ -801,6 +860,8 @@ Java_com_pdflib_pdflib_PDF_1continue_1text(JNIEnv *jenv, jclass jcls,
     char * PDF_VOLATILE _arg1;
     int len;
 
+    PDF_JAVA_SANITY_CHECK_VOID(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
 
     try {
@@ -817,6 +878,7 @@ Java_com_pdflib_pdflib_PDF_1show_1boxed(JNIEnv *jenv, jclass jcls,
     PDF_ENV jarg0, jstring jarg1, jfloat jarg2, jfloat jarg3, jfloat jarg4,
     jfloat jarg5, jstring jarg6, jstring jarg7)
 {
+    jint _jresult = 0;
     int PDF_VOLATILE _result;
     PDF *p;
     EXC_STUFF;
@@ -829,6 +891,8 @@ Java_com_pdflib_pdflib_PDF_1show_1boxed(JNIEnv *jenv, jclass jcls,
     char * PDF_VOLATILE _arg7;
     int len;
 
+    PDF_JAVA_SANITY_CHECK(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
 
     try {
@@ -843,11 +907,12 @@ Java_com_pdflib_pdflib_PDF_1show_1boxed(JNIEnv *jenv, jclass jcls,
 	_result = PDF_show_boxed(p,_arg1,_arg2,_arg3,_arg4,_arg5,_arg6,_arg7);
     } catch;
 
+    _jresult = (jint) _result;
     ReleaseStringPDFChars(jenv, jarg1, _arg1);
     ReleaseStringPDFChars(jenv, jarg6, _arg6);
     ReleaseStringPDFChars(jenv, jarg7, _arg7);
 
-    return (jint) _result;
+    return (jint) _jresult;
 }
 
 JNIEXPORT void JNICALL
@@ -859,6 +924,8 @@ Java_com_pdflib_pdflib_PDF_1set_1text_1pos(JNIEnv *jenv, jclass jcls,
     float _arg1;
     float _arg2;
 
+    PDF_JAVA_SANITY_CHECK_VOID(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
     _arg1 = flt2nat(jarg1);
     _arg2 = flt2nat(jarg2);
@@ -880,6 +947,8 @@ Java_com_pdflib_pdflib_PDF_1stringwidth(JNIEnv *jenv, jclass jcls,
     float _arg3;
     int len;
 
+    PDF_JAVA_SANITY_CHECK(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
 
     try {
@@ -905,6 +974,8 @@ Java_com_pdflib_pdflib_PDF_1setdash(JNIEnv *jenv, jclass jcls,
     float _arg1;
     float _arg2;
 
+    PDF_JAVA_SANITY_CHECK_VOID(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
     _arg1 = flt2nat(jarg1);
     _arg2 = flt2nat(jarg2);
@@ -924,7 +995,9 @@ Java_com_pdflib_pdflib_PDF_1setpolydash(JNIEnv *jenv, jclass jcls,
     int i;
     jsize PDF_VOLATILE length;
 
-    p = *(PDF **) &jarg0;
+    PDF_JAVA_SANITY_CHECK_VOID(jarg0);
+
+    p = PDF_ENV2PTR(jarg0);
     length = (*jenv)->GetArrayLength(jenv, jarg1);
 
     try {
@@ -949,6 +1022,8 @@ Java_com_pdflib_pdflib_PDF_1setflat(JNIEnv *jenv, jclass jcls,
     EXC_STUFF;
     float _arg1;
 
+    PDF_JAVA_SANITY_CHECK_VOID(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
     _arg1 = flt2nat(jarg1);
 
@@ -964,6 +1039,8 @@ Java_com_pdflib_pdflib_PDF_1setlinejoin(JNIEnv *jenv, jclass jcls,
     EXC_STUFF;
     int _arg1;
 
+    PDF_JAVA_SANITY_CHECK_VOID(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
     _arg1 = (int )jarg1;
 
@@ -979,6 +1056,8 @@ Java_com_pdflib_pdflib_PDF_1setlinecap(JNIEnv *jenv, jclass jcls,
     EXC_STUFF;
     int _arg1;
 
+    PDF_JAVA_SANITY_CHECK_VOID(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
     _arg1 = (int )jarg1;
 
@@ -994,6 +1073,8 @@ Java_com_pdflib_pdflib_PDF_1setmiterlimit(JNIEnv *jenv, jclass jcls,
     EXC_STUFF;
     float _arg1;
 
+    PDF_JAVA_SANITY_CHECK_VOID(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
     _arg1 = flt2nat(jarg1);
 
@@ -1009,6 +1090,8 @@ Java_com_pdflib_pdflib_PDF_1setlinewidth(JNIEnv *jenv, jclass jcls,
     EXC_STUFF;
     float _arg1;
 
+    PDF_JAVA_SANITY_CHECK_VOID(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
      _arg1 = flt2nat(jarg1);
 
@@ -1023,6 +1106,8 @@ Java_com_pdflib_pdflib_PDF_1save(JNIEnv *jenv, jclass jcls,
     PDF *p;
     EXC_STUFF;
 
+    PDF_JAVA_SANITY_CHECK_VOID(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
 
     try {     PDF_save(p);
@@ -1036,6 +1121,8 @@ Java_com_pdflib_pdflib_PDF_1restore(JNIEnv *jenv, jclass jcls,
     PDF *p;
     EXC_STUFF;
 
+    PDF_JAVA_SANITY_CHECK_VOID(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
 
     try {     PDF_restore(p);
@@ -1051,6 +1138,8 @@ Java_com_pdflib_pdflib_PDF_1translate(JNIEnv *jenv, jclass jcls,
     float _arg1;
     float _arg2;
 
+    PDF_JAVA_SANITY_CHECK_VOID(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
     _arg1 = flt2nat(jarg1);
     _arg2 = flt2nat(jarg2);
@@ -1068,6 +1157,8 @@ Java_com_pdflib_pdflib_PDF_1scale(JNIEnv *jenv, jclass jcls,
     float _arg1;
     float _arg2;
 
+    PDF_JAVA_SANITY_CHECK_VOID(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
     _arg1 = flt2nat(jarg1);
     _arg2 = flt2nat(jarg2);
@@ -1084,6 +1175,8 @@ Java_com_pdflib_pdflib_PDF_1rotate(JNIEnv *jenv, jclass jcls,
     EXC_STUFF;
     float _arg1;
 
+    PDF_JAVA_SANITY_CHECK_VOID(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
     _arg1 = flt2nat(jarg1);
 
@@ -1100,6 +1193,8 @@ Java_com_pdflib_pdflib_PDF_1skew(JNIEnv *jenv, jclass jcls,
     float _arg1;
     float _arg2;
 
+    PDF_JAVA_SANITY_CHECK_VOID(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
     _arg1 = flt2nat(jarg1);
     _arg2 = flt2nat(jarg2);
@@ -1115,6 +1210,8 @@ Java_com_pdflib_pdflib_PDF_1concat(JNIEnv *jenv, jclass jcls,
     PDF *p;
     EXC_STUFF;
 
+    PDF_JAVA_SANITY_CHECK_VOID(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
 
     try {     PDF_concat(p, flt2nat(a), flt2nat(b), flt2nat(c), flt2nat(d),
@@ -1131,6 +1228,8 @@ Java_com_pdflib_pdflib_PDF_1moveto(JNIEnv *jenv, jclass jcls,
     float _arg1;
     float _arg2;
 
+    PDF_JAVA_SANITY_CHECK_VOID(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
     _arg1 = flt2nat(jarg1);
     _arg2 = flt2nat(jarg2);
@@ -1148,6 +1247,8 @@ Java_com_pdflib_pdflib_PDF_1lineto(JNIEnv *jenv, jclass jcls,
     float _arg1;
     float _arg2;
 
+    PDF_JAVA_SANITY_CHECK_VOID(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
     _arg1 = flt2nat(jarg1);
     _arg2 = flt2nat(jarg2);
@@ -1170,6 +1271,8 @@ Java_com_pdflib_pdflib_PDF_1curveto(JNIEnv *jenv, jclass jcls,
     float _arg5;
     float _arg6;
 
+    PDF_JAVA_SANITY_CHECK_VOID(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
     _arg1 = flt2nat(jarg1);
     _arg2 = flt2nat(jarg2);
@@ -1192,6 +1295,8 @@ Java_com_pdflib_pdflib_PDF_1circle(JNIEnv *jenv, jclass jcls,
     float _arg2;
     float _arg3;
 
+    PDF_JAVA_SANITY_CHECK_VOID(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
     _arg1 = flt2nat(jarg1);
     _arg2 = flt2nat(jarg2);
@@ -1214,6 +1319,8 @@ Java_com_pdflib_pdflib_PDF_1arc(JNIEnv *jenv, jclass jcls,
     float _arg4;
     float _arg5;
 
+    PDF_JAVA_SANITY_CHECK_VOID(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
     _arg1 = flt2nat(jarg1);
     _arg2 = flt2nat(jarg2);
@@ -1236,6 +1343,8 @@ Java_com_pdflib_pdflib_PDF_1rect(JNIEnv *jenv, jclass jcls,
     float _arg3;
     float _arg4;
 
+    PDF_JAVA_SANITY_CHECK_VOID(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
     _arg1 = flt2nat(jarg1);
     _arg2 = flt2nat(jarg2);
@@ -1252,6 +1361,8 @@ Java_com_pdflib_pdflib_PDF_1closepath(JNIEnv *jenv, jclass jcls, PDF_ENV jarg0)
     PDF *p;
     EXC_STUFF;
 
+    PDF_JAVA_SANITY_CHECK_VOID(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
 
     try {     PDF_closepath(p);
@@ -1264,6 +1375,8 @@ Java_com_pdflib_pdflib_PDF_1stroke(JNIEnv *jenv, jclass jcls, PDF_ENV jarg0)
     PDF *p;
     EXC_STUFF;
 
+    PDF_JAVA_SANITY_CHECK_VOID(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
 
     try {     PDF_stroke(p);
@@ -1277,6 +1390,8 @@ Java_com_pdflib_pdflib_PDF_1closepath_1stroke(JNIEnv *jenv, jclass jcls,
     PDF *p;
     EXC_STUFF;
 
+    PDF_JAVA_SANITY_CHECK_VOID(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
 
     try {     PDF_closepath_stroke(p);
@@ -1290,6 +1405,8 @@ Java_com_pdflib_pdflib_PDF_1fill(JNIEnv *jenv, jclass jcls,
     PDF *p;
     EXC_STUFF;
 
+    PDF_JAVA_SANITY_CHECK_VOID(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
 
     try {     PDF_fill(p);
@@ -1303,6 +1420,8 @@ Java_com_pdflib_pdflib_PDF_1fill_1stroke(JNIEnv *jenv, jclass jcls,
     PDF *p;
     EXC_STUFF;
 
+    PDF_JAVA_SANITY_CHECK_VOID(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
 
     try {     PDF_fill_stroke(p);
@@ -1316,6 +1435,8 @@ Java_com_pdflib_pdflib_PDF_1closepath_1fill_1stroke(JNIEnv *jenv, jclass jcls,
     PDF *p;
     EXC_STUFF;
 
+    PDF_JAVA_SANITY_CHECK_VOID(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
 
     try {     PDF_closepath_fill_stroke(p);
@@ -1328,6 +1449,8 @@ Java_com_pdflib_pdflib_PDF_1endpath(JNIEnv *jenv, jclass jcls, PDF_ENV jarg0)
     PDF *p;
     EXC_STUFF;
 
+    PDF_JAVA_SANITY_CHECK_VOID(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
 
     try {     PDF_endpath(p);
@@ -1340,126 +1463,14 @@ Java_com_pdflib_pdflib_PDF_1clip(JNIEnv *jenv, jclass jcls, PDF_ENV jarg0)
     PDF *p;
     EXC_STUFF;
 
+    PDF_JAVA_SANITY_CHECK_VOID(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
 
     try {     PDF_clip(p);
     } catch;
 }
 
-JNIEXPORT void JNICALL
-Java_com_pdflib_pdflib_PDF_1setgray_1fill(JNIEnv *jenv, jclass jcls,
-    PDF_ENV jarg0, jfloat jarg1)
-{
-    PDF *p;
-    EXC_STUFF;
-    float _arg1;
-    float zero = flt2nat(tofloat(0));
-
-    p = PDF_ENV2PTR(jarg0);
-    _arg1 = flt2nat(jarg1);
-
-    try {
-	PDF_setcolor(p, "fill", "gray", _arg1, zero, zero, zero);
-    } catch;
-}
-
-JNIEXPORT void JNICALL
-Java_com_pdflib_pdflib_PDF_1setgray_1stroke(JNIEnv *jenv, jclass jcls,
-    PDF_ENV jarg0, jfloat jarg1)
-{
-    PDF *p;
-    EXC_STUFF;
-    float _arg1;
-    float zero = flt2nat(tofloat(0));
-
-    p = PDF_ENV2PTR(jarg0);
-    _arg1 = flt2nat(jarg1);
-
-    try {
-	PDF_setcolor(p, "stroke", "gray", _arg1, zero, zero, zero);
-    } catch;
-}
-
-JNIEXPORT void JNICALL
-Java_com_pdflib_pdflib_PDF_1setgray(JNIEnv *jenv, jclass jcls,
-    PDF_ENV jarg0, jfloat jarg1)
-{
-    PDF *p;
-    EXC_STUFF;
-    float _arg1;
-    float zero = flt2nat(tofloat(0));
-
-    p = PDF_ENV2PTR(jarg0);
-    _arg1 = flt2nat(jarg1);
-
-    try {
-	PDF_setcolor(p, "both", "gray", _arg1, zero, zero, zero);
-    } catch;
-}
-
-JNIEXPORT void JNICALL
-Java_com_pdflib_pdflib_PDF_1setrgbcolor_1fill(JNIEnv *jenv, jclass jcls,
-    PDF_ENV jarg0, jfloat jarg1, jfloat jarg2, jfloat jarg3)
-{
-    PDF *p;
-    EXC_STUFF;
-    float _arg1;
-    float _arg2;
-    float _arg3;
-    float zero = flt2nat(tofloat(0));
-
-    p = PDF_ENV2PTR(jarg0);
-    _arg1 = flt2nat(jarg1);
-    _arg2 = flt2nat(jarg2);
-    _arg3 = flt2nat(jarg3);
-
-    try {     PDF_setrgbcolor_fill(p,_arg1,_arg2,_arg3);
-	PDF_setcolor(p, "fill", "rgb", _arg1, _arg2, _arg3, zero);
-    } catch;
-}
-
-JNIEXPORT void JNICALL
-Java_com_pdflib_pdflib_PDF_1setrgbcolor_1stroke(JNIEnv *jenv, jclass jcls,
-    PDF_ENV jarg0, jfloat jarg1, jfloat jarg2, jfloat jarg3)
-{
-    PDF *p;
-    EXC_STUFF;
-    float _arg1;
-    float _arg2;
-    float _arg3;
-    float zero = flt2nat(tofloat(0));
-
-    p = PDF_ENV2PTR(jarg0);
-    _arg1 = flt2nat(jarg1);
-    _arg2 = flt2nat(jarg2);
-    _arg3 = flt2nat(jarg3);
-
-    try {
-	PDF_setcolor(p, "stroke", "rgb", _arg1, _arg2, _arg3, zero);
-    } catch;
-}
-
-JNIEXPORT void JNICALL
-Java_com_pdflib_pdflib_PDF_1setrgbcolor(JNIEnv *jenv, jclass jcls,
-    PDF_ENV jarg0, jfloat jarg1, jfloat jarg2, jfloat jarg3)
-{
-    PDF *p;
-    EXC_STUFF;
-    float _arg1;
-    float _arg2;
-    float _arg3;
-    float zero = flt2nat(tofloat(0));
-
-    p = PDF_ENV2PTR(jarg0);
-    _arg1 = flt2nat(jarg1);
-    _arg2 = flt2nat(jarg2);
-    _arg3 = flt2nat(jarg3);
-
-    try {
-	PDF_setcolor(p, "both", "rgb", _arg1, _arg2, _arg3, zero);
-    } catch;
-}
-
 JNIEXPORT void JNICALL
 Java_com_pdflib_pdflib_PDF_1place_1image(JNIEnv *jenv, jclass jcls,
     PDF_ENV jarg0, jint jarg1, jfloat jarg2, jfloat jarg3, jfloat jarg4)
@@ -1471,6 +1482,8 @@ Java_com_pdflib_pdflib_PDF_1place_1image(JNIEnv *jenv, jclass jcls,
     float _arg3;
     float _arg4;
 
+    PDF_JAVA_SANITY_CHECK_VOID(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
     _arg1 = (int )jarg1;
     _arg2 = flt2nat(jarg2);
@@ -1503,6 +1516,8 @@ Java_com_pdflib_pdflib_PDF_1open_1image(JNIEnv *jenv, jclass jcls,
     int _arg8;
     char * PDF_VOLATILE _arg9;
 
+    PDF_JAVA_SANITY_CHECK(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
 
     try {
@@ -1548,6 +1563,8 @@ Java_com_pdflib_pdflib_PDF_1close_1image(JNIEnv *jenv, jclass jcls,
     EXC_STUFF;
     int _arg1;
 
+    PDF_JAVA_SANITY_CHECK_VOID(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
     _arg1 = (int )jarg1;
 
@@ -1568,6 +1585,8 @@ Java_com_pdflib_pdflib_PDF_1open_1image_1file(JNIEnv *jenv, jclass jcls,
     char * PDF_VOLATILE _arg3;
     int _arg4;
 
+    PDF_JAVA_SANITY_CHECK(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
 
     try {
@@ -1588,8 +1607,8 @@ Java_com_pdflib_pdflib_PDF_1open_1image_1file(JNIEnv *jenv, jclass jcls,
 
 JNIEXPORT jint JNICALL
 Java_com_pdflib_pdflib_PDF_1open_1CCITT(JNIEnv *jenv, jclass jcls,
-    PDF_ENV jarg0, jstring jarg1, jint jarg2, jint jarg3, jint jarg4, jint jarg5,
-    jint jarg6)
+    PDF_ENV jarg0, jstring jarg1, jint jarg2, jint jarg3, jint jarg4,
+    jint jarg5, jint jarg6)
 {
     jint _jresult = 0;
     int PDF_VOLATILE _result;
@@ -1602,6 +1621,8 @@ Java_com_pdflib_pdflib_PDF_1open_1CCITT(JNIEnv *jenv, jclass jcls,
     int _arg5;
     int _arg6;
 
+    PDF_JAVA_SANITY_CHECK(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
 
     try {
@@ -1632,6 +1653,8 @@ Java_com_pdflib_pdflib_PDF_1add_1bookmark(JNIEnv *jenv, jclass jcls,
     int _arg2;
     int _arg3;
 
+    PDF_JAVA_SANITY_CHECK(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
 
     try {
@@ -1656,6 +1679,8 @@ Java_com_pdflib_pdflib_PDF_1set_1info(JNIEnv *jenv, jclass jcls,
     char * PDF_VOLATILE _arg1;
     char * PDF_VOLATILE _arg2;
 
+    PDF_JAVA_SANITY_CHECK_VOID(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
 
     try {
@@ -1686,6 +1711,8 @@ Java_com_pdflib_pdflib_PDF_1attach_1file(JNIEnv *jenv, jclass jcls,
     char * PDF_VOLATILE _arg8;
     char * PDF_VOLATILE _arg9;
 
+    PDF_JAVA_SANITY_CHECK_VOID(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
 
     try {
@@ -1726,6 +1753,8 @@ Java_com_pdflib_pdflib_PDF_1add_1note(JNIEnv *jenv, jclass jcls,
     char * PDF_VOLATILE _arg7;
     int _arg8;
 
+    PDF_JAVA_SANITY_CHECK_VOID(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
 
     try {
@@ -1761,6 +1790,8 @@ Java_com_pdflib_pdflib_PDF_1add_1pdflink(JNIEnv *jenv, jclass jcls,
     int _arg6;
     char * PDF_VOLATILE _arg7;
 
+    PDF_JAVA_SANITY_CHECK_VOID(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
 
     try {
@@ -1792,6 +1823,8 @@ Java_com_pdflib_pdflib_PDF_1add_1launchlink(JNIEnv *jenv, jclass jcls,
     float _arg4;
     char * PDF_VOLATILE _arg5;
 
+    PDF_JAVA_SANITY_CHECK_VOID(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
 
     try {
@@ -1821,6 +1854,8 @@ Java_com_pdflib_pdflib_PDF_1add_1locallink(JNIEnv *jenv, jclass jcls,
     int _arg5;
     char * PDF_VOLATILE _arg6;
 
+    PDF_JAVA_SANITY_CHECK_VOID(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
 
     try {
@@ -1850,6 +1885,8 @@ Java_com_pdflib_pdflib_PDF_1add_1weblink(JNIEnv *jenv, jclass jcls,
     float _arg4;
     char *PDF_VOLATILE _arg5;
 
+    PDF_JAVA_SANITY_CHECK_VOID(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
 
     try {
@@ -1874,6 +1911,8 @@ Java_com_pdflib_pdflib_PDF_1set_1border_1style(JNIEnv *jenv, jclass jcls,
     char * PDF_VOLATILE _arg1;
     float _arg2;
 
+    PDF_JAVA_SANITY_CHECK_VOID(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
 
     try {
@@ -1896,6 +1935,8 @@ Java_com_pdflib_pdflib_PDF_1set_1border_1color(JNIEnv *jenv, jclass jcls,
     float _arg2;
     float _arg3;
 
+    PDF_JAVA_SANITY_CHECK_VOID(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
     _arg1 = flt2nat(jarg1);
     _arg2 = flt2nat(jarg2);
@@ -1914,6 +1955,8 @@ Java_com_pdflib_pdflib_PDF_1set_1border_1dash(JNIEnv *jenv, jclass jcls,
     float _arg1;
     float _arg2;
 
+    PDF_JAVA_SANITY_CHECK_VOID(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
     _arg1 = flt2nat(jarg1);
     _arg2 = flt2nat(jarg2);
@@ -1934,6 +1977,8 @@ Java_com_pdflib_pdflib_PDF_1open_1pdi(JNIEnv *jenv, jclass jcls,
     char * PDF_VOLATILE _arg2 = NULL;
     int _arg3;
 
+    PDF_JAVA_SANITY_CHECK(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
     _arg3 = (int )jarg3;
 
@@ -1960,6 +2005,8 @@ Java_com_pdflib_pdflib_PDF_1close_1pdi(JNIEnv *jenv, jclass jcls,
     EXC_STUFF;
     int _arg1;
 
+    PDF_JAVA_SANITY_CHECK_VOID(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
     _arg1 = (int )jarg1;
 
@@ -1980,6 +2027,8 @@ Java_com_pdflib_pdflib_PDF_1place_1pdi_1page(JNIEnv *jenv, jclass jcls,
     float _arg4;
     float _arg5;
 
+    PDF_JAVA_SANITY_CHECK_VOID(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
     _arg1 = (int )jarg1;
     _arg2 = flt2nat(jarg2);
@@ -2003,6 +2052,8 @@ Java_com_pdflib_pdflib_PDF_1open_1pdi_1page(JNIEnv *jenv, jclass jcls,
     int _arg2;
     char * PDF_VOLATILE _arg3 = NULL;
 
+    PDF_JAVA_SANITY_CHECK(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
     _arg1 = (int )jarg1;
     _arg2 = (int )jarg2;
@@ -2025,6 +2076,8 @@ Java_com_pdflib_pdflib_PDF_1close_1pdi_1page(JNIEnv *jenv, jclass jcls,
     EXC_STUFF;
     int _arg1;
 
+    PDF_JAVA_SANITY_CHECK_VOID(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
     _arg1 = (int )jarg1;
 
@@ -2046,6 +2099,8 @@ Java_com_pdflib_pdflib_PDF_1get_1pdi_1parameter(JNIEnv *jenv, jclass jcls,
     int _arg4;
     int len;
 
+    PDF_JAVA_SANITY_CHECK(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
     _arg2 = (int )jarg2;
     _arg3 = (int )jarg3;
@@ -2078,6 +2133,8 @@ Java_com_pdflib_pdflib_PDF_1get_1pdi_1value(JNIEnv *jenv, jclass jcls,
     int _arg3;
     int _arg4;
 
+    PDF_JAVA_SANITY_CHECK(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
     _arg2 = (int )jarg2;
     _arg3 = (int )jarg3;
@@ -2104,6 +2161,8 @@ Java_com_pdflib_pdflib_PDF_1makespotcolor(JNIEnv *jenv, jclass jcls,
     char * PDF_VOLATILE _arg1 = NULL;
     int len;
 
+    PDF_JAVA_SANITY_CHECK(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
 
     try {
@@ -2131,6 +2190,8 @@ Java_com_pdflib_pdflib_PDF_1setcolor(JNIEnv *jenv, jclass jcls,
     float _arg5;
     float _arg6;
 
+    PDF_JAVA_SANITY_CHECK_VOID(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
     _arg3 = flt2nat(jarg3);
     _arg4 = flt2nat(jarg4);
@@ -2162,6 +2223,8 @@ Java_com_pdflib_pdflib_PDF_1begin_1pattern(JNIEnv *jenv, jclass jcls,
     float _arg4;
     int _arg5;
 
+    PDF_JAVA_SANITY_CHECK(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
     _arg1 = flt2nat(jarg1);
     _arg2 = flt2nat(jarg2);
@@ -2184,6 +2247,8 @@ Java_com_pdflib_pdflib_PDF_1end_1pattern(JNIEnv *jenv, jclass jcls,
     PDF *p;
     EXC_STUFF;
 
+    PDF_JAVA_SANITY_CHECK_VOID(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
 
     try {     PDF_end_pattern(p);
@@ -2201,6 +2266,8 @@ Java_com_pdflib_pdflib_PDF_1begin_1template(JNIEnv *jenv, jclass jcls,
     float _arg1;
     float _arg2;
 
+    PDF_JAVA_SANITY_CHECK(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
     _arg1 = flt2nat(jarg1);
     _arg2 = flt2nat(jarg2);
@@ -2219,6 +2286,8 @@ Java_com_pdflib_pdflib_PDF_1end_1template(JNIEnv *jenv, jclass jcls,
     PDF *p;
     EXC_STUFF;
 
+    PDF_JAVA_SANITY_CHECK_VOID(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
 
     try {     PDF_end_template(p);
@@ -2238,6 +2307,8 @@ Java_com_pdflib_pdflib_PDF_1arcn(JNIEnv *jenv, jclass jcls,
     float _arg4;
     float _arg5;
 
+    PDF_JAVA_SANITY_CHECK_VOID(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
     _arg1 = flt2nat(jarg1);
     _arg2 = flt2nat(jarg2);
@@ -2257,6 +2328,8 @@ Java_com_pdflib_pdflib_PDF_1add_1thumbnail(JNIEnv *jenv, jclass jcls,
     EXC_STUFF;
     int _arg1;
 
+    PDF_JAVA_SANITY_CHECK_VOID(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
     _arg1 = (int )jarg1;
 
@@ -2278,6 +2351,8 @@ Java_com_pdflib_pdflib_PDF_1setmatrix(JNIEnv *jenv, jclass jcls,
     float _arg5;
     float _arg6;
 
+    PDF_JAVA_SANITY_CHECK_VOID(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
     _arg1 = flt2nat(jarg1);
     _arg2 = flt2nat(jarg2);
@@ -2297,6 +2372,8 @@ Java_com_pdflib_pdflib_PDF_1initgraphics(JNIEnv *jenv, jclass jcls,
     PDF *p;
     EXC_STUFF;
 
+    PDF_JAVA_SANITY_CHECK_VOID(jarg0);
+
     p = PDF_ENV2PTR(jarg0);
 
     try {     PDF_initgraphics(p);
