@@ -1,7 +1,7 @@
 /*---------------------------------------------------------------------------*
  |              PDFlib - A library for generating PDF on the fly             |
  +---------------------------------------------------------------------------+
- | Copyright (c) 1997-2001 PDFlib GmbH and Thomas Merz. All rights reserved. |
+ | Copyright (c) 1997-2002 PDFlib GmbH and Thomas Merz. All rights reserved. |
  +---------------------------------------------------------------------------+
  |    This software is NOT in the public domain.  It can be used under two   |
  |    substantially different licensing terms:                               |
@@ -27,7 +27,7 @@
  |    however, will receive warranty and support statements in writing.      |
  *---------------------------------------------------------------------------*/
 
-/* $Id: p_annots.c,v 1.15 2001/03/21 16:00:59 tm Exp $
+/* $Id: p_annots.c,v 1.15.2.12 2002/06/14 16:28:38 tm Exp $
  *
  * PDFlib routines for annnotations
  *
@@ -126,33 +126,40 @@ static void
 pdf_write_border_style(PDF *p, pdf_annot *ann)
 {
     /* don't write the default values */
-    if (ann->border_style == border_solid && ann->border_width == (float) 1.0 &&
-	p->border_red == (float) 0.0 && p->border_green == (float) 0.0 &&
-	p->border_blue == (float) 0.0 &&
-	p->border_dash1 == (float) 3.0 && p->border_dash2 == (float) 3.0)
+    if (ann->border_style == border_solid &&
+        ann->border_width == (float) 1.0 &&
+        ann->border_red == (float) 0.0 &&
+        ann->border_green == (float) 0.0 &&
+        ann->border_blue == (float) 0.0 &&
+        ann->border_dash1 == (float) 3.0 &&
+        ann->border_dash2 == (float) 3.0)
 	return;
 
-    pdf_puts(p, "/BS");
-    pdf_begin_dict(p);			/* BS dict */
-    pdf_puts(p, "/Type/Border\n");
+    if (ann->type != ann_attach) {
+	pdf_puts(p, "/BS");
+	pdf_begin_dict(p);			/* BS dict */
+	pdf_puts(p, "/Type/Border\n");
 
-    if (ann->border_style != border_solid)
-	pdf_printf(p, "/S/%s\n", pdf_border_style_names[ann->border_style]);
+	if (ann->border_style != border_solid)
+	    pdf_printf(p, "/S/%s\n", pdf_border_style_names[ann->border_style]);
 
-    if (ann->border_style == border_dashed)
-	pdf_printf(p, "/D[%f %f]\n", ann->border_dash1, ann->border_dash2);
+	if (ann->border_style == border_dashed)
+	    pdf_printf(p, "/D[%f %f]\n", ann->border_dash1, ann->border_dash2);
 
-    pdf_end_dict(p);			/* BS dict */
+	pdf_end_dict(p);			/* BS dict */
 
-    /* Write the Border key in old-style PDF 1.1 format */
-    pdf_printf(p, "/Border[0 0 %f", ann->border_width);
+	/* Write the Border key in old-style PDF 1.1 format */
+	pdf_printf(p, "/Border[0 0 %f", ann->border_width);
 
-    if (ann->border_style == border_dashed &&
-	(ann->border_dash1 != (float) 0.0 || ann->border_dash2 != (float) 0.0))
-	/* set dashed border */
-	pdf_printf(p, "[%f %f]", ann->border_dash1, ann->border_dash2);
+	if (ann->border_style == border_dashed &&
+	    (ann->border_dash1 != (float) 0.0 || ann->border_dash2 !=
+	    (float) 0.0))
+	    /* set dashed border */
+	    pdf_printf(p, "[%f %f]", ann->border_dash1, ann->border_dash2);
 
-    pdf_puts(p, "]\n");
+	pdf_puts(p, "]\n");
+
+    }
 
     /* write annotation color */
     pdf_printf(p, "/C[%f %f %f]\n",
@@ -193,7 +200,7 @@ pdf_write_page_annots(PDF *p)
 	switch (ann->type) {
 	    case ann_text:
 		pdf_puts(p, "/Subtype/Text\n");
-		pdf_printf(p, "/Rect[%f %f %f %f]\n", 
+		pdf_printf(p, "/Rect[%f %f %f %f]\n",
 		    ann->rect.llx, ann->rect.lly, ann->rect.urx, ann->rect.ury);
 
 		pdf_write_border_style(p, ann);
@@ -224,7 +231,7 @@ pdf_write_page_annots(PDF *p)
 
 	    case ann_locallink:
 		pdf_puts(p, "/Subtype/Link\n");
-		pdf_printf(p, "/Rect[%f %f %f %f]\n", 
+		pdf_printf(p, "/Rect[%f %f %f %f]\n",
 		    ann->rect.llx, ann->rect.lly, ann->rect.urx, ann->rect.ury);
 
 		pdf_write_border_style(p, ann);
@@ -262,7 +269,7 @@ pdf_write_page_annots(PDF *p)
 
 	    case ann_pdflink:
 		pdf_puts(p, "/Subtype/Link\n");
-		pdf_printf(p, "/Rect[%f %f %f %f]\n", 
+		pdf_printf(p, "/Rect[%f %f %f %f]\n",
 		    ann->rect.llx, ann->rect.lly, ann->rect.urx, ann->rect.ury);
 
 		pdf_write_border_style(p, ann);
@@ -272,30 +279,32 @@ pdf_write_page_annots(PDF *p)
 		pdf_puts(p, "/Type/Action/S/GoToR\n");
 
 		if (ann->dest.type == retain) {
-		    pdf_printf(p, "/D[%d 0 R/XYZ null null 0]\n",
+		    pdf_printf(p, "/D[%d/XYZ null null 0]\n",
 			    ann->dest.page-1);	/* zero-based */
 		} else if (ann->dest.type == fitpage) {
 		    /* zero-based */
-		    pdf_printf(p, "/D[%d 0 R/Fit]\n", ann->dest.page-1);
+		    pdf_printf(p, "/D[%d/Fit]\n", ann->dest.page-1);
 		} else if (ann->dest.type == fitwidth) {
 		    /* Trick: we don't know the height of a future page yet,
 		     * so we use a "large" value for top which will do for
 		     * most pages. If it doesn't work, not much harm is done.
 		     */
 		    /* zero-based */
-		    pdf_printf(p, "/D[%d 0 R/FitH 2000]\n", ann->dest.page-1);
+		    pdf_printf(p, "/D[%d/FitH 2000]\n", ann->dest.page-1);
 		} else if (ann->dest.type == fitheight) {
 		    /* zero-based */
-		    pdf_printf(p, "/D[%d 0 R/FitV 0]\n", ann->dest.page-1);
+		    pdf_printf(p, "/D[%d/FitV 0]\n", ann->dest.page-1);
 		} else if (ann->dest.type == fitbbox) {
 		    /* zero-based */
-		    pdf_printf(p, "/D[%d 0 R/FitB]\n", ann->dest.page-1);
+		    pdf_printf(p, "/D[%d/FitB]\n", ann->dest.page-1);
 		}
 
 		pdf_puts(p, "/F");
 		pdf_begin_dict(p);			/* F dict */
-		pdf_puts(p, "/Type/FileSpec\n");
-		pdf_printf(p, "/F(%s)\n", ann->filename);
+		pdf_puts(p, "/Type/Filespec\n");
+		pdf_printf(p, "/F");
+		pdf_quote_string2(p, ann->filename, (int)strlen(ann->filename));
+		pdf_puts(p, "\n");
 		pdf_end_dict(p);			/* F dict */
 
 		pdf_end_dict(p);			/* A dict */
@@ -304,7 +313,7 @@ pdf_write_page_annots(PDF *p)
 
 	    case ann_launchlink:
 		pdf_puts(p, "/Subtype/Link\n");
-		pdf_printf(p, "/Rect[%f %f %f %f]\n", 
+		pdf_printf(p, "/Rect[%f %f %f %f]\n",
 		    ann->rect.llx, ann->rect.lly, ann->rect.urx, ann->rect.ury);
 
 		pdf_write_border_style(p, ann);
@@ -315,8 +324,10 @@ pdf_write_page_annots(PDF *p)
 
 		pdf_puts(p, "/F");
 		pdf_begin_dict(p);			/* F dict */
-		pdf_puts(p, "/Type/FileSpec\n");
-		pdf_printf(p, "/F(%s)\n", ann->filename);
+		pdf_puts(p, "/Type/Filespec\n");
+		pdf_printf(p, "/F");
+		pdf_quote_string2(p, ann->filename, (int)strlen(ann->filename));
+		pdf_puts(p, "\n");
 		pdf_end_dict(p);			/* F dict */
 
 		pdf_end_dict(p);			/* A dict */
@@ -325,20 +336,23 @@ pdf_write_page_annots(PDF *p)
 
 	    case ann_weblink:
 		pdf_puts(p, "/Subtype/Link\n");
-		pdf_printf(p, "/Rect[%f %f %f %f]\n", 
+		pdf_printf(p, "/Rect[%f %f %f %f]\n",
 		    ann->rect.llx, ann->rect.lly, ann->rect.urx, ann->rect.ury);
 
 		pdf_write_border_style(p, ann);
 
-		pdf_printf(p, "/A<</S/URI/URI(%s)>>\n", 
-				    ann->filename);
+		pdf_printf(p, "/A<</S/URI/URI");
+		pdf_quote_string2(p, ann->filename, (int)strlen(ann->filename));
+		pdf_puts(p, ">>\n");
 		break;
 
 	    case ann_attach:
 		pdf_puts(p, "/Subtype/FileAttachment\n");
-		pdf_printf(p, "/Rect[%f %f %f %f]\n", 
+		pdf_printf(p, "/Rect[%f %f %f %f]\n",
 		    ann->rect.llx, ann->rect.lly, ann->rect.urx, ann->rect.ury);
 
+		pdf_write_border_style(p, ann);
+
 		if (ann->icon != icon_file_pushpin)	/* pushpin is default */
 		    pdf_printf(p, "/Name/%s\n",
 		    	pdf_icon_names[ann->icon]);
@@ -354,18 +368,20 @@ pdf_write_page_annots(PDF *p)
 		    pdf_quote_string(p, ann->contents);
 		    pdf_putc(p, PDF_NEWLINE);
 		}
-		    
+
 		/* the icon is too small without these flags (=28) */
 		pdf_printf(p, "/F %d\n",
 			pdf_ann_flag_print |
-			pdf_ann_flag_nozoom | 
+			pdf_ann_flag_nozoom |
 			pdf_ann_flag_norotate);
 
 		pdf_puts(p, "/FS");
 		pdf_begin_dict(p);			/* FS dict */
-		pdf_puts(p, "/Type/FileSpec\n");
+		pdf_puts(p, "/Type/Filespec\n");
 
-		pdf_printf(p, "/F(%s)\n", ann->filename);
+		pdf_printf(p, "/F");
+		pdf_quote_string2(p, ann->filename, (int)strlen(ann->filename));
+		pdf_puts(p, "\n");
 
 		/* alloc id for the actual embedded file stream */
 		ann->obj_id = pdf_alloc_id(p);
@@ -391,8 +407,11 @@ pdf_write_page_annots(PDF *p)
 	pdf_begin_obj(p, ann->obj_id);	/* EmbeddedFile */
 	pdf_puts(p, "<</Type/EmbeddedFile\n");
 
-	if (ann->mimetype)
-	    pdf_printf(p, "/Subtype (%s)\n", ann->mimetype);
+	if (ann->mimetype) {
+	    pdf_puts(p, "/Subtype/");
+	    pdf_quote_name(p, ann->mimetype, strlen(ann->mimetype));
+	    pdf_puts(p, "\n");
+	}
 
 	if (p->compresslevel)
 	    pdf_puts(p, "/Filter/FlateDecode\n");
@@ -409,6 +428,8 @@ pdf_write_page_annots(PDF *p)
 	src.init	= pdf_data_source_file_init;
 	src.fill	= pdf_data_source_file_fill;
 	src.terminate	= pdf_data_source_file_terminate;
+	src.length	= (long) 0;
+	src.offset	= (long) 0;
 
 	pdf_compress(p, &src);
 
@@ -571,22 +592,15 @@ PDF_attach_file(
 
     if (description != NULL) {
 	ann->contents = (char *) pdf_strdup(p, description);
-#ifdef PDFLIB_EBCDIC
-	if (!pdf_is_unicode(ann->contents))
-	    pdf_make_ascii(ann->contents);
-#endif
     }
 
     if (author != NULL) {
 	ann->title = (char *) pdf_strdup(p, author);
-#ifdef PDFLIB_EBCDIC
-	if (!pdf_is_unicode(ann->title))
-	    pdf_make_ascii(ann->title);
-#endif
     }
 
-    if (mimetype != NULL)
+    if (mimetype != NULL) {
 	ann->mimetype = (char *) pdf_strdup(p, mimetype);
+    }
 
     pdf_add_annot(p, ann);
 }
@@ -646,24 +660,16 @@ PDF_add_note(
     else
 	pdf_error(p, PDF_ValueError,
 		"Unknown icon type '%s' for text note", icon);
-	
+
     /* title may be NULL */
     if (title != NULL) {
 	ann->title = pdf_strdup(p, title);
-#ifdef PDFLIB_EBCDIC
-	if (!pdf_is_unicode(ann->title))
-	    pdf_make_ascii(ann->title);
-#endif
     } else
 	ann->title = NULL;
 
     /* It is legal to create an empty text annnotation */
     if (contents != NULL) {
 	ann->contents = pdf_strdup(p, contents);
-#ifdef PDFLIB_EBCDIC
-	if (!pdf_is_unicode(ann->contents))
-	    pdf_make_ascii(ann->contents);
-#endif
     } else
 	ann->contents = NULL;
 
@@ -807,7 +813,7 @@ PDF_add_locallink(
     else if (!strcmp(desttype, "fitbbox"))
 	ann->dest.type 	= fitbbox;
     else
-	pdf_error(p, PDF_ValueError,	
+	pdf_error(p, PDF_ValueError,
 	    "Unknown destination type '%s' in PDF_add_locallink", desttype);
 
     ann->rect.llx = llx;
