@@ -1,7 +1,7 @@
 /*---------------------------------------------------------------------------*
  |              PDFlib - A library for generating PDF on the fly             |
  +---------------------------------------------------------------------------+
- | Copyright (c) 1997-2001 PDFlib GmbH and Thomas Merz. All rights reserved. |
+ | Copyright (c) 1997-2002 PDFlib GmbH and Thomas Merz. All rights reserved. |
  +---------------------------------------------------------------------------+
  |    This software is NOT in the public domain.  It can be used under two   |
  |    substantially different licensing terms:                               |
@@ -27,7 +27,7 @@
  |    however, will receive warranty and support statements in writing.      |
  *---------------------------------------------------------------------------*/
 
-/* $Id: p_basic.c,v 1.61.2.5 2001/05/16 14:25:35 rjs Exp $
+/* $Id: p_basic.c,v 1.61.2.26 2002/05/31 12:18:11 tm Exp $
  *
  * PDFlib general routines
  *
@@ -39,23 +39,25 @@
 #include <string.h>
 #ifndef WINCE
 #include <time.h>
+#include "p_intern.h"
 #else
+#include "p_intern.h"
 #include <winbase.h>
 #endif
 
-#include "p_intern.h"
 #include "p_font.h"
+#include "p_md5.h"
 
 #if !defined(WIN32) && !defined(OS2)
 #include <unistd.h>
 #endif
 
-#if defined(WIN32) || defined(OS2)
+#if (defined(WIN32) || defined(OS2)) && !defined(WINCE)
 #include <fcntl.h>
 #include <io.h>
 #endif
 
-#ifdef MAC
+#if defined(MAC) || defined(MACOSX)
 #include <Files.h>
 #endif
 
@@ -66,6 +68,142 @@
 #undef WIN32_LEAN_AND_MEAN
 #endif
 
+static PDFlib_api PDFlib = {
+    /* general functions */
+    PDF_get_api,
+    PDF_get_majorversion,
+    PDF_get_minorversion,
+    PDF_boot,
+    PDF_shutdown,
+    PDF_new2,
+    PDF_get_opaque,
+    PDF_new,
+    PDF_delete,    
+    PDF_open_file,
+    PDF_open_fp,
+    PDF_open_mem,
+    PDF_close,
+    PDF_begin_page,
+    PDF_end_page,
+
+    /* parameter handling */
+    PDF_set_parameter,
+    PDF_set_value,
+    PDF_get_parameter,
+    PDF_get_value,
+
+    /* text and font handling */
+    PDF_findfont,
+    PDF_setfont,    
+    PDF_encoding_get_name,
+
+    /* text output */
+    PDF_show,
+    PDF_show_xy,
+    PDF_continue_text,    
+    PDF_show_boxed,
+    PDF_set_text_matrix,
+    PDF_set_text_pos,
+    PDF_stringwidth,
+    PDF_show2,
+    PDF_show_xy2,
+    PDF_continue_text2,
+    PDF_stringwidth2,
+
+    /* graphics state */
+    PDF_setdash,
+    PDF_setpolydash,
+    PDF_setflat,
+    PDF_setlinejoin,
+    PDF_setlinecap,
+    PDF_setmiterlimit,
+    PDF_setlinewidth,
+    PDF_initgraphics,
+    PDF_save,
+    PDF_restore,
+    PDF_translate,
+    PDF_scale,
+    PDF_rotate,
+    PDF_skew,    
+    PDF_concat,
+    PDF_setmatrix,
+
+    /* path construction, painting, and clipping */
+    PDF_moveto,
+    PDF_lineto,    
+    PDF_curveto,
+    PDF_circle,
+    PDF_arc,
+    PDF_arcn,
+    PDF_rect,
+    PDF_closepath,
+    PDF_stroke,
+    PDF_closepath_stroke,
+    PDF_fill,
+    PDF_fill_stroke,
+    PDF_closepath_fill_stroke,
+    PDF_endpath,
+    PDF_clip,
+
+    /* color handling */
+    PDF_setgray_fill,
+    PDF_setgray_stroke,
+    PDF_setgray,    
+    PDF_setrgbcolor_fill,
+    PDF_setrgbcolor_stroke,
+    PDF_setrgbcolor,
+    PDF_makespotcolor,
+    PDF_setcolor,
+
+    /* pattern definition */
+    PDF_begin_pattern,
+    PDF_end_pattern,
+
+    /* template definition */
+    PDF_begin_template,
+    PDF_end_template,
+
+
+    /* image handling */
+    PDF_place_image,
+    PDF_open_image,
+    PDF_open_image_file,
+    PDF_close_image,
+    PDF_add_thumbnail,
+
+    /* fax-compressed data processing */
+    PDF_open_CCITT,
+
+    /* bookmarks and document info fields */
+    PDF_add_bookmark,
+    PDF_set_info,
+
+    /* file attachments, notes, and links */
+    PDF_attach_file,
+    PDF_add_note,
+    PDF_add_pdflink,
+    PDF_add_launchlink,
+    PDF_add_locallink,
+    PDF_add_weblink,
+    PDF_set_border_style,
+    PDF_set_border_color,
+    PDF_set_border_dash,
+
+    /* PDF import (requires the PDI library) */
+    PDF_open_pdi,
+    PDF_close_pdi,
+    PDF_open_pdi_page,
+    PDF_place_pdi_page,
+    PDF_close_pdi_page,
+    PDF_get_pdi_parameter,
+    PDF_get_pdi_value,
+
+    /* output stream handling */ 
+    PDF_get_buffer,
+
+    NULL
+};
+
 PDFLIB_API void PDFLIB_CALL
 PDF_boot(void)
 {
@@ -82,6 +220,12 @@ PDF_shutdown(void)
     /* nothing yet */
 }
 
+PDFLIB_API PDFlib_api * PDFLIB_CALL
+PDF_get_api(void)
+{
+    return &PDFlib;
+}
+
 #if (defined(WIN32) || defined(__CYGWIN)) && defined(PDFLIB_EXPORTS)
 
 /*
@@ -90,6 +234,9 @@ PDF_shutdown(void)
  * be used to boot thread-global resources for PDFlib
  * (mainly font-related stuff).
  */
+BOOL WINAPI
+DllMain(HANDLE hModule, DWORD ul_reason_for_call, LPVOID lpReserved);
+
 BOOL WINAPI
 DllMain(HANDLE hModule, DWORD ul_reason_for_call, LPVOID lpReserved)
 {
@@ -318,8 +465,7 @@ pdf_init_document(PDF *p)
     p->currentobj	= (id) 0;
 
     p->pnodes_capacity = PNODES_CHUNKSIZE;
-    p->pnodes = (id *) p->malloc(p,
-	    sizeof(id) * p->pnodes_capacity, fn);
+    p->pnodes = (id *) p->malloc(p, sizeof(id) * p->pnodes_capacity, fn);
 
     p->pnodes[0]	= pdf_alloc_id(p);
     p->current_pnode	= 0;
@@ -334,6 +480,9 @@ pdf_init_document(PDF *p)
     p->bookmark_dest	= retain;
     p->base		= NULL;
 
+    p->md5 = p->malloc(p, sizeof(MD5_CTX), fn);
+    MD5_Init(p->md5);
+
     PDF_SET_STATE(p, fn, pdf_state_document);
 }
 
@@ -365,8 +514,8 @@ pdf_malloc(PDF *p, size_t size, const char *caller)
 
 #ifdef DEBUG
     if (p != NULL && p->debug['m'])
-	fprintf(stderr, "%x malloced, size %d from %s, page %d\n",
-		(int)ret, size, caller, p->current_page);
+	fprintf(stderr, "%p malloced, size %d from %s, page %d\n",
+		ret, size, caller, p->current_page);
 #endif
 
     /* Special error handling at first allocation (for p itself) */
@@ -405,8 +554,8 @@ pdf_calloc(PDF *p, size_t size, const char *caller)
 
 #ifdef DEBUG
     if (p->debug['c'])
-	fprintf(stderr, "%x calloced, size %d from %s, page %d\n",
-		(int) ret, size, caller, p->current_page);
+	fprintf(stderr, "%p calloced, size %d from %s, page %d\n",
+		ret, size, caller, p->current_page);
 #endif
 
     return ret;
@@ -424,8 +573,8 @@ pdf_realloc(PDF *p, void *mem, size_t size, const char *caller)
 
 #ifdef DEBUG
     if (p->debug['r'])
-	fprintf(stderr, "%x realloced to %x, %d from %s, page %d\n",
-		(int) mem, (int) ret, (int) size, caller, p->current_page);
+	fprintf(stderr, "%p realloced to %p, %d from %s, page %d\n",
+		mem, ret, (int) size, caller, p->current_page);
 #endif
     return ret;
 }
@@ -435,7 +584,7 @@ pdf_free(PDF *p, void *mem)
 {
 #ifdef DEBUG
     if (p->debug['f'])
-	fprintf(stderr, "%x freed, page %d\n", (int) mem, p->current_page);
+	fprintf(stderr, "%p freed, page %d\n", mem, p->current_page);
 #endif
 
     /* We mustn't raise a fatal error here to avoid potential recursion */
@@ -508,7 +657,8 @@ PDF_new2(
 	pdf.magic = PDF_MAGIC;
 	pdf.opaque = opaque;
 
-	(*errorhandler)(&pdf, PDF_MemoryError, "Couldn't allocate PDF object");
+	(*errorhandler)(&pdf, PDF_MemoryError,
+	    "Couldn't allocate PDFlib object");
 
 	return NULL;	/* Won't happen because of the error handler */
     }
@@ -534,6 +684,7 @@ PDF_new2(
     p->calloc		= pdf_calloc;
 
     p->compresslevel	= PDF_DEFAULT_COMPRESSION;
+    p->inheritgs	= pdf_true;
 
     p->resourcefilename	= NULL;
     p->resources	= NULL;
@@ -564,6 +715,7 @@ PDF_new2(
 #endif
     pdf_boot_stream(p);
 
+
     PDF_TRACE((" [%p]\n", p));
 
     return p;
@@ -581,46 +733,85 @@ PDF_get_opaque(PDF *p)
     return p->opaque;
 }
 
+#if defined(MAC) || defined(MACOSX)
+#if defined(TARGET_API_MAC_CARBON)
+OSStatus FSPathMakeFSSpec(const UInt8 *path, FSSpec *spec);
+
+OSStatus
+FSPathMakeFSSpec(const UInt8 *path, FSSpec *spec)
+{
+	OSStatus	result;
+	FSRef		ref;
+
+	/* 
+	 * Convert the POSIX path to an FSRef.
+	 * For some reason this doesn't work with CodeWarrior so we use
+	 * the old-style method there.
+	 */
+	result = FSPathMakeRef(path, &ref, NULL);
+	
+	if (result != noErr)
+	    return result;
+	
+	/* and then convert the FSRef to an FSSpec */
+	result = FSGetCatalogInfo(&ref, kFSCatInfoNone, NULL, NULL, spec, NULL);
+
+	return result;
+}
+#endif	/* TARGET_API_MAC_CARBON */
+#endif	/* defined(MAC) || defined(MACOSX) */
+
 /* close the output file, if opened with PDF_open_file();
  * close the output stream if opened 
  */
 static void
 pdf_close_file(PDF *p)
 {
-#ifdef MAC
-    FCBPBRec	fcbInfo;
-    Str32	name;
+#if defined(MAC) || defined(MACOSX)
+#if !defined(TARGET_API_MAC_CARBON) || defined(__MWERKS__)
+    FCBPBRec    fcbInfo;
+    Str32       name;
+#endif	/* TARGET_API_MAC_CARBON */
     FInfo	fInfo;
     FSSpec	fSpec;
-#endif
+#endif	/* defined(MAC) || defined(MACOSX) */
 
     if (p->fp == NULL)
 	return;
 
-#ifdef MAC
-/*  Contributed by Leonard Rosenthol:
- *  On the MacOS, files are not associated with applications by extensions.
- *  Instead, it uses a pair of values called the type & creator.
- *  This block of code sets those values for PDF files.
- */
-    memset(&fcbInfo, 0, sizeof(FCBPBRec));
-    fcbInfo.ioRefNum = (short) p->fp->handle;
-    fcbInfo.ioNamePtr = name;
-    if (!PBGetFCBInfoSync(&fcbInfo) &&
-	FSMakeFSSpec(fcbInfo.ioFCBVRefNum, fcbInfo.ioFCBParID, name, &fSpec)
-		== noErr) {
-	    FSpGetFInfo(&fSpec, &fInfo);
-	    fInfo.fdType = 'PDF ';
-	    fInfo.fdCreator = 'CARO';
-	    FSpSetFInfo(&fSpec, &fInfo);
-	}
-#endif
-
     /*
      * If filename is set, we started with PDF_open_file; therefore
      * we also close the file ourselves.
      */
     if (p->filename && p->writeproc) {
+#if defined(MAC) || defined(MACOSX)
+	/* set the proper type and creator for the output file */
+#if defined(TARGET_API_MAC_CARBON) && !defined(__MWERKS__)
+
+	if (FSPathMakeFSSpec((const UInt8 *) p->filename, &fSpec) == noErr) {
+	    FSpGetFInfo(&fSpec, &fInfo);
+	    fInfo.fdType = 'PDF ';
+	    fInfo.fdCreator = 'CARO';
+	    FSpSetFInfo(&fSpec, &fInfo);
+	}
+#else
+
+	memset(&fcbInfo, 0, sizeof(FCBPBRec));
+	fcbInfo.ioRefNum = (short) p->fp->handle;
+	fcbInfo.ioNamePtr = name;
+
+	if (!PBGetFCBInfoSync(&fcbInfo) &&
+	    FSMakeFSSpec(fcbInfo.ioFCBVRefNum, fcbInfo.ioFCBParID, name, &fSpec)
+		    == noErr) {
+	    FSpGetFInfo(&fSpec, &fInfo);
+	    fInfo.fdType = 'PDF ';
+	    fInfo.fdCreator = 'CARO';
+	    FSpSetFInfo(&fSpec, &fInfo);
+	}
+
+#endif	/* !defined(TARGET_API_MAC_CARBON) || defined(__MWERKS__) */
+#endif	/* defined(MAC) || defined(MACOSX) */
+
 	if (strcmp(p->filename, "-"))
 	    fclose(p->fp);
 	p->free(p, p->filename);
@@ -708,6 +899,15 @@ pdf_init_all(PDF *p)
     pdf_init_stream(p);
     pdf_init_colorspaces(p);
     pdf_init_pattern(p);
+
+    /* clients may set char/word spacing and horizontal scaling outside pages
+    ** for PDF_stringwidth() calculations, and they may set a color for use
+    ** in PDF_makespotcolor(). here we set the defaults.
+    */
+    p->sl = 0;			
+    pdf_init_tstate(p);
+    pdf_init_cstate(p);
+
 #ifdef HAVE_PDI
     pdf_reset_pdi(p);		/* reset all PDI contexts */
 #endif
@@ -728,7 +928,7 @@ pdf_write_header(PDF *p)
 
     if (p->compatibility == PDF_1_4)
 	pdf_puts(p, "%PDF-1.4\n");	      	/* PDF 1.4 header */
-    if (p->compatibility == PDF_1_3)
+    else if (p->compatibility == PDF_1_3)
 	pdf_puts(p, "%PDF-1.3\n");	      	/* PDF 1.3 header */
     else
 	pdf_puts(p, "%PDF-1.2\n");	      	/* PDF 1.2 header */
@@ -798,13 +998,13 @@ PDF_open_file(PDF *p, const char *filename)
 	p->filename = NULL;
 
     } else {
-#ifndef MAC
+#if !(defined(MAC) && defined(__MWERKS__))
 	if (filename && !strcmp(filename, "-")) {
 	    p->filename = NULL;
 	    p->fp = stdout;
 #if !defined(__MWERKS__) && (defined(WIN32) || defined(OS2))
 #if defined WINCE
-	    _setmode(fileno(fp), _O_BINARY);
+	    _setmode(fileno(stdout), _O_BINARY);
 #else
 	    setmode(fileno(stdout), O_BINARY);
 #endif /* !WINCE */
@@ -951,9 +1151,12 @@ pdf_make_tree(PDF *p, id parent_id, id *pnodes, id *pages, int n_pages)
 static void
 pdf_wrapup_document(PDF *p)
 {
+    static const char bin2hex[] = PDF_STRING_0123456789ABCDEF;
+
     long	pos;
     id		i;
     id		pages_id;
+    unsigned char digest[MD5_DIGEST_LENGTH];
 
     pdf_write_info(p);
 
@@ -1048,6 +1251,29 @@ pdf_wrapup_document(PDF *p)
     pdf_printf(p, "/Size %ld\n", p->currentobj + 1);
     pdf_printf(p, "/Info %ld 0 R\n", p->info_id);
     pdf_printf(p, "/Root %ld 0 R\n", p->root_id);
+
+    /* put some more data into the MD5 digest */
+    MD5_Update(p->md5, (unsigned char *) &pos, sizeof pos);
+    MD5_Update(p->md5, (unsigned char *) p, sizeof (PDF));
+    if (p->filename && strcmp(p->filename, "-") != 0)
+	MD5_Update(p->md5, (unsigned char *) p->filename, strlen(p->filename));
+
+    /* write the digest to the ID array */
+    MD5_Final(digest, p->md5);
+    pdf_puts(p, "/ID[<");
+    for (i = 0; i < MD5_DIGEST_LENGTH; ++i)
+    {
+	pdf_putc(p, bin2hex[digest[i] >> 4]);
+	pdf_putc(p, bin2hex[digest[i] & 0x0F]);
+    }
+    pdf_puts(p, "><");
+    for (i = 0; i < MD5_DIGEST_LENGTH; ++i)
+    {
+	pdf_putc(p, bin2hex[digest[i] >> 4]);
+	pdf_putc(p, bin2hex[digest[i] & 0x0F]);
+    }
+    pdf_puts(p, ">]\n");
+
     pdf_end_dict(p);				/* trailer */
 
     pdf_puts(p, "startxref\n");
@@ -1092,6 +1318,10 @@ pdf_cleanup_document(PDF *p)
 	p->free(p, p->pnodes);
 	p->pnodes = NULL;
     }
+    if (p->md5) {
+	p->free(p, p->md5);
+	p->md5 = NULL;
+    }
     pdf_cleanup_info(p);
     pdf_cleanup_fonts(p);
     pdf_cleanup_outlines(p);
@@ -1286,6 +1516,11 @@ PDF_end_page(PDF *p)
 	pdf_error(p, PDF_RuntimeError, "Unmatched save level at end of page");
 
 
+    /* restore text parameter and color defaults for out-of-page usage.
+    */
+    pdf_init_tstate(p);
+    pdf_init_cstate(p);
+
     pdf_end_contents_section(p);
 
     /* Page object */
@@ -1345,8 +1580,8 @@ PDF_end_page(PDF *p)
     }
 
     /*
-     * The duration can be placed in the page dictionary (/D) or the
-     * the transition dictionary (/Dur). We put it here so it can
+     * The duration can be placed in the transition dictionary (/D)
+     * or in the page dictionary (/Dur). We put it here so it can
      * be used without setting a transition effect.
      */
 
