@@ -1,7 +1,7 @@
 /*---------------------------------------------------------------------------*
  |              PDFlib - A library for generating PDF on the fly             |
  +---------------------------------------------------------------------------+
- | Copyright (c) 1997-2001 PDFlib GmbH and Thomas Merz. All rights reserved. |
+ | Copyright (c) 1997-2002 PDFlib GmbH and Thomas Merz. All rights reserved. |
  +---------------------------------------------------------------------------+
  |    This software is NOT in the public domain.  It can be used under two   |
  |    substantially different licensing terms:                               |
@@ -27,7 +27,7 @@
  |    however, will receive warranty and support statements in writing.      |
  *---------------------------------------------------------------------------*/
 
-/* $Id: p_cid.h,v 1.10 2001/04/23 11:23:52 tm Exp $
+/* $Id: p_cid.h,v 1.10.10.3 2002/04/28 20:01:47 tm Exp $
  *
  * Header file for PDFlib CID font information
  *
@@ -36,8 +36,16 @@
 #ifndef P_CID_H
 #define P_CID_H
 
-typedef struct { const char *name; pdf_charcoll charcoll; } pdf_cmap;
-typedef struct { const char *ordering; int supplement; } pdf_charcoll_names;
+typedef struct {
+    const char *name;
+    pdf_charcoll charcoll;
+    int compatibility;
+} pdf_cmap;
+
+typedef struct {
+    const char *ordering;
+    int supplement;
+} pdf_charcoll_names;
 
 pdf_charcoll_names charcoll_names[] = {
     { "GB1",		0 },	/* cc_simplified_chinese */
@@ -50,62 +58,70 @@ pdf_charcoll_names charcoll_names[] = {
 /* predefined CMaps and the corresponding character collection */
 
 pdf_cmap cmaps[] = {
-    { "GB-EUC-H",	cc_simplified_chinese },
-    { "GB-EUC-V",	cc_simplified_chinese },
-    { "GBpc-EUC-H",	cc_simplified_chinese },
-    { "GBpc-EUC-V",	cc_simplified_chinese },
-    { "GBK-EUC-H",	cc_simplified_chinese },
-    { "GBK-EUC-V",	cc_simplified_chinese },
-    { "UniGB-UCS2-H",	cc_simplified_chinese },
-    { "UniGB-UCS2-V",	cc_simplified_chinese },
+    { "GB-EUC-H",	cc_simplified_chinese, PDF_1_2 },
+    { "GB-EUC-V",	cc_simplified_chinese, PDF_1_2 },
+    { "GBpc-EUC-H",	cc_simplified_chinese, PDF_1_2 },
+    { "GBpc-EUC-V",	cc_simplified_chinese, PDF_1_3 },
+    { "GBK-EUC-H",	cc_simplified_chinese, PDF_1_3 },
+    { "GBK-EUC-V",	cc_simplified_chinese, PDF_1_3 },
+    { "GBKp-EUC-H",	cc_simplified_chinese, PDF_1_4 },
+    { "GBKp-EUC-V",	cc_simplified_chinese, PDF_1_4 },
+    { "GBK2K-H",	cc_simplified_chinese, PDF_1_4 },
+    { "GBK2K-V",	cc_simplified_chinese, PDF_1_4 },
+    { "UniGB-UCS2-H",	cc_simplified_chinese, PDF_1_3 },
+    { "UniGB-UCS2-V",	cc_simplified_chinese, PDF_1_3 },
 
-    { "B5pc-H",		cc_traditional_chinese },
-    { "B5pc-V",		cc_traditional_chinese },
-    { "ETen-B5-H",	cc_traditional_chinese },
-    { "ETen-B5-V",	cc_traditional_chinese },
-    { "ETenms-B5-H",	cc_traditional_chinese },
-    { "ETenms-B5-V",	cc_traditional_chinese },
-    { "CNS-EUC-H",	cc_traditional_chinese },
-    { "CNS-EUC-V",	cc_traditional_chinese },
-    { "UniCNS-UCS2-H",	cc_traditional_chinese },
-    { "UniCNS-UCS2-V",	cc_traditional_chinese },
+    { "B5pc-H",		cc_traditional_chinese, PDF_1_2 },
+    { "B5pc-V",		cc_traditional_chinese, PDF_1_2 },
+    { "HKscs-B5-H",	cc_traditional_chinese, PDF_1_4 },
+    { "HKscs-B5-V",	cc_traditional_chinese, PDF_1_4 },
+    { "ETen-B5-H",	cc_traditional_chinese, PDF_1_2 },
+    { "ETen-B5-V",	cc_traditional_chinese, PDF_1_2 },
+    { "ETenms-B5-H",	cc_traditional_chinese, PDF_1_3 },
+    { "ETenms-B5-V",	cc_traditional_chinese, PDF_1_3 },
+    { "CNS-EUC-H",	cc_traditional_chinese, PDF_1_2 },
+    { "CNS-EUC-V",	cc_traditional_chinese, PDF_1_2 },
+    { "UniCNS-UCS2-H",	cc_traditional_chinese, PDF_1_3 },
+    { "UniCNS-UCS2-V",	cc_traditional_chinese, PDF_1_3 },
 
-    { "83pv-RKSJ-H",	cc_japanese },
-    { "90ms-RKSJ-H",	cc_japanese },
-    { "90ms-RKSJ-V",	cc_japanese },
-    { "90msp-RKSJ-H",	cc_japanese },
-    { "90msp-RKSJ-V",	cc_japanese },
-    { "90pv-RKSJ-H",	cc_japanese },
-    { "Add-RKSJ-H",	cc_japanese },
-    { "Add-RKSJ-V",	cc_japanese },
-    { "EUC-H",		cc_japanese },
-    { "EUC-V",		cc_japanese },
-    { "Ext-RKSJ-H",	cc_japanese },
-    { "Ext-RKSJ-V",	cc_japanese },
-    { "H",		cc_japanese },
-    { "V",		cc_japanese },
-    { "UniJIS-UCS2-H",	cc_japanese },
-    { "UniJIS-UCS2-V",	cc_japanese },
-    { "UniJIS-UCS2-HW-H",	cc_japanese },
-    { "UniJIS-UCS2-HW-V",	cc_japanese },
+    { "83pv-RKSJ-H",	cc_japanese, PDF_1_2 },
+    { "90ms-RKSJ-H",	cc_japanese, PDF_1_2 },
+    { "90ms-RKSJ-V",	cc_japanese, PDF_1_2 },
+    { "90msp-RKSJ-H",	cc_japanese, PDF_1_3 },
+    { "90msp-RKSJ-V",	cc_japanese, PDF_1_3 },
+    { "90pv-RKSJ-H",	cc_japanese, PDF_1_2 },
+    { "Add-RKSJ-H",	cc_japanese, PDF_1_2 },
+    { "Add-RKSJ-V",	cc_japanese, PDF_1_2 },
+    { "EUC-H",		cc_japanese, PDF_1_3 },
+    { "EUC-V",		cc_japanese, PDF_1_3 },
+    { "Ext-RKSJ-H",	cc_japanese, PDF_1_2 },
+    { "Ext-RKSJ-V",	cc_japanese, PDF_1_2 },
+    { "H",		cc_japanese, PDF_1_2 },
+    { "V",		cc_japanese, PDF_1_2 },
+    { "UniJIS-UCS2-H",	cc_japanese, PDF_1_3 },
+    { "UniJIS-UCS2-V",	cc_japanese, PDF_1_3 },
+    { "UniJIS-UCS2-HW-H",	cc_japanese, PDF_1_3 },
+    { "UniJIS-UCS2-HW-V",	cc_japanese, PDF_1_3 },
 
-    { "KSC-EUC-H",	cc_korean },
-    { "KSC-EUC-V",	cc_korean },
-    { "KSCms-UHC-H",	cc_korean },
-    { "KSCms-UHC-V",	cc_korean },
-    { "KSCms-UHC-HW-H",	cc_korean },
-    { "KSCms-UHC-HW-V",	cc_korean },
-    { "KSCpc-EUC-H",	cc_korean },
-    { "UniKS-UCS2-H",	cc_korean },
-    { "UniKS-UCS2-V",	cc_korean },
+    { "KSC-EUC-H",	cc_korean, PDF_1_2 },
+    { "KSC-EUC-V",	cc_korean, PDF_1_2 },
+    { "KSCms-UHC-H",	cc_korean, PDF_1_2 },
+    { "KSCms-UHC-V",	cc_korean, PDF_1_2 },
+    { "KSCms-UHC-HW-H",	cc_korean, PDF_1_3 },
+    { "KSCms-UHC-HW-V",	cc_korean, PDF_1_3 },
+    { "KSCpc-EUC-H",	cc_korean, PDF_1_2 },
+    { "UniKS-UCS2-H",	cc_korean, PDF_1_3 },
+    { "UniKS-UCS2-V",	cc_korean, PDF_1_3 },
 
-    { "Identity-H",	cc_identity },
-    { "Identity-V",	cc_identity }
+    { "Identity-H",	cc_identity, PDF_1_2 },
+    { "Identity-V",	cc_identity, PDF_1_2 }
 };
 
 /* Font descriptors for the preinstalled CID fonts */
 
 const pdf_font pdf_cid_metrics[] = {
+
+/* Acrobat 4 standard fonts */
 /* ---------------------------------------------------------- */
 {	"HeiseiKakuGo-W5",		/* FontName */
 	cid,				/* encoding */
@@ -361,6 +377,159 @@ const pdf_font pdf_cid_metrics[] = {
 	91,				/* StdVW */
 	91,				/* StdHW */
 
+	0,				/* numOfChars */
+	(CharMetricInfo *) NULL,	/* cmi */
+	0,				/* numOfTracks */
+	(TrackKernData *) NULL,		/* tkd */
+	0,				/* numOfPairs */
+	(PairKernData *) NULL,		/* pkd */
+	0,				/* numOfComps */
+	(CompCharData *) NULL,		/* ccd */
+					/* skip character metrics array */
+},
+
+/* Acrobat 5 standard fonts */
+/* ---------------------------------------------------------- */
+{	"HYSMyeongJoStd-Medium-Acro",	/* FontName */
+	cid,				/* encoding */
+	0,				/* used on current page */
+	0,				/* embed */
+	(char *) NULL,			/* font file */
+	0L,				/* Object id */
+	6L,				/* Font flags */
+	CIDFontType0,			/* font type */
+	NULL,				/* TT name */
+	(char *) NULL,			/* Native encoding */
+	cc_korean,			/* Character collection */
+	(float) 0.0,			/* ItalicAngle */
+	pdf_true,			/* isFixedPitch */
+	-28,				/* llx */
+	-148,				/* lly */
+	1001,				/* urx */
+	880,				/* ury */
+	-100,				/* UnderlinePosition */
+	50,				/* UnderlineThickness */
+	737,				/* CapHeight */
+	553,				/* xHeight */
+	752,				/* Ascender */
+	-271,				/* Descender */
+	58,				/* StdVW */
+	0,				/* StdHW */
+
+	0,				/* numOfChars */
+	(CharMetricInfo *) NULL,	/* cmi */
+	0,				/* numOfTracks */
+	(TrackKernData *) NULL,		/* tkd */
+	0,				/* numOfPairs */
+	(PairKernData *) NULL,		/* pkd */
+	0,				/* numOfComps */
+	(CompCharData *) NULL,		/* ccd */
+					/* skip character metrics array */
+},
+
+/* ---------------------------------------------------------- */
+{	"KozMinPro-Regular-Acro",	/* FontName */
+	cid,				/* encoding */
+	0,				/* used on current page */
+	0,				/* embed */
+	(char *) NULL,			/* font file */
+	0L,				/* Object id */
+	6L,				/* Font flags */
+	CIDFontType0,			/* font type */
+	NULL,				/* TT name */
+	(char *) NULL,			/* Native encoding */
+	cc_japanese,			/* Character collection */
+	(float) 0.0,			/* ItalicAngle */
+	pdf_true,			/* isFixedPitch */
+	-195,				/* llx */
+	-272,				/* lly */
+	1110,				/* urx */
+	1075,				/* ury */
+	-100,				/* UnderlinePosition */
+	50,				/* UnderlineThickness */
+	737,				/* CapHeight */
+	553,				/* xHeight */
+	752,				/* Ascender */
+	-271,				/* Descender */
+	58,				/* StdVW */
+	0,				/* StdHW */
+
+	0,				/* numOfChars */
+	(CharMetricInfo *) NULL,	/* cmi */
+	0,				/* numOfTracks */
+	(TrackKernData *) NULL,		/* tkd */
+	0,				/* numOfPairs */
+	(PairKernData *) NULL,		/* pkd */
+	0,				/* numOfComps */
+	(CompCharData *) NULL,		/* ccd */
+					/* skip character metrics array */
+},
+
+/* ---------------------------------------------------------- */
+{	"MSungStd-Light-Acro",		/* FontName */
+	cid,				/* encoding */
+	0,				/* used on current page */
+	0,				/* embed */
+	(char *) NULL,			/* font file */
+	0L,				/* Object id */
+	6L,				/* Font flags */
+	CIDFontType0,			/* font type */
+	NULL,				/* TT name */
+	(char *) NULL,			/* Native encoding */
+	cc_traditional_chinese,		/* Character collection */
+	(float) 0.0,			/* ItalicAngle */
+	pdf_true,			/* isFixedPitch */
+	-160,				/* llx */
+	-249,				/* lly */
+	1015,				/* urx */
+	1071,				/* ury */
+	-100,				/* UnderlinePosition */
+	50,				/* UnderlineThickness */
+	737,				/* CapHeight */
+	553,				/* xHeight */
+	752,				/* Ascender */
+	-271,				/* Descender */
+	58,				/* StdVW */
+	0,				/* StdHW */
+
+	0,				/* numOfChars */
+	(CharMetricInfo *) NULL,	/* cmi */
+	0,				/* numOfTracks */
+	(TrackKernData *) NULL,		/* tkd */
+	0,				/* numOfPairs */
+	(PairKernData *) NULL,		/* pkd */
+	0,				/* numOfComps */
+	(CompCharData *) NULL,		/* ccd */
+					/* skip character metrics array */
+},
+
+/* ---------------------------------------------------------- */
+{	"STSongStd-Light-Acro",		/* FontName */
+	cid,				/* encoding */
+	0,				/* used on current page */
+	0,				/* embed */
+	(char *) NULL,			/* font file */
+	0L,				/* Object id */
+	6L,				/* Font flags */
+	CIDFontType0,			/* font type */
+	NULL,				/* TT name */
+	(char *) NULL,			/* Native encoding */
+	cc_simplified_chinese,		/* Character collection */
+	(float) 0.0,			/* ItalicAngle */
+	pdf_true,			/* isFixedPitch */
+	-134,				/* llx */
+	-254,				/* lly */
+	1001,				/* urx */
+	905,				/* ury */
+	-100,				/* UnderlinePosition */
+	50,				/* UnderlineThickness */
+	737,				/* CapHeight */
+	535,				/* xHeight */
+	752,				/* Ascender */
+	-271,				/* Descender */
+	58,				/* StdVW */
+	0,				/* StdHW */
+
 	0,				/* numOfChars */
 	(CharMetricInfo *) NULL,	/* cmi */
 	0,				/* numOfTracks */
