@@ -1,7 +1,7 @@
 /*---------------------------------------------------------------------------*
  |              PDFlib - A library for generating PDF on the fly             |
  +---------------------------------------------------------------------------+
- | Copyright (c) 1997-2001 PDFlib GmbH and Thomas Merz. All rights reserved. |
+ | Copyright (c) 1997-2002 PDFlib GmbH and Thomas Merz. All rights reserved. |
  +---------------------------------------------------------------------------+
  |    This software is NOT in the public domain.  It can be used under two   |
  |    substantially different licensing terms:                               |
@@ -27,7 +27,7 @@
  |    however, will receive warranty and support statements in writing.      |
  *---------------------------------------------------------------------------*/
 
-/* $Id: p_color.c,v 1.29 2001/04/01 12:00:28 tm Exp $
+/* $Id: p_color.c,v 1.29.2.10 2002/06/17 14:19:49 tm Exp $
  *
  * PDFlib color routines
  *
@@ -72,6 +72,8 @@ pdf_init_cstate(PDF *p)
 static void
 pdf_setgraycolor(PDF *p, const char *type, float g)
 {
+    pdf_cstate *cs = &p->cstate[p->sl];
+
     if (g < 0.0 || g > EPSILON ) {
 	pdf_error(p, PDF_NonfatalError, 
 	    "Bogus gray value (%f) in PDF_setcolor", g);
@@ -79,15 +81,25 @@ pdf_setgraycolor(PDF *p, const char *type, float g)
     }
 
     if (!strcmp(type, "fill") || !strcmp(type, "both")) {
-	pdf_printf(p, "%f g\n", g);
-	p->cstate[p->sl].fill.cs	= DeviceGray;
-	p->cstate[p->sl].fill.val.gray	= g;
+	if (cs->fill.cs != DeviceGray || cs->fill.val.gray != g)
+	{
+	    if (PDF_GET_STATE(p) != pdf_state_document)
+		pdf_printf(p, "%f g\n", g);
+
+	    cs->fill.cs		= DeviceGray;
+	    cs->fill.val.gray	= g;
+	}
     }
 
     if (!strcmp(type, "stroke") || !strcmp(type, "both")) {
-	pdf_printf(p, "%f G\n", g);
-	p->cstate[p->sl].stroke.cs	= DeviceGray;
-	p->cstate[p->sl].stroke.val.gray= g;
+	if (cs->stroke.cs != DeviceGray || cs->stroke.val.gray != g)
+	{
+	    if (PDF_GET_STATE(p) != pdf_state_document)
+		pdf_printf(p, "%f G\n", g);
+
+	    cs->stroke.cs	= DeviceGray;
+	    cs->stroke.val.gray	= g;
+	}
     }
 }
 
@@ -141,6 +153,8 @@ PDF_setgray(PDF *p, float g)
 static void
 pdf_setrgbcolor2(PDF *p, const char *type, float red, float green, float blue)
 {
+    pdf_cstate *cs = &p->cstate[p->sl];
+
     if (red < 0.0 || red > EPSILON || green < 0.0 || green > EPSILON ||
 	blue < 0.0 || blue > EPSILON) {
 	pdf_error(p, PDF_NonfatalError, 
@@ -155,21 +169,31 @@ pdf_setrgbcolor2(PDF *p, const char *type, float red, float green, float blue)
     }
 
     if (!strcmp(type, "fill") || !strcmp(type, "both")) {
-	pdf_printf(p, "%f %f %f rg\n", red, green, blue);
+	if (cs->fill.cs != DeviceRGB || cs->fill.val.rgb.r != red ||
+	    cs->fill.val.rgb.g != green || cs->fill.val.rgb.b != blue)
+	{
+	    if (PDF_GET_STATE(p) != pdf_state_document)
+		pdf_printf(p, "%f %f %f rg\n", red, green, blue);
 
-	p->cstate[p->sl].fill.cs	= DeviceRGB;
-	p->cstate[p->sl].fill.val.rgb.r = red;
-	p->cstate[p->sl].fill.val.rgb.g = green;
-	p->cstate[p->sl].fill.val.rgb.b = blue;
+	    cs->fill.cs		= DeviceRGB;
+	    cs->fill.val.rgb.r	= red;
+	    cs->fill.val.rgb.g	= green;
+	    cs->fill.val.rgb.b	= blue;
+	}
     }
 
     if (!strcmp(type, "stroke") || !strcmp(type, "both")) {
-	pdf_printf(p, "%f %f %f RG\n", red, green, blue);
+	if (cs->stroke.cs != DeviceRGB || cs->stroke.val.rgb.r != red ||
+	    cs->stroke.val.rgb.g != green || cs->stroke.val.rgb.b != blue)
+	{
+	    if (PDF_GET_STATE(p) != pdf_state_document)
+		pdf_printf(p, "%f %f %f RG\n", red, green, blue);
 
-	p->cstate[p->sl].stroke.cs		= DeviceRGB;
-	p->cstate[p->sl].stroke.val.rgb.r	= red;
-	p->cstate[p->sl].stroke.val.rgb.g	= green;
-	p->cstate[p->sl].stroke.val.rgb.b	= blue;
+	    cs->stroke.cs		= DeviceRGB;
+	    cs->stroke.val.rgb.r	= red;
+	    cs->stroke.val.rgb.g	= green;
+	    cs->stroke.val.rgb.b	= blue;
+	}
     }
 }
 
@@ -232,6 +256,8 @@ pdf_setcmykcolor(
     float yellow,
     float black)
 {
+    pdf_cstate *cs = &p->cstate[p->sl];
+
     if (cyan < 0.0 || cyan > EPSILON || magenta < 0.0 || magenta > EPSILON ||
 	yellow < 0.0 || yellow > EPSILON || black < 0.0 || black > EPSILON) {
 	pdf_error(p, PDF_NonfatalError, 
@@ -241,23 +267,35 @@ pdf_setcmykcolor(
     }
 
     if (!strcmp(type, "fill") || !strcmp(type, "both")) {
-	pdf_printf(p, "%f %f %f %f k\n", cyan, magenta, yellow, black);
+	if (cs->fill.cs != DeviceCMYK ||
+	    cs->fill.val.cmyk.c != cyan || cs->fill.val.cmyk.m != magenta ||
+	    cs->fill.val.cmyk.y != yellow || cs->fill.val.cmyk.k != black)
+	{
+	    if (PDF_GET_STATE(p) != pdf_state_document)
+		pdf_printf(p, "%f %f %f %f k\n", cyan, magenta, yellow, black);
 
-	p->cstate[p->sl].fill.cs		= DeviceCMYK;
-	p->cstate[p->sl].fill.val.cmyk.c	= cyan;
-	p->cstate[p->sl].fill.val.cmyk.m	= magenta;
-	p->cstate[p->sl].fill.val.cmyk.y	= yellow;
-	p->cstate[p->sl].fill.val.cmyk.k	= black;
+	    cs->fill.cs		= DeviceCMYK;
+	    cs->fill.val.cmyk.c	= cyan;
+	    cs->fill.val.cmyk.m	= magenta;
+	    cs->fill.val.cmyk.y	= yellow;
+	    cs->fill.val.cmyk.k	= black;
+	}
     }
 
     if (!strcmp(type, "stroke") || !strcmp(type, "both")) {
-	pdf_printf(p, "%f %f %f %f K\n", cyan, magenta, yellow, black);
+	if (cs->stroke.cs != DeviceCMYK ||
+	    cs->stroke.val.cmyk.c != cyan || cs->stroke.val.cmyk.m != magenta ||
+	    cs->stroke.val.cmyk.y != yellow || cs->stroke.val.cmyk.k != black)
+	{
+	    if (PDF_GET_STATE(p) != pdf_state_document)
+		pdf_printf(p, "%f %f %f %f K\n", cyan, magenta, yellow, black);
 
-	p->cstate[p->sl].stroke.cs		= DeviceCMYK;
-	p->cstate[p->sl].stroke.val.cmyk.c	= cyan;
-	p->cstate[p->sl].stroke.val.cmyk.m	= magenta;
-	p->cstate[p->sl].stroke.val.cmyk.y	= yellow;
-	p->cstate[p->sl].stroke.val.cmyk.k	= black;
+	    cs->stroke.cs		= DeviceCMYK;
+	    cs->stroke.val.cmyk.c	= cyan;
+	    cs->stroke.val.cmyk.m	= magenta;
+	    cs->stroke.val.cmyk.y	= yellow;
+	    cs->stroke.val.cmyk.k	= black;
+	}
     }
 }
 
@@ -342,9 +380,7 @@ pdf_write_doc_colorspaces(PDF *p)
 
 	    if (cs->cs == PatternCS) {
 		pdf_printf(p, "[/Pattern");	/* pattern color space */
-		/* TODO: find correct spot name
-		name = cs->alternate.cs.name;
-		*/
+		name = p->colorspaces[cs->alternate.val.sep.cs].name;
 	    } else
 		name = cs->name;
 
@@ -425,13 +461,17 @@ pdf_add_colorspace(PDF *p, pdf_colorspace cs, const char *spotname)
 {
     int	slot;
 
-    /* TODO: optimize and check for known color space combinations
     if (cs == Separation)
-    for (slot = 0; slot < p->colorspaces_number; slot++) {
-	if (!strcmp(p->colorspaces[slot].name, spotname))
-	    return slot;
+    {
+	for (slot = 0; slot < p->colorspaces_number; slot++)
+	{
+	    if (p->colorspaces[slot].name != (char *) 0 &&
+		!strcmp(p->colorspaces[slot].name, spotname))
+	    {
+		return slot;
+	    }
+	}
     }
-    */
 
     slot = p->colorspaces_number;
 
@@ -469,13 +509,14 @@ PDF_makespotcolor(PDF *p, const char *spotname, int len)
 {
     static const char fn[] = "PDF_makespotcolor";
     char *quotedspotname;
+    int ret;
 
     PDF_TRACE(("%s\t(pdf[%p], \"%s\");\n", fn, (void *) p, spotname));
 
     if (PDF_SANITY_CHECK_FAILED(p))
 	return -1;
 
-    PDF_CHECK_SCOPE(p, fn, pdf_state_ppt);
+    PDF_CHECK_SCOPE(p, fn, pdf_state_ppt | pdf_state_document);
 
     if (spotname == NULL || *spotname == 0)
 	pdf_error(p, PDF_ValueError, "Empty spot color name");
@@ -492,35 +533,58 @@ PDF_makespotcolor(PDF *p, const char *spotname, int len)
     if (len > PDF_MAX_SPOTNAME)
 	pdf_error(p, PDF_ValueError, "Spot color name too long");
 
-    quotedspotname = (char *) p->malloc(p, (size_t) (2*len + 1),
+    quotedspotname = (char *) p->malloc(p, (size_t) (3*len + 1),
 			"PDF_makespotcolor");
 
+
     pdf_make_quoted_name(p, spotname, (size_t) len, quotedspotname);
 
-    return pdf_add_colorspace(p, Separation, quotedspotname);
+
+    ret = pdf_add_colorspace(p, Separation, quotedspotname);
+
+    p->free(p, quotedspotname);
+
+    return ret;
 }
 
 static void
 pdf_setspotcolor(PDF *p, const char *type, int spotcolor, float tint)
 {
+    pdf_cstate *cs = &p->cstate[p->sl];
+
     if (spotcolor < 0 || spotcolor >= p->colorspaces_number)
 	pdf_error(p, PDF_ValueError,
 	    "Invalid spot color number %d in PDF_setcolor", spotcolor);
 
     if (tint < 0.0 || tint > EPSILON )
 	pdf_error(p, PDF_ValueError,
-	    "Bogus spot color tint value (%f) in PDF_setcolor", tint);
-
+	    "Bogus spot color tint value (%f) in PDF_setcolor", tint); 
     if (!strcmp(type, "fill") || !strcmp(type, "both")) {
-	pdf_printf(p, "/CS%d cs %f scn\n", spotcolor, tint);
-	p->cstate[p->sl].fill.cs = Separation;
-	p->cstate[p->sl].fill.val.sep.tint = tint;
+	if (cs->fill.cs != Separation ||
+	    cs->fill.val.sep.cs != spotcolor ||
+	    cs->fill.val.sep.tint != tint)
+	{
+	    if (PDF_GET_STATE(p) != pdf_state_document)
+		pdf_printf(p, "/CS%d cs %f scn\n", spotcolor, tint);
+
+	    p->cstate[p->sl].fill.cs = Separation;
+	    p->cstate[p->sl].fill.val.sep.cs = spotcolor;
+	    p->cstate[p->sl].fill.val.sep.tint = tint;
+	}
     }
 
     if (!strcmp(type, "stroke") || !strcmp(type, "both")) {
-	pdf_printf(p, "/CS%d CS %f SCN\n", spotcolor, tint);
-	p->cstate[p->sl].stroke.cs = Separation;
-	p->cstate[p->sl].stroke.val.sep.tint = tint;
+	if (cs->stroke.cs != Separation ||
+	    cs->stroke.val.sep.cs != spotcolor ||
+	    cs->stroke.val.sep.tint != tint)
+	{
+	    if (PDF_GET_STATE(p) != pdf_state_document)
+		pdf_printf(p, "/CS%d CS %f SCN\n", spotcolor, tint);
+
+	    p->cstate[p->sl].stroke.cs = Separation;
+	    p->cstate[p->sl].stroke.val.sep.cs = spotcolor;
+	    p->cstate[p->sl].stroke.val.sep.tint = tint;
+	}
     }
 
     p->colorspaces[spotcolor].used_on_current_page = pdf_true;
@@ -541,7 +605,7 @@ PDF_setcolor(
     if (PDF_SANITY_CHECK_FAILED(p))
 	return;
 
-    PDF_CHECK_SCOPE(p, fn, pdf_state_ppt);
+    PDF_CHECK_SCOPE(p, fn, pdf_state_ppt | pdf_state_document);
 
     if (!fstype || !*fstype)
 	pdf_error(p, PDF_ValueError, "Missing type in PDF_setcolor");
