@@ -1,7 +1,7 @@
 /*---------------------------------------------------------------------------*
  |              PDFlib - A library for generating PDF on the fly             |
  +---------------------------------------------------------------------------+
- | Copyright (c) 1997-2001 PDFlib GmbH and Thomas Merz. All rights reserved. |
+ | Copyright (c) 1997-2002 PDFlib GmbH and Thomas Merz. All rights reserved. |
  +---------------------------------------------------------------------------+
  |    This software is NOT in the public domain.  It can be used under two   |
  |    substantially different licensing terms:                               |
@@ -27,7 +27,7 @@
  |    however, will receive warranty and support statements in writing.      |
  *---------------------------------------------------------------------------*/
 
-/* $Id: p_draw.c,v 1.15 2001/04/06 14:07:45 york Exp $
+/* $Id: p_draw.c,v 1.15.2.4 2002/05/23 14:04:38 york Exp $
  *
  * PDFlib drawing routines
  *
@@ -377,7 +377,7 @@ PDF_fill(PDF *p)
 	    break;
 
 	default:
-	    pdf_error(p, PDF_RuntimeError,
+	    pdf_error(p, PDF_SystemError,
 		"Inconsistent fill rule value in PDF_fill");
     }
 
@@ -406,7 +406,7 @@ PDF_fill_stroke(PDF *p)
 	    break;
 
 	default:
-	    pdf_error(p, PDF_RuntimeError,
+	    pdf_error(p, PDF_SystemError,
 		"Inconsistent fill rule value in PDF_fill_stroke");
     }
 
@@ -427,22 +427,22 @@ PDF_closepath_fill_stroke(PDF *p)
 
     switch (p->fillrule) {
 	case pdf_fill_winding:
-	pdf_puts(p, "b\n");
-	break;
+	    pdf_puts(p, "b\n");
+	    break;
 
-    case pdf_fill_evenodd:
-	pdf_puts(p, "b*\n");
-	break;
+	case pdf_fill_evenodd:
+	    pdf_puts(p, "b*\n");
+	    break;
 
-    default:
-	pdf_error(p, PDF_RuntimeError,
-	    "Inconsistent fill rule value in PDF_closepath_fill_stroke");
+	default:
+	    pdf_error(p, PDF_SystemError,
+		"Inconsistent fill rule value in PDF_closepath_fill_stroke");
     }
 
     pdf_end_path(p);
 }
 
-/* Path clipping operators */
+/* Path clipping operator */
 
 PDFLIB_API void PDFLIB_CALL
 PDF_clip(PDF *p)
@@ -466,7 +466,7 @@ PDF_clip(PDF *p)
 	    break;
 
 	default:
-	    pdf_error(p, PDF_RuntimeError,
+	    pdf_error(p, PDF_SystemError,
 		"Inconsistent fill rule value in PDF_clip");
 	    return;
     }
