@@ -1,7 +1,7 @@
 /*---------------------------------------------------------------------------*
  |              PDFlib - A library for generating PDF on the fly             |
  +---------------------------------------------------------------------------+
- | Copyright (c) 1997-2001 PDFlib GmbH and Thomas Merz. All rights reserved. |
+ | Copyright (c) 1997-2002 PDFlib GmbH and Thomas Merz. All rights reserved. |
  +---------------------------------------------------------------------------+
  |    This software is NOT in the public domain.  It can be used under two   |
  |    substantially different licensing terms:                               |
@@ -27,7 +27,7 @@
  |    however, will receive warranty and support statements in writing.      |
  *---------------------------------------------------------------------------*/
 
-/* $Id: p_font.c,v 1.42.10.3 2001/05/14 16:05:20 rjs Exp $
+/* $Id: p_font.c,v 1.42.10.25 2002/06/14 16:28:38 tm Exp $
  *
  * PDFlib font handling routines
  *
@@ -42,6 +42,7 @@
 #include "p_cid.h"
 #include "p_encoding.h"
 
+
 /*
  * Include files with more built-in font metrics can easily be added here.
  */
@@ -359,6 +360,13 @@ pdf_get_metrics_cid(
     if (cmap == NUMBER_OF_CMAPS)
 	return pdf_false;
 
+    /* Check whether this CMap is supported in the desired PDF version */
+    if (cmaps[cmap].compatibility > p->compatibility) {
+	pdf_error(p, PDF_RuntimeError,
+	    "CMap %s is not supported in PDF %d.%d", enc,
+	    p->compatibility/10, p->compatibility % 10);
+    }
+
     /* Check whether the font name is among the known CID fonts */
     for (slot = 0; slot < SIZEOF_CID_METRICS; slot++) {
 	if (!strcmp(pdf_cid_metrics[slot].name, fontname))
@@ -450,9 +458,7 @@ PDF_findfont(PDF *p, const char *fontname, const char *encoding, int embed)
     } else {
 	if (!strcmp(encoding, "host")) {
 	/* special name for the platform-specific host encoding */
-#if (defined(AS400) || defined(OS390)) && defined(PDF_HOSTENCODING_SUPPORTED)
-	/* keep "host" encoding as-is */
-#elif defined(PDFLIB_EBCDIC)
+#if defined(PDFLIB_EBCDIC)
 	encoding = "ebcdic";
 #elif defined(MAC)
 	encoding = "macroman";
@@ -565,6 +571,7 @@ PDF_findfont(PDF *p, const char *fontname, const char *encoding, int embed)
 	if (pdf_get_metrics_core(p, &p->fonts[slot], mastername, enc))
 	    break;
 
+    
 	/* Check available AFM files (dynamically loaded) */
 	filename = pdf_find_resource(p, "FontAFM", mastername, pdf_false);
 	if (filename != NULL && pdf_get_metrics_afm(p, &p->fonts[slot],
@@ -579,12 +586,12 @@ PDF_findfont(PDF *p, const char *fontname, const char *encoding, int embed)
 
 #ifdef PDF_TRUETYPE_SUPPORTED
 
-#if defined(WIN32) && defined(PDF_HOSTFONTS_SUPPORTED)
+#if defined(PDF_HOSTFONTS_SUPPORTED)
 	/* Check available TrueType fonts (installed on the host) */
 	if (!p->debug['h'] &&
 	    pdf_get_metrics_tt(p, &p->fonts[slot], mastername, enc, NULL,embed))
 	    break;
-#endif /* WIN32 */
+#endif /* PDF_HOSTFONTS_SUPPORTED */
 
 	/* Check available TrueType files (dynamically loaded) */
 	filename = pdf_find_resource(p, "FontOutline", mastername, pdf_false);
@@ -639,13 +646,15 @@ PDF_findfont(PDF *p, const char *fontname, const char *encoding, int embed)
     if (embed) {
 	if (p->fonts[slot].type != Type1 &&
 	    p->fonts[slot].type != MMType1 &&
-	    p->fonts[slot].type != TrueType)
+	    p->fonts[slot].type != TrueType &&
+	    p->fonts[slot].type != Type1C)
 		pdf_error(p, PDF_SystemError, "Bad font type in PDF_findfont");
 
 	/* search for file name if no host font available */
-	if (!p->fonts[slot].img)
+	if (!p->fonts[slot].img) {
 	    filename =
 		pdf_find_resource(p, "FontOutline", mastername, pdf_true);
+	}
 
 	/* font file not configured, and no host font available */
 	if (filename == NULL && !p->fonts[slot].img) {
@@ -732,10 +741,10 @@ PFA_data_fill(PDF *p, PDF_data_source *src)
 
     len = (int) strlen(s);
 
-    if (s[len-1] == '\n' && s[len-2] == '\r') {
+    if (len >= 2 && s[len-1] == '\n' && s[len-2] == '\r') {
 	s[len-2] = '\n';		/* adjust DOS-style line ends */
 	len--;
-    } else if (s[len-1] == '\r') {
+    } else if (len >= 1 && s[len-1] == '\r') {
 	s[len-1] = '\n';		/* adjust Mac-style line ends */
     }
 
@@ -754,9 +763,6 @@ PFA_data_fill(PDF *p, PDF_data_source *src)
 	case ascii:
 	    t1_private->length1 += (size_t) len;
 	    src->bytes_available = (size_t) len;
-#ifdef PDFLIB_EBCDIC
-	    pdf_make_ascii_len((char *) src->next_byte, src->bytes_available);
-#endif
 	    break;
 
 	case encrypted:
@@ -899,6 +905,7 @@ t1data_terminate(PDF *p, PDF_data_source *src)
     p->free(p, (void *) src->buffer_start);
 }
 
+
 void
 pdf_write_encoding(PDF *p, pdf_encodingvector *encoding, id encoding_id)
 {
@@ -928,13 +935,16 @@ pdf_make_fontflag(PDF *p, pdf_font *font)
 {
     (void) p;
 
-    if (font->type != Type1 && font->type != MMType1 && font->type != TrueType)
+    if (font->type != Type1 && font->type != MMType1 &&
+	font->type != TrueType && font->type != Type1C)
 	return;
 
     if (font->isFixedPitch)
 	font->flags |= FIXEDWIDTH;
 
-    if (!strcmp(font->encodingScheme, "AdobeStandardEncoding"))
+    if (!strcmp(font->encodingScheme, "AdobeStandardEncoding") ||
+	font->encoding == winansi || font->encoding == macroman ||
+	font->encoding == ebcdic)
 	font->flags |= ADOBESTANDARD;
     else
 	font->flags |= SYMBOL;
@@ -959,9 +969,6 @@ pdf_write_fontdescriptor(
     id fontdescriptor_id,
     id fontfile_id)
 {
-#ifdef PDFLIB_EBCDIC    
-	char afontname[128];
-#endif
 
     /* write the font descriptor with a preallocated id */
 
@@ -999,6 +1006,8 @@ pdf_write_fontdescriptor(
 	    pdf_printf(p, "/FontFile %ld 0 R\n", fontfile_id);
 	else if (font->type == TrueType)
 	    pdf_printf(p, "/FontFile2 %ld 0 R\n", fontfile_id);
+	else if (font->type == Type1C)
+	    pdf_printf(p, "/FontFile3 %ld 0 R\n", fontfile_id);
 	else
 	    pdf_error(p, PDF_SystemError,
 		"Bad font type in pdf_write_fontdescriptor");
@@ -1008,10 +1017,13 @@ pdf_write_fontdescriptor(
     pdf_end_obj(p);				/* font descriptor */
 }
 
+static PDF_data_source *pdf_make_t1src(PDF *p,
+    pdf_font *font, PDF_data_source *t1src);
+static void pdf_close_t1src(PDF *p, PDF_data_source *t1src);
+
 static void
 pdf_put_t1font(PDF *p, int slot)
 {
-    FILE	*fontfile = NULL;
     int		i, j;
     id		encoding_id = BAD_ID, fontdescriptor_id, fontfile_id;
     id		length_id, length1_id, length2_id, length3_id;
@@ -1058,7 +1070,7 @@ pdf_put_t1font(PDF *p, int slot)
 
     /* check for font file */
     if (font->embed == 1) {
-	if ((fontfile = fopen(font->fontfilename, READMODE)) == NULL)
+	if (pdf_make_t1src(p, font, &t1src) == NULL)
 	    pdf_error(p, PDF_IOError, "Couldn't open font file '%s'",
 	    			font->fontfilename);
     }
@@ -1097,7 +1109,15 @@ pdf_put_t1font(PDF *p, int slot)
 	}
     }
 
-    pdf_printf(p, "/BaseFont/%s\n", font->name);
+    pdf_puts(p, "/BaseFont/");
+#ifndef PDFLIB_EBCDIC
+    pdf_quote_name(p, font->name, strlen(font->name));
+#else
+    afontname = pdf_strdup(p, font->name);
+    pdf_make_ascii(afontname);
+    pdf_quote_name(p, afontname, strlen(afontname));
+#endif
+    pdf_puts(p, "\n");
 
     fontdescriptor_id = pdf_alloc_id(p);
     pdf_printf(p, "/FontDescriptor %ld 0 R\n", fontdescriptor_id);
@@ -1117,7 +1137,6 @@ pdf_put_t1font(PDF *p, int slot)
 
     /* font files */
     if (font->embed == 1) {
-	unsigned char magic;
 
 	pdf_begin_obj(p, fontfile_id);
 	pdf_begin_dict(p);
@@ -1140,17 +1159,6 @@ pdf_put_t1font(PDF *p, int slot)
 	pdf_begin_stream(p);			/* font data stream */
 	stream_start = pdf_tell(p);
 
-	magic = (unsigned char) getc(fontfile);
-	(void) ungetc(magic, fontfile);
-
-	t1src.init	= t1data_init;
-	t1src.fill	= (magic == PFB_MARKER ? PFB_data_fill : PFA_data_fill);
-	t1src.terminate	= t1data_terminate;
-
-	t1src.private_data = (unsigned char *)
-		p->malloc(p, sizeof(t1_private_data), "pdf_put_t1font");
-	((t1_private_data *) t1src.private_data)->fontfile = fontfile;
-
 	if (p->debug['a']) {
 	    pdf_ASCIIHexEncode(p, &t1src);
 	} else {
@@ -1185,8 +1193,7 @@ pdf_put_t1font(PDF *p, int slot)
 		((t1_private_data *) t1src.private_data)->length3);
 	pdf_end_obj(p);
 
-	fclose(((t1_private_data *) t1src.private_data)->fontfile);
-	p->free(p, (void *) t1src.private_data);
+	pdf_close_t1src(p, &t1src);
     }
 
     if (p->flush & PDF_FLUSH_CONTENT)
@@ -1249,7 +1256,8 @@ pdf_write_doc_fonts(PDF *p)
 	if (p->fonts[slot].type == Type1 || p->fonts[slot].type == MMType1)
 	    pdf_put_t1font(p, slot);
 #ifdef PDF_TRUETYPE_SUPPORTED
-	else if (p->fonts[slot].type == TrueType)
+	else if (p->fonts[slot].type == TrueType ||
+		p->fonts[slot].type == Type1C)
 	    pdf_put_ttfont(p, slot);
 #endif /* PDF_TRUETYPE_SUPPORTED */
 	else if (p->fonts[slot].type == CIDFontType0)
@@ -1307,3 +1315,76 @@ pdf_init_font_struct(PDF *p, pdf_font *font)
     font->filelen		= 0L;
     font->img			= (pdf_byte *) NULL;
 }
+
+PDFLIB_API const char * PDFLIB_CALL
+PDF_encoding_get_name(PDF *p, const char *encoding, int slot)
+{
+    static const char fn[] = "PDF_encoding_get_name";
+    int enc;
+
+    PDF_TRACE(("%s\t(pdf[%p], \"%s\", %04X);\n",
+	fn, (void *) p, encoding, slot));
+
+    if (PDF_SANITY_CHECK_FAILED(p))
+	return NULL;
+
+    PDF_CHECK_SCOPE(p, fn, pdf_state_document | pdf_state_ppt | pdf_state_path);
+
+    if (!strcmp(encoding, "builtin") || !strcmp(encoding, "host"))
+	pdf_error(p, PDF_ValueError, "Can't query encoding '%s'", encoding);
+
+    if (slot < 0 || slot > 255)
+	pdf_error(p, PDF_ValueError,
+	    "Illegal slot number %d in PDF_encoding_get_name", slot);
+
+    /* search for a registered encoding */
+    for (enc = 0; enc < ENC_MAX && p->encodings[enc] != NULL; enc++) {
+	if (p->encodings[enc]->apiname &&
+	    !strcmp(encoding, p->encodings[enc]->apiname))
+	    break;
+    }
+
+    if (enc == ENC_MAX || p->encodings[enc] == NULL)
+	pdf_error(p, PDF_ValueError,
+	    "Encoding '%s' not found in PDF_encoding_get_name", encoding); 
+    if (p->encodings[enc]->chars[slot])
+	return p->encodings[enc]->chars[slot];
+    else
+	return ".notdef";
+}
+
+
+static PDF_data_source *
+pdf_make_t1src (PDF *p, pdf_font *font, PDF_data_source *t1src)
+{
+	FILE *fontfile;
+	
+	if ((fontfile = fopen(font->fontfilename, READMODE)) == NULL) {
+	    return NULL;
+	} else {
+	    unsigned char magic;
+	    
+	    t1src->private_data = (unsigned char *)
+		    p->malloc(p, sizeof(t1_private_data), "pdf_make_t1src");
+	    ((t1_private_data *) t1src->private_data)->fontfile = fontfile;
+
+	    magic = (unsigned char) getc(fontfile);
+	    (void) ungetc(magic, fontfile);
+
+	    t1src->init	= t1data_init;
+	    t1src->fill	= (magic == PFB_MARKER ? PFB_data_fill : PFA_data_fill);
+	    t1src->terminate = t1data_terminate;
+	}
+	return t1src;
+}
+
+static void
+pdf_close_t1src(PDF *p, PDF_data_source *t1src)
+{
+
+	if (((t1_private_data *) t1src->private_data)->fontfile)
+		fclose(((t1_private_data *) t1src->private_data)->fontfile);
+
+	p->free(p, (void *) t1src->private_data);
+}
+
