@@ -1,7 +1,7 @@
 /*---------------------------------------------------------------------------*
  |              PDFlib - A library for generating PDF on the fly             |
  +---------------------------------------------------------------------------+
- | Copyright (c) 1997-2001 PDFlib GmbH and Thomas Merz. All rights reserved. |
+ | Copyright (c) 1997-2002 PDFlib GmbH and Thomas Merz. All rights reserved. |
  +---------------------------------------------------------------------------+
  |    This software is NOT in the public domain.  It can be used under two   |
  |    substantially different licensing terms:                               |
@@ -27,7 +27,7 @@
  |    however, will receive warranty and support statements in writing.      |
  *---------------------------------------------------------------------------*/
 
-/* $Id: p_gif.c,v 1.11 2001/03/29 17:53:51 tm Exp $
+/* $Id: p_gif.c,v 1.11.2.5 2002/05/23 17:55:48 tm Exp $
  *
  * GIF processing for PDFlib
  *
@@ -219,11 +219,21 @@ pdf_open_GIF_data(
     int		imageNumber = 1;	/* we read only the first image */
     pdf_image	*image;
     int		mask = -1;
+    pdf_bool	colorize = pdf_false;
 
     image = &p->images[imageslot];
 
     if (stringparam && *stringparam) {
-	if (!strcmp(stringparam, "mask"))
+        if (!strcmp(stringparam, "colorize")) {
+	    if (intparam >= p->colorspaces_number) {
+		fclose(image->fp);
+                pdf_error(p, PDF_ValueError,
+		    "Invalid color number %d for image %s", intparam, filename);
+	    }
+            image->colorspace = (pdf_colorspace) (intparam + LastCS);
+	    colorize = pdf_true;
+
+	} else if (!strcmp(stringparam, "mask"))
 	    pdf_error(p, PDF_ValueError, "Can't handle GIF image mask");
 
 	else if (!strcmp(stringparam, "masked")) {
@@ -235,7 +245,8 @@ pdf_open_GIF_data(
 		    	filename);
 
 	    if (mask >= p->images_capacity || !p->images[mask].in_use ||
-		p->images[mask].colorspace != ImageMask) {
+		p->images[mask].colorspace != ImageMask ||
+		p->images[mask].strips != 1) {
 		pdf_error(p, PDF_ValueError,
 		    "Bad image mask (no %d) for image '%s'", mask, filename);
 	    }
@@ -381,7 +392,9 @@ pdf_open_GIF_data(
 	    return -1;
 	}
 
-	image->colorspace	= Indexed;
+	if (!colorize)
+	    image->colorspace	= Indexed;
+
 	image->components	= 1;
 	image->bpc		= 8;
 	image->width		= (float) LM_to_uint(buf[4],buf[5]);
