@@ -1,7 +1,7 @@
 /*---------------------------------------------------------------------------*
  |              PDFlib - A library for generating PDF on the fly             |
  +---------------------------------------------------------------------------+
- | Copyright (c) 1997-2001 PDFlib GmbH and Thomas Merz. All rights reserved. |
+ | Copyright (c) 1997-2002 PDFlib GmbH and Thomas Merz. All rights reserved. |
  +---------------------------------------------------------------------------+
  |    This software is NOT in the public domain.  It can be used under two   |
  |    substantially different licensing terms:                               |
@@ -27,7 +27,7 @@
  |    however, will receive warranty and support statements in writing.      |
  *---------------------------------------------------------------------------*/
 
-/* $Id: p_hyper.c,v 1.15 2001/03/21 16:00:59 tm Exp $
+/* $Id: p_hyper.c,v 1.15.2.11 2002/06/17 23:34:57 tm Exp $
  *
  * PDFlib routines for hypertext stuff: bookmarks, document info, transitions
  *
@@ -37,11 +37,14 @@
 #include <string.h>
 #ifndef WINCE
 #include <time.h>
+#include "p_intern.h"
 #else
+#include "p_intern.h"
 #include <winbase.h>
 #endif
 
 #include "p_intern.h"
+#include "p_md5.h"
 
 /* We can't work with pointers in the outline objects because
  * the complete outline block may be reallocated. Therefore we use
@@ -245,10 +248,6 @@ PDF_add_bookmark(PDF *p, const char *text, int parent, int open)
     self->text		= NULL;
 
     self->text		= pdf_strdup(p, text);
-#ifdef PDFLIB_EBCDIC
-    if (!pdf_is_unicode(self->text))
-        pdf_make_ascii(self->text);
-#endif
     self->dest.page	= p->current_page;
     self->dest.type	= p->bookmark_dest;
     self->self		= pdf_alloc_id(p);
@@ -313,7 +312,8 @@ PDF_set_info(PDF *p, const char *key, const char *val)
 	return;
     }
 
-    if (!strcmp(key, "Producer") || !strcmp(key, "CreationDate")) {
+    if (!strcmp(key, "Producer") || !strcmp(key, "CreationDate") ||
+	!strcmp(key, "ModDate")) {
 	pdf_error(p, PDF_NonfatalError, "Illegal info key '%s' (ignored)", key);
 	return;
     }
@@ -322,30 +322,45 @@ PDF_set_info(PDF *p, const char *key, const char *val)
     val_buf = pdf_strdup(p, val);
     key_buf = pdf_strdup(p, key);
 
-#ifdef PDFLIB_EBCDIC
-    if (!pdf_is_unicode(val_buf))
-	pdf_make_ascii(val_buf);
-
-    /* key_buf will be converted in pdf_printf() */
-#endif
 
     if (!strcmp(key_buf, "Keywords")) {
+	if (p->Keywords)
+	    p->free(p, p->Keywords);
 	p->Keywords = val_buf;
     } else if (!strcmp(key_buf, "Subject")) {
+	if (p->Subject)
+	    p->free(p, p->Subject);
 	p->Subject = val_buf;
     } else if (!strcmp(key_buf, "Title")) {
+	if (p->Title)
+	    p->free(p, p->Title);
 	p->Title = val_buf;
     } else if (!strcmp(key_buf, "Creator")) {
+	if (p->Creator)
+	    p->free(p, p->Creator);
 	p->Creator = val_buf;
     } else if (!strcmp(key_buf, "Author")) {
+	if (p->Author)
+	    p->free(p, p->Author);
 	p->Author = val_buf;
     } else { /* user-defined keyword */
+	/* special handling required for "Trapped" */
+	if (!strcmp(key_buf, "Trapped")) {
+	    if (strcmp(val, "True") &&
+		strcmp(val, "False") &&
+		strcmp(val, "Unknown")) {
+		    p->free(p, val_buf);
+		    p->free(p, key_buf);
+		    pdf_error(p, PDF_RuntimeError,
+			"Bad value '%s' for Trapped info key", val);
+	    }
+	}
 	/* check for invalid characters in the key name */
 	if (strpbrk(key_buf, " %()<>[]{}/#") != (char *) 0) {
 	    p->free(p, val_buf);
 	    p->free(p, key_buf);
 	    pdf_error(p, PDF_ValueError,
-		"Illegal character in custom info key '%s'", key_buf);
+		"Illegal character in custom info key '%s'", key);
 	}
 
 	newentry = (pdf_info *) p->malloc(p, sizeof(pdf_info), "PDF_set_info");
@@ -372,6 +387,7 @@ pdf_write_info(PDF *p)
     SYSTEMTIME  st;
 #endif
     pdf_info	*info;
+    char time_str[50];
 
     pdf_begin_obj(p, p->info_id);	/* Info object */
 
@@ -387,49 +403,70 @@ pdf_write_info(PDF *p)
 	pdf_puts(p, "/Keywords ");
 	pdf_quote_string(p, p->Keywords);
 	pdf_putc(p, PDF_NEWLINE);
+	MD5_Update(p->md5,
+	    (unsigned char *) p->Keywords, pdf_strlen(p->Keywords));
     }
     if (p->Subject) {
 	pdf_puts(p, "/Subject ");
 	pdf_quote_string(p, p->Subject);
 	pdf_putc(p, PDF_NEWLINE);
+	MD5_Update(p->md5,
+	    (unsigned char *) p->Subject, pdf_strlen(p->Subject));
     }
     if (p->Title) {
 	pdf_puts(p, "/Title ");
 	pdf_quote_string(p, p->Title);
 	pdf_putc(p, PDF_NEWLINE);
+	MD5_Update(p->md5,
+	    (unsigned char *) p->Title, pdf_strlen(p->Title));
     }
     if (p->Creator) {
 	pdf_puts(p, "/Creator ");
 	pdf_quote_string(p, p->Creator);
 	pdf_putc(p, PDF_NEWLINE);
+	MD5_Update(p->md5,
+	    (unsigned char *) p->Creator, pdf_strlen(p->Creator));
     }
     if (p->Author) {
 	pdf_puts(p, "/Author ");
 	pdf_quote_string(p, p->Author);
 	pdf_putc(p, PDF_NEWLINE);
+	MD5_Update(p->md5,
+	    (unsigned char *) p->Author, pdf_strlen(p->Author));
     }
     if (p->userinfo) {
 	for (info = p->userinfo; info != NULL; info = info->next) {
 	    pdf_printf(p, "/%s ", info->key);
-	    pdf_quote_string(p, info->value);
+
+	    if (strcmp(info->key, "Trapped")) {
+		pdf_quote_string(p, info->value);
+	    } else {
+		/* "Trapped" requires special handling since it's a name */
+		pdf_putc(p, PDF_SLASH);
+		pdf_quote_name(p, info->value, pdf_strlen(info->value) - 1);
+	    }
+
 	    pdf_putc(p, PDF_NEWLINE);
+	    MD5_Update(p->md5,
+		(unsigned char *) info->value, pdf_strlen(info->value));
 	}
     }
 
 #ifndef WINCE
     time(&timer);
     ltime = *localtime(&timer);
-    pdf_printf(p, "/CreationDate (D:%04d%02d%02d%02d%02d%02d)\n",
+    sprintf(time_str, "%04d%02d%02d%02d%02d%02d",
 	    ltime.tm_year + 1900, ltime.tm_mon + 1, ltime.tm_mday,
 	    ltime.tm_hour, ltime.tm_min, ltime.tm_sec);
-
 #else
-
-   GetLocalTime (&st);
-   pdf_printf(p, "/CreationDate (D:%04d%02d%02d%02d%02d%02d)\n",
-	     st.wYear, st.wMonth, st.wDay, st.wHour, st.wMinute, st.wSecond);
+    GetLocalTime (&st);
+    sprintf(time_str, "%04d%02d%02d%02d%02d%02d",
+	    st.wYear, st.wMonth, st.wDay, st.wHour, st.wMinute, st.wSecond);
 #endif	/* !WINCE */
 
+    MD5_Update(p->md5, (unsigned char *) time_str, strlen(time_str));
+    pdf_printf(p, "/CreationDate (D:%s)\n", time_str);
+
     /*
      * If you change the /Producer entry your license to use
      * PDFlib will be void!
