@@ -1,7 +1,7 @@
 /*---------------------------------------------------------------------------*
  |              PDFlib - A library for generating PDF on the fly             |
  +---------------------------------------------------------------------------+
- | Copyright (c) 1997-2001 PDFlib GmbH and Thomas Merz. All rights reserved. |
+ | Copyright (c) 1997-2002 PDFlib GmbH and Thomas Merz. All rights reserved. |
  +---------------------------------------------------------------------------+
  |    This software is NOT in the public domain.  It can be used under two   |
  |    substantially different licensing terms:                               |
@@ -27,7 +27,7 @@
  |    however, will receive warranty and support statements in writing.      |
  *---------------------------------------------------------------------------*/
 
-/* $Id: p_intern.h,v 1.62.10.5 2001/05/15 14:53:33 york Exp $
+/* $Id: p_intern.h,v 1.62.10.25 2002/06/14 16:28:38 tm Exp $
  *
  * PDFlib internal definitions
  *
@@ -64,17 +64,19 @@
 /* TrueType font support */
 #define PDF_TRUETYPE_SUPPORTED
 
-/* Support for host fonts (currently only TrueType fonts on Windows).
- * Requires PDF_TRUETYPE_SUPPORTED and WIN32.
-*/
+/* Support for host fonts */
+#if defined(WIN32) || ((defined(MAC) || defined(MACOSX)) && defined(HAVE_PDI))
 #define PDF_HOSTFONTS_SUPPORTED
+#endif
 
 /* Support for pulling encodings from the host system */
 #define PDF_HOSTENCODING_SUPPORTED
 
 /* ----------------------------------- macros ------------------------------- */
 
+#define PDFLIBSERIAL		"PDFLIBSERIAL"		/* name of env var. */
 #define RESOURCEFILE		"PDFLIBRESOURCE"	/* name of env var. */
+
 #ifndef MVS
 #define DEFAULTRESOURCEFILE	"pdflib.upr"
 #else
@@ -105,7 +107,7 @@
 #define PATTERN_CHUNKSIZE	4		/* pattern */
 
 /* The following shouldn't require any changes */
-#define ENC_MAX			32		/* max number of encodings */
+#define ENC_MAX			128		/* max number of encodings */
 #define PDF_MAX_SAVE_LEVEL	10		/* max number of save levels */
 #define PDF_FILENAMELEN		1024		/* maximum file name length */
 #define PDF_MAX_PARAMSTRING	128		/* image parameter string */
@@ -186,6 +188,8 @@
 #define PDF_EXCLAM		((char) 0x21)		/* ASCII '!'  */
 #define PDF_COMMA		((char) 0x2c)		/* ASCII ','  */
 #define PDF_SEMICOLON		((char) 0x3b)		/* ASCII ';'  */
+#define PDF_O			((char) 0x4f)		/* ASCII 'O'  */
+#define PDF_T			((char) 0x54)		/* ASCII 'T'  */
 #define PDF_n			((char) 0x6e)		/* ASCII 'n'  */
 #define PDF_r			((char) 0x72)		/* ASCII 'r'  */
 #define PDF_z			((char) 0x7a)		/* ASCII 'z'  */
@@ -194,11 +198,6 @@
 #define PDF_STRING_0123456789ABCDEF	\
 	"\060\061\062\063\064\065\066\067\070\071\101\102\103\104\105\106"
 
-#ifdef PDFLIB_EBCDIC
-#define pdf_make_ascii(s)		pdf_ebcdic2ascii(s)
-#define pdf_make_ascii_len(s, l)	pdf_ebcdic2ascii_len(s, l)
-#define pdf_make_ebcdic(s)		pdf_ascii2ebcdic(s)
-#endif
 
 /* The Unicode byte order mark (BOM) signals Unicode strings */
 #define PDF_BOM0		0376		/* '\xFE' */
@@ -213,7 +212,7 @@
 # ifndef DEBUG_TRACE_FILE
 #  if defined(MVS)
 #   define DEBUG_TRACE_FILE	"pdftrace"
-#  elif defined(MAC)
+#  elif defined(MAC) || defined(AS400)
 #   define DEBUG_TRACE_FILE	"PDFlib.trace"
 #  elif defined(WIN32)
 #   define DEBUG_TRACE_FILE	"/PDFlib.trace"
@@ -231,8 +230,7 @@
  * s must not be NULL.
  */
 #define pdf_is_unicode(s) \
-	(((unsigned char *)(s))[0] != 0 &&		\
-	 ((unsigned char *)(s))[0] == PDF_BOM0 &&	\
+	(((unsigned char *)(s))[0] == PDF_BOM0 &&	\
 	 ((unsigned char *)(s))[1] == PDF_BOM1)
 
 /* --------------------------- typedefs and enums --------------------------- */
@@ -320,6 +318,9 @@ typedef enum {
 #define PDI_DEFINED
 typedef struct PDI_s PDI;	/* The opaque PDI type */
 
+/* must be kept in sync with the box_names array in pi_pdi.c */
+typedef enum { use_art, use_bleed, use_crop, use_media, use_trim } pdf_usebox;
+
 /* Destination structure for bookmarks and links */
 typedef struct {
     pdf_desttype type;
@@ -352,8 +353,14 @@ typedef struct {
     float	startx;		/* starting x point of the subpath */
     float	starty;		/* starting y point of the subpath */
 
-    /* other general gstate parameters nyi */
+    float	lwidth;		/* line width */
+    int		lcap;		/* line cap style */
+    int		ljoin;		/* line join style */
+    float	miter;		/* line join style */
+    float	flatness;	/* line join style */
+    pdf_bool	dashed;		/* line dashing in effect */
 
+    /* LATER: rendering intent and flatness */
 } pdf_gstate;
 
 /* ------------------------ text state ---------------------------- */
@@ -455,6 +462,9 @@ struct PDF_s {
     int		pdi_capacity;		/* currently allocated size */
     int		pdi_number;		/* next available PDI number */
     pdf_bool	pdi_strict;		/* strict parsing enabled */
+    pdf_usebox	pdi_usebox;
+    char	*pdi_strbuf;		/* string buffer for pdi parms */
+    int		pdi_strbuflen;		/* current size of pdi_strbuf */
 
     /* ------------------------ resource stuff ------------------------- */
     pdf_category *resources;		/* anchor for the resource list */
@@ -545,6 +555,7 @@ struct PDF_s {
     pdf_bool	underline;		/* underline text */
     pdf_bool	overline;		/* overline text */
     pdf_bool	strikeout;		/* strikeout text */
+    pdf_bool	inheritgs;		/* inherit gstate in templates */
 
     pdf_fillrule	fillrule;	/* nonzero or evenodd fill rule */
 
@@ -561,6 +572,7 @@ struct PDF_s {
     int		chars_on_this_line;
     int		compresslevel;		/* zlib compression level */
     char	debug[256];		/* debug flags */
+    void	*md5;			/* MD5 digest context for file ID */
 
     /* the list of known encoding vectors */
     pdf_encodingvector *encodings[ENC_MAX];
@@ -578,6 +590,9 @@ struct PDF_data_source_s {
     pdf_byte		*buffer_start;
     size_t		buffer_length;
     void		*private_data;
+    long		offset;		/* start of data to read */
+    long		length;		/* length of data to read */
+    long		total;		/* total bytes read so far */
 };
 
 /* ------ Private functions for library-internal use only --------- */
@@ -625,6 +640,7 @@ void	pdf_init_gstate(PDF *p);
 void	pdf_matrix_mul(const pdf_matrix *M, pdf_matrix *N);
 pdf_bool pdf_is_identity_matrix(pdf_matrix *m);
 void	pdf_concat_raw(PDF *p, pdf_matrix *m);
+void	pdf_reset_gstate(PDF *p);
 
 #ifdef HAVE_PDI
 /* p_pdi.c */
@@ -698,19 +714,18 @@ void	pdf_init_stream(PDF *p);
 void	pdf_flush_stream(PDF *p);
 void	pdf_close_stream(PDF *p);
 
-#ifdef PDFLIB_EBCDIC
-void	pdf_ebcdic2ascii(char *s);
-void	pdf_ebcdic2ascii_len(char *s, size_t len);
-void	pdf_ascii2ebcdic(char *s);
-#endif
 
 void	pdf_compress_init(PDF *p);
 void	pdf_compress_end(PDF *p);
 int	pdf_compresslevel(PDF *p);
 
 /* p_util.c */
+#if defined(MVS) && defined(I370)
+size_t pdf_ftell(FILE *fp);
+#endif
 char   *pdf_strdup(PDF *p, const char *text);
 size_t	pdf_strlen(const char *text);
+char   *pdf_getenv(const char *name);
 void	pdf_quote_string(PDF *p, const char *string);
 void	pdf_quote_string2(PDF *p, const char *text, int len);
 void	pdf_quote_name(PDF *p, const char *text, size_t len);
