@@ -1,7 +1,7 @@
 /*---------------------------------------------------------------------------*
  |              PDFlib - A library for generating PDF on the fly             |
  +---------------------------------------------------------------------------+
- | Copyright (c) 1997-2001 PDFlib GmbH and Thomas Merz. All rights reserved. |
+ | Copyright (c) 1997-2002 PDFlib GmbH and Thomas Merz. All rights reserved. |
  +---------------------------------------------------------------------------+
  |    This software is NOT in the public domain.  It can be used under two   |
  |    substantially different licensing terms:                               |
@@ -27,7 +27,7 @@
  |    however, will receive warranty and support statements in writing.      |
  *---------------------------------------------------------------------------*/
 
-/* $Id: p_jpeg.c,v 1.10 2001/04/20 08:54:13 tm Exp $
+/* $Id: p_jpeg.c,v 1.10.10.9 2002/05/15 17:56:55 tm Exp $
  *
  * JPEG processing for PDFlib
  *
@@ -107,19 +107,19 @@ typedef enum {		/* JPEG marker codes			*/
   M_EXP   = 0xdf,	/* expand reference image(s)		*/
   
   M_APP0  = 0xe0,	/* application marker, used for JFIF	*/
-  M_APP1  = 0xe1,	/* application marker			*/
-  M_APP2  = 0xe2,	/* application marker			*/
+  M_APP1  = 0xe1,	/* application marker, used for Exif	*/
+  M_APP2  = 0xe2,	/* application marker, used for FlashPix*/
   M_APP3  = 0xe3,	/* application marker			*/
   M_APP4  = 0xe4,	/* application marker			*/
   M_APP5  = 0xe5,	/* application marker			*/
   M_APP6  = 0xe6,	/* application marker			*/
   M_APP7  = 0xe7,	/* application marker			*/
-  M_APP8  = 0xe8,	/* application marker			*/
+  M_APP8  = 0xe8,	/* application marker, used for SPIFF	*/
   M_APP9  = 0xe9,	/* application marker			*/
   M_APP10 = 0xea,	/* application marker			*/
   M_APP11 = 0xeb,	/* application marker			*/
   M_APP12 = 0xec,	/* application marker			*/
-  M_APP13 = 0xed,	/* application marker			*/
+  M_APP13 = 0xed,	/* application marker, used by Photoshop*/
   M_APP14 = 0xee,	/* application marker, used by Adobe	*/
   M_APP15 = 0xef,	/* application marker			*/
   
@@ -226,35 +226,22 @@ pdf_open_JPEG_data(
     const char *stringparam,
     int intparam)
 {
+    static const char fn[] = "pdf_open_JPEG_data";
+
     int b, c, unit;
     unsigned long i, length;
 #define APP_MAX 255
     unsigned char appstring[APP_MAX];
-    int SOF_done = pdf_false;
+    pdf_byte *app13;
+    pdf_byte *s;
     pdf_image *image;
     int mask = -1;
     pdf_bool adobeflag = pdf_false;
+    pdf_bool SOF_done = pdf_false;
+    pdf_bool colorize = pdf_false;
 
     image = &p->images[imageslot];
 
-    if (stringparam && *stringparam) {
-	if (!strcmp(stringparam, "mask")) {
-	    pdf_error(p, PDF_ValueError, "Can't handle JPEG image mask");
-
-	} else if (!strcmp(stringparam, "masked")) {
-	    mask = intparam;
-	    if (mask >= 0 &&
-		(mask >= p->images_capacity || !p->images[mask].in_use ||
-		p->images[mask].colorspace != ImageMask)) {
-		    pdf_error(p, PDF_ValueError,
-		    "Bad image mask (no %d) for image '%s'", mask, filename);
-	    }
-	}
-	else
-	    pdf_error(p, PDF_ValueError,
-	    	"Unknown parameter %s in pdf_open_JPEG", stringparam);
-    }
-
     if ((image->fp = fopen(filename, READMODE)) == NULL) {
 	if (p->debug['i'])
 	    pdf_error(p, PDF_NonfatalError,
@@ -262,7 +249,6 @@ pdf_open_JPEG_data(
 	return -1;		/* Couldn't open JPEG file */
     }
 
-    image->mask			= mask;
     image->compression		= dct;
     image->use_raw		= pdf_true;
 
@@ -271,6 +257,9 @@ pdf_open_JPEG_data(
     image->src.terminate	= pdf_data_source_JPEG_terminate;
     image->src.private_data	= (void *) image;
 
+#if defined(MVS) && defined(I370)
+    image->info.jpeg.startpos = 0L;
+#else
   /* Tommy's special trick for Macintosh JPEGs: simply skip some  */
   /* hundred bytes at the beginning of the file!		  */
   do {
@@ -306,6 +295,7 @@ pdf_open_JPEG_data(
       break;
     }
   } while (!feof(image->fp));
+#endif	/* !MVS || !I370 */
 
 #define BOGUS_LENGTH	768
   /* Heuristics: if we are that far from the start chances are
@@ -381,9 +371,9 @@ pdf_open_JPEG_data(
 	 * per JFIF spec version 1.02.
 	 */
 
-#define ASPECT_RATIO	0	/* JFIF unit byte: aspect ratio only */
-#define DOTS_PER_INCH	1	/* JFIF unit byte: dots per inch     */
-#define DOTS_PER_CM	2	/* JFIF unit byte: dots per cm       */
+#define JFIF_ASPECT_RATIO	0	/* JFIF unit byte: aspect ratio only */
+#define JFIF_DOTS_PER_INCH	1	/* JFIF unit byte: dots per inch     */
+#define JFIF_DOTS_PER_CM	2	/* JFIF unit byte: dots per cm       */
 
 #define PDF_STRING_JFIF	"\112\106\111\106"
 
@@ -400,15 +390,15 @@ pdf_open_JPEG_data(
 	  }
 
 	  switch (unit) {
-	    case DOTS_PER_INCH:
+	    case JFIF_DOTS_PER_INCH:
 	      break;
 
-	    case DOTS_PER_CM:
+	    case JFIF_DOTS_PER_CM:
 	      image->dpi_x *= (float) 2.54;
 	      image->dpi_y *= (float) 2.54;
 	      break;
 
-	    case ASPECT_RATIO:
+	    case JFIF_ASPECT_RATIO:
 	      image->dpi_x *= -1;
 	      image->dpi_y *= -1;
 	      break;
@@ -420,6 +410,74 @@ pdf_open_JPEG_data(
 
         break;
 
+#ifdef NYI
+        /* LATER: read SPIFF marker */
+      case M_APP8:				/* check for SPIFF marker */
+
+        break;
+#endif
+
+      case M_APP13:				/* check for Photoshop marker */
+	length = get_2bytes(image->fp);
+
+	/* get marker contents */
+	length -= 2;			/* account for two length bytes */
+	app13 = p->malloc(p, length, fn);
+
+	if (fread(app13, 1, length, image->fp) != length) {
+	    fclose(image->fp);
+	    p->free(p, app13);
+	    if (p->debug['i']) {
+		pdf_error(p, PDF_NonfatalError,
+		"JPEG file '%s' is damaged (Photoshop marker too short)",
+		filename);
+	    }
+	    return -1;
+	}
+
+#define PS_HEADER_LEN		14
+#define PDF_STRING_Photoshop	"\120\150\157\164\157\163\150\157\160"
+#define PDF_STRING_8BIM		"\070\102\111\115"
+#define RESOLUTION_INFO_ID	0x03ED	/* resolution info resource block */
+#define PS_FIXED_TO_FLOAT(h, l)	((float) (h) + ((float) (l)/(1<<16)))
+
+	/* Not a valid Photoshop marker */
+	if (length < 9 || strncmp(PDF_STRING_Photoshop, (char *) app13, 9)) {
+	    p->free(p, app13);
+	    break;
+	}
+
+	/* walk all image resource blocks and look for ResolutionInfo */
+	for (s = app13 + PS_HEADER_LEN; s < app13 + length; /* */) {
+	    long len;
+	    unsigned int type;
+
+	    if (strncmp((char *) s, PDF_STRING_8BIM, 4))
+		break;	/* out of sync */
+	    s += 4;	/* identifying string */
+
+	    type = (unsigned int) ((s[0]<<8) + s[1]);
+	    s += 2;	/* resource type */
+
+	    s += *s + ((*s & 1) ? 1 : 2);	/* resource name */
+
+	    len = (((((s[0]<<8) + s[1])<<8) + s[2])<<8) + s[3];
+	    s += 4;	/* Size */
+
+	    if (type == RESOLUTION_INFO_ID && len >= 16) {
+		  image->dpi_x =
+			PS_FIXED_TO_FLOAT((s[0]<<8) + s[1], (s[2]<<8) + s[3]);
+		  image->dpi_y =
+			PS_FIXED_TO_FLOAT((s[8]<<8) + s[9], (s[10]<<8) + s[11]);
+		  break;
+	    }
+
+	    s += len + ((len & 1) ? 1 : 0); 	/* Data */
+	}
+
+	p->free(p, app13);
+        break;
+
       case M_APP14:				/* check for Adobe marker */
 	length = get_2bytes(image->fp);
 
@@ -427,6 +485,8 @@ pdf_open_JPEG_data(
 	  b = getc(image->fp);
 	  if (i < APP_MAX)			/* store marker in appstring */
 	    appstring[i] = (unsigned char) b;
+	  else
+	    break;
 	}
 
 	/* 
@@ -469,49 +529,88 @@ pdf_open_JPEG_data(
     }
   }
 
-  /* do some sanity checks with the parameters */
-  if (image->height <= 0 || image->width <= 0 || image->components <= 0) {
-    fclose(image->fp);
-    if (p->debug['i'])
-	pdf_error(p, PDF_NonfatalError,
-	    "Bad image parameters in JPEG file '%s' (w=%d h=%d, colors=%d",
-	    filename, image->width, image->height, image->components);
-    return -1;
-  }
-
-  if (image->bpc != 8) {
-    fclose(image->fp);
-    if (p->debug['i'])
-	pdf_error(p, PDF_NonfatalError, 
-	    "Bad number of bits per pixel (%d) in JPEG file '%s'",
-	    image->bpc, filename);
-    return -1;
-  }
-
-  switch (image->components) {
-    case 1:
-	image->colorspace = DeviceGray;
-	break;
-
-    case 3:
-	image->colorspace = DeviceRGB;
-	break;
-
-    case 4:
-	image->colorspace = DeviceCMYK;
-	/* special handling of Photoshop-generated CMYK JPEG files */
-	if (adobeflag)
-	    image->invert = pdf_true;
-	break;
-
-    default:
+    /* do some sanity checks with the parameters */
+    if (image->height <= 0 || image->width <= 0 || image->components <= 0) {
 	fclose(image->fp);
 	if (p->debug['i'])
 	    pdf_error(p, PDF_NonfatalError,
-	    	"Unknown number of color components (%d) in JPEG file '%s'",
-		image->components, filename);
+		"Bad image parameters in JPEG file '%s' (w=%d h=%d, colors=%d",
+		filename, image->width, image->height, image->components);
 	return -1;
-  }
+    }
+
+    if (image->bpc != 8) {
+	fclose(image->fp);
+	if (p->debug['i'])
+	    pdf_error(p, PDF_NonfatalError, 
+		"Bad number of bits per pixel (%d) in JPEG file '%s'",
+		image->bpc, filename);
+	return -1;
+    }
+
+    if (stringparam && *stringparam) {
+	if (!strcmp(stringparam, "mask")) {
+	    fclose(image->fp);
+	    pdf_error(p, PDF_ValueError, "Can't handle JPEG image mask");
+
+	} else if (!strcmp(stringparam, "colorize")) {
+	    if (image->components != 1) {
+		fclose(image->fp);
+		pdf_error(p, PDF_ValueError,
+		    "Can't colorize JPEG image with more than 1 component");
+	    }
+	    if (intparam >= p->colorspaces_number) {
+		fclose(image->fp);
+                pdf_error(p, PDF_ValueError,
+		    "Invalid color number %d for image %s", intparam, filename);
+	    }
+            image->colorspace = (pdf_colorspace) (intparam + LastCS);
+	    colorize = pdf_true;
+
+	} else if (!strcmp(stringparam, "masked")) {
+	    mask = intparam;
+	    if (mask >= 0 && (mask >= p->images_capacity ||
+		!p->images[mask].in_use || p->images[mask].strips != 1 ||
+		p->images[mask].colorspace != ImageMask)) {
+		    fclose(image->fp);
+		    pdf_error(p, PDF_ValueError,
+		    "Bad image mask (no %d) for image '%s'", mask, filename);
+	    }
+	}
+	else {
+	    fclose(image->fp);
+	    pdf_error(p, PDF_ValueError,
+	    	"Unknown parameter %s in pdf_open_JPEG", stringparam);
+	}
+    }
+
+    image->mask = mask;
+
+    switch (image->components) {
+	case 1:
+	    if (!colorize)
+		image->colorspace = DeviceGray;
+	    break;
+
+	case 3:
+	    image->colorspace = DeviceRGB;
+	    break;
+
+	case 4:
+	    image->colorspace = DeviceCMYK;
+	    /* special handling of Photoshop-generated CMYK JPEG files */
+	    if (adobeflag)
+		image->invert = pdf_true;
+	    break;
+
+	default:
+	    fclose(image->fp);
+	    if (p->debug['i'])
+		pdf_error(p, PDF_NonfatalError,
+		    "Unknown number of color components (%d) in JPEG file '%s'",
+		    image->components, filename);
+	    return -1;
+    }
 
     image->in_use	= pdf_true;		/* mark slot as used */
 
