@@ -1,7 +1,7 @@
 /*---------------------------------------------------------------------------*
  |              PDFlib - A library for generating PDF on the fly             |
  +---------------------------------------------------------------------------+
- | Copyright (c) 1997-2001 PDFlib GmbH and Thomas Merz. All rights reserved. |
+ | Copyright (c) 1997-2002 PDFlib GmbH and Thomas Merz. All rights reserved. |
  +---------------------------------------------------------------------------+
  |    This software is NOT in the public domain.  It can be used under two   |
  |    substantially different licensing terms:                               |
@@ -27,7 +27,7 @@
  |    however, will receive warranty and support statements in writing.      |
  *---------------------------------------------------------------------------*/
 
-/* $Id: p_params.c,v 1.33.2.1 2001/05/14 18:03:15 york Exp $
+/* $Id: p_params.c,v 1.33.2.15 2002/05/31 09:50:29 tm Exp $
  *
  * PDFlib parameter handling
  *
@@ -146,9 +146,10 @@ PDF_set_parameter(PDF *p, const char *key, const char *value)
     switch (i)
     {
 	case PDF_PARAMETER_RESOURCEFILE:
-	    /* discard old resource file name, if any */
 	    if (p->resourcefilename)
-		p->free(p, p->resourcefilename);
+		pdf_error(p, PDF_RuntimeError,
+		    "parameter 'resourcefile' can be set only once");
+
 	    p->resourcefilename = pdf_strdup(p, value);
 	    break;
 
@@ -206,7 +207,9 @@ PDF_set_parameter(PDF *p, const char *key, const char *value)
 		if (*c == 'u') {
 		    PDF_CHECK_SET_SCOPE(p, key, pdf_state_document);
 		    fflush(p->fp);
+#ifndef WINCE
 		    setbuf(p->fp, (char *) 0);
+#endif
 		}
 	    }
 	    break;
@@ -257,7 +260,8 @@ PDF_set_parameter(PDF *p, const char *key, const char *value)
 
 	case PDF_PARAMETER_PREFIX:
 	    if (p->prefix)
-		p->free(p, p->prefix);
+		pdf_error(p, PDF_RuntimeError,
+		    "parameter 'prefix' can be set only once");
 
 	    p->prefix = pdf_strdup(p, value);
 	    break;
@@ -278,10 +282,31 @@ PDF_set_parameter(PDF *p, const char *key, const char *value)
 	    p->debug['p'] = (char) pdf_bool_value(p, key, value);
 	    break;
 
+	case PDF_PARAMETER_INHERITGSTATE:
+	    p->inheritgs = pdf_bool_value(p, key, value);
+	    break;
+
 	case PDF_PARAMETER_PDISTRICT:
 	    p->pdi_strict = pdf_bool_value(p, key, value);
 	    break;
 
+	case PDF_PARAMETER_PDIUSEBOX:
+	    if (!strcmp(value, "art")) {
+		p->pdi_usebox = use_art;
+	    } else if (!strcmp(value, "bleed")) {
+		p->pdi_usebox = use_bleed;
+	    } else if (!strcmp(value, "crop")) {
+		p->pdi_usebox = use_crop;
+	    } else if (!strcmp(value, "media")) {
+		p->pdi_usebox = use_media;
+	    } else if (!strcmp(value, "trim")) {
+		p->pdi_usebox = use_trim;
+	    } else {
+		pdf_error(p, PDF_ValueError,
+		    "Unknown 'pdiusebox' value '%s'", value);
+	    }
+	    break;
+
 	case PDF_PARAMETER_PASSTHROUGH:
 	    p->debug['P'] = (char) !pdf_bool_value(p, key, value);
 	    break;
@@ -370,6 +395,8 @@ PDF_set_parameter(PDF *p, const char *key, const char *value)
 	    p->base = pdf_strdup(p, value);
 	    break;
 
+	case PDF_PARAMETER_SERIAL:
+	    break;
 
 	default:
 	    pdf_error(p, PDF_ValueError, "Can't set parameter '%s'", key);
@@ -377,12 +404,23 @@ PDF_set_parameter(PDF *p, const char *key, const char *value)
     } /* switch */
 }
 
+static float
+pdf_value(PDF *p, const char *key, float value, int minver)
+{
+    if (p->compatibility < minver)
+	pdf_error(p, PDF_RuntimeError,
+	    "Parameter '%s' requires PDF %d.%d or higher",
+		key, minver / 10, minver % 10);
+
+    return value;
+}
+
 static float
 pdf_pos_value(PDF *p, const char *key, float value, int minver)
 {
     if (p->compatibility < minver)
 	pdf_error(p, PDF_RuntimeError,
-	    "Parameter '%s' requires PDF%d.%d or higher",
+	    "Parameter '%s' requires PDF %d.%d or higher",
 		key, minver / 10, minver % 10);
 
     if (value <= (float) 0)
@@ -436,7 +474,7 @@ PDF_set_value(PDF *p, const char *key, float value)
 	    break;
 
 	case PDF_PARAMETER_FLOATDIGITS:
-	    if (4 <= ivalue && ivalue <= 6)
+	    if (3 <= ivalue && ivalue <= 6)
 		p->floatdigits = ivalue;
 	    else
 		pdf_error(p, PDF_ValueError,
@@ -472,67 +510,67 @@ PDF_set_value(PDF *p, const char *key, float value)
 	    break;
 
 	case PDF_PARAMETER_CROPBOX_LLX:
-	    p->CropBox.llx = pdf_pos_value(p, key, value, PDF_1_2);
+	    p->CropBox.llx = pdf_value(p, key, value, PDF_1_2);
 	    break;
 
 	case PDF_PARAMETER_CROPBOX_LLY:
-	    p->CropBox.lly = pdf_pos_value(p, key, value, PDF_1_2);
+	    p->CropBox.lly = pdf_value(p, key, value, PDF_1_2);
 	    break;
 
 	case PDF_PARAMETER_CROPBOX_URX:
-	    p->CropBox.urx = pdf_pos_value(p, key, value, PDF_1_2);
+	    p->CropBox.urx = pdf_value(p, key, value, PDF_1_2);
 	    break;
 
 	case PDF_PARAMETER_CROPBOX_URY:
-	    p->CropBox.ury = pdf_pos_value(p, key, value, PDF_1_2);
+	    p->CropBox.ury = pdf_value(p, key, value, PDF_1_2);
 	    break;
 
 	case PDF_PARAMETER_BLEEDBOX_LLX:
-	    p->BleedBox.llx = pdf_pos_value(p, key, value, PDF_1_3);
+	    p->BleedBox.llx = pdf_value(p, key, value, PDF_1_3);
 	    break;
 
 	case PDF_PARAMETER_BLEEDBOX_LLY:
-	    p->BleedBox.lly = pdf_pos_value(p, key, value, PDF_1_3);
+	    p->BleedBox.lly = pdf_value(p, key, value, PDF_1_3);
 	    break;
 
 	case PDF_PARAMETER_BLEEDBOX_URX:
-	    p->BleedBox.urx = pdf_pos_value(p, key, value, PDF_1_3);
+	    p->BleedBox.urx = pdf_value(p, key, value, PDF_1_3);
 	    break;
 
 	case PDF_PARAMETER_BLEEDBOX_URY:
-	    p->BleedBox.ury = pdf_pos_value(p, key, value, PDF_1_3);
+	    p->BleedBox.ury = pdf_value(p, key, value, PDF_1_3);
 	    break;
 
 	case PDF_PARAMETER_TRIMBOX_LLX:
-	    p->TrimBox.llx = pdf_pos_value(p, key, value, PDF_1_3);
+	    p->TrimBox.llx = pdf_value(p, key, value, PDF_1_3);
 	    break;
 
 	case PDF_PARAMETER_TRIMBOX_LLY:
-	    p->TrimBox.lly = pdf_pos_value(p, key, value, PDF_1_3);
+	    p->TrimBox.lly = pdf_value(p, key, value, PDF_1_3);
 	    break;
 
 	case PDF_PARAMETER_TRIMBOX_URX:
-	    p->TrimBox.urx = pdf_pos_value(p, key, value, PDF_1_3);
+	    p->TrimBox.urx = pdf_value(p, key, value, PDF_1_3);
 	    break;
 
 	case PDF_PARAMETER_TRIMBOX_URY:
-	    p->TrimBox.ury = pdf_pos_value(p, key, value, PDF_1_3);
+	    p->TrimBox.ury = pdf_value(p, key, value, PDF_1_3);
 	    break;
 
 	case PDF_PARAMETER_ARTBOX_LLX:
-	    p->ArtBox.llx = pdf_pos_value(p, key, value, PDF_1_3);
+	    p->ArtBox.llx = pdf_value(p, key, value, PDF_1_3);
 	    break;
 
 	case PDF_PARAMETER_ARTBOX_LLY:
-	    p->ArtBox.lly = pdf_pos_value(p, key, value, PDF_1_3);
+	    p->ArtBox.lly = pdf_value(p, key, value, PDF_1_3);
 	    break;
 
 	case PDF_PARAMETER_ARTBOX_URX:
-	    p->ArtBox.urx = pdf_pos_value(p, key, value, PDF_1_3);
+	    p->ArtBox.urx = pdf_value(p, key, value, PDF_1_3);
 	    break;
 
 	case PDF_PARAMETER_ARTBOX_URY:
-	    p->ArtBox.ury = pdf_pos_value(p, key, value, PDF_1_3);
+	    p->ArtBox.ury = pdf_value(p, key, value, PDF_1_3);
 	    break;
 
 	case PDF_PARAMETER_LEADING:
@@ -572,12 +610,34 @@ PDF_set_value(PDF *p, const char *key, float value)
 PDFLIB_API float PDFLIB_CALL
 PDF_get_value(PDF *p, const char *key, float mod)
 {
-    int i;
+    int i = -1;
     int imod = (int) mod;
     float result = (float) 0;
 
-    PDF_TRACE(("PDF_get_value\t(pdf[%p], \"%s\", %f);",
-	(void *) p, key, mod));
+    PDF_TRACE(("PDF_get_value\t(pdf[%p], \"%s\", %f);", (void *) p, key, mod));
+
+    /* some parameters can be retrieved with p == 0.
+    */
+    if (key != NULL && (i = get_index(key)) != -1)
+    {
+	switch (i)
+	{
+	    case PDF_PARAMETER_MAJOR:
+		result = PDFLIB_MAJORVERSION;
+		PDF_TRACE((" [%f]\n", result));
+		return result;
+
+	    case PDF_PARAMETER_MINOR:
+		result = PDFLIB_MINORVERSION;
+		PDF_TRACE((" [%f]\n", result));
+		return result;
+
+	    case PDF_PARAMETER_REVISION:
+		result = PDFLIB_REVISION;
+		PDF_TRACE((" [%f]\n", result));
+		return result;
+	} /* switch */
+    }
 
     if (PDF_SANITY_CHECK_FAILED(p))
 	return (float) 0;
@@ -585,7 +645,7 @@ PDF_get_value(PDF *p, const char *key, float mod)
     if (key == NULL || !*key)
 	pdf_error(p, PDF_ValueError, "Tried to get NULL parameter");
 
-    if ((i = get_index(key)) == -1)
+    if (i == -1)
 	pdf_error(p, PDF_ValueError,
 	    "Tried to get unknown parameter '%s'", key);
 
@@ -597,6 +657,14 @@ PDF_get_value(PDF *p, const char *key, float mod)
 
     switch (i)
     {
+	case PDF_PARAMETER_PAGEWIDTH:
+	    result = p->width;
+	    break;
+
+	case PDF_PARAMETER_PAGEHEIGHT:
+	    result = p->height;
+	    break;
+
 	case PDF_PARAMETER_IMAGEWIDTH:
 	    if (imod < 0 || imod >= p->images_capacity
 		|| !p->images[imod].in_use) {
@@ -705,18 +773,6 @@ PDF_get_value(PDF *p, const char *key, float mod)
 	    result = (float) (p->fonts[imod].descender / (float) 1000);
 	    break;
 
-	case PDF_PARAMETER_MAJOR:
-	    result = PDFLIB_MAJORVERSION;
-	    break;
-
-	case PDF_PARAMETER_MINOR:
-	    result = PDFLIB_MINORVERSION;
-	    break;
-
-	case PDF_PARAMETER_REVISION:
-	    result = PDFLIB_REVISION;
-	    break;
-
 	default:
 	    pdf_error(p, PDF_ValueError, "Can't get parameter '%s'", key);
 	    break;
@@ -729,19 +785,41 @@ PDF_get_value(PDF *p, const char *key, float mod)
 PDFLIB_API const char * PDFLIB_CALL
 PDF_get_parameter(PDF *p, const char *key, float mod)
 {
-    int i;
+    int i = -1;
     const char *result = NULL;
 
     PDF_TRACE(("PDF_get_parameter\t(pdf[%p], \"%s\", %f);",
 	(void *) p, key, mod));
 
+    /* some parameters can be retrieved with p == 0.
+    */
+    if (key != NULL && (i = get_index(key)) != -1)
+    {
+	switch (i)
+	{
+	    case PDF_PARAMETER_VERSION:
+		result = PDFLIB_VERSIONSTRING;
+		PDF_TRACE((" [%s]\n", result));
+		return result;
+
+	    case PDF_PARAMETER_PDI:
+#ifdef HAVE_PDI
+		result = "true";
+#else
+		result = "false";
+#endif
+		PDF_TRACE((" [%s]\n", result));
+		return result;
+	} /* switch */
+    }
+
     if (PDF_SANITY_CHECK_FAILED(p))
 	return (const char *) NULL;
 
     if (key == NULL || !*key)
 	pdf_error(p, PDF_ValueError, "Tried to get NULL parameter");
 
-    if ((i = get_index(key)) == -1)
+    if (i == -1)
 	pdf_error(p, PDF_ValueError,
 	    "Tried to get unknown parameter '%s'", key);
 
@@ -773,16 +851,21 @@ PDF_get_parameter(PDF *p, const char *key, float mod)
 	    result = (p->strikeout ? "true" : "false");
 	    break;
 
-	case PDF_PARAMETER_VERSION:
-	    result = PDFLIB_VERSIONSTRING;
+	case PDF_PARAMETER_INHERITGSTATE:
+	    result = (p->inheritgs ? "true" : "false");
 	    break;
 
-	case PDF_PARAMETER_PDI:
-#ifdef HAVE_PDI
-	    result = "true";
-#else
-	    result = "false";
-#endif
+	case PDF_PARAMETER_SCOPE:
+	    switch (p->state_stack[p->state_sp]) {
+		case pdf_state_object:	result = "object";	break;
+		case pdf_state_document:result = "document";	break;
+		case pdf_state_page:	result = "page";	break;
+		case pdf_state_pattern:	result = "pattern";	break;
+		case pdf_state_template:result = "template";	break;
+		case pdf_state_path:	result = "path";	break;
+		default:		result = "(unknown)";	break;
+	    }
+
 	    break;
 
 	default:
