@@ -1,7 +1,7 @@
 /*---------------------------------------------------------------------------*
  |              PDFlib - A library for generating PDF on the fly             |
  +---------------------------------------------------------------------------+
- | Copyright (c) 1997-2001 PDFlib GmbH and Thomas Merz. All rights reserved. |
+ | Copyright (c) 1997-2002 PDFlib GmbH and Thomas Merz. All rights reserved. |
  +---------------------------------------------------------------------------+
  |    This software is NOT in the public domain.  It can be used under two   |
  |    substantially different licensing terms:                               |
@@ -27,7 +27,7 @@
  |    however, will receive warranty and support statements in writing.      |
  *---------------------------------------------------------------------------*/
 
-/* $Id: p_text.c,v 1.25.2.3 2001/05/16 16:35:31 york Exp $
+/* $Id: p_text.c,v 1.25.2.17 2002/06/11 11:34:14 york Exp $
  *
  * PDFlib text routines
  *
@@ -59,9 +59,11 @@ pdf_begin_text(PDF *p, pdf_bool setpos)
 	p->gstate[p->sl].y = (float) 0.0;
     }
 
-    /* output the matrix if values have been set before */
+    /* output the matrix. this will result in one superfluous Tm
+     * iff the first action on a new page is eg. a show_xy(0, 0).
+     */
     m = &p->tstate[p->sl].m;
-    if (setpos && !pdf_is_identity_matrix(m)) {
+    if (setpos) {
 	pdf_printf(p, "%f %f %f %f %f %f Tm\n",
 			  m->a, m->b, m->c, m->d, m->e, m->f);
     }
@@ -122,6 +124,12 @@ pdf_init_tstate(PDF *p)
 void
 pdf_set_char_spacing(PDF *p, float spacing)
 {
+    if (PDF_GET_STATE(p) == pdf_state_document)
+    {
+	p->tstate[p->sl].c = spacing;
+	return;
+    }
+
     /*
      * We take care of spacing values != 0 in the text output functions,
      * but must explicitly reset here.
@@ -139,6 +147,12 @@ pdf_set_char_spacing(PDF *p, float spacing)
 void
 pdf_set_word_spacing(PDF *p, float spacing)
 {
+    if (PDF_GET_STATE(p) == pdf_state_document)
+    {
+	p->tstate[p->sl].w = spacing;
+	return;
+    }
+
     /*
      * We take care of spacing values != 0 in the text output functions,
      * but must explicitly reset here.
@@ -158,6 +172,12 @@ pdf_set_horiz_scaling(PDF *p, float scale)
     if (scale == (float) 0.0)
 	pdf_error(p, PDF_ValueError, "Bad horizontal text scaling 0");
 	
+    if (PDF_GET_STATE(p) == pdf_state_document)
+    {
+	p->tstate[p->sl].h = scale / (float) 100.0;
+	return;
+    }
+
     if (scale == 100 * p->tstate[p->sl].h)
 	return;
 
@@ -200,6 +220,12 @@ pdf_set_text_rendering(PDF *p, int mode)
 void
 pdf_set_text_rise(PDF *p, float rise)
 {
+    if (PDF_GET_STATE(p) == pdf_state_document)
+    {
+	p->tstate[p->sl].rise = rise;
+	return;
+    }
+
     if (rise == p->tstate[p->sl].rise)
 	return;
 
@@ -223,7 +249,7 @@ float a, float b, float c, float d, float e, float f)
     if (PDF_SANITY_CHECK_FAILED(p))
 	return;
 
-    PDF_CHECK_SCOPE(p, fn, pdf_state_ppt);
+    PDF_CHECK_SCOPE(p, fn, pdf_state_ppt | pdf_state_document);
 
     p->tstate[p->sl].m.a = a;
     p->tstate[p->sl].m.b = b;
@@ -232,7 +258,8 @@ float a, float b, float c, float d, float e, float f)
     p->tstate[p->sl].m.e = e;
     p->tstate[p->sl].m.f = f;
 
-    pdf_begin_text(p, pdf_true);
+    if (PDF_GET_STATE(p) != pdf_state_document)
+	pdf_begin_text(p, pdf_true);
 }
 
 PDFLIB_API void PDFLIB_CALL
@@ -258,44 +285,21 @@ PDF_set_text_pos(PDF *p, float x, float y)
 
 /* String width calculations */
 
-PDFLIB_API float PDFLIB_CALL
-PDF_stringwidth(PDF *p, const char *text, int font, float size)
-{
-    return PDF_stringwidth2(p, text, 0, font, size);
-}
-
-PDFLIB_API float PDFLIB_CALL
-PDF_stringwidth2(PDF *p, const char *text, int len, int font, float size)
+static float
+pdf_str_width(PDF *p, const char *text, int len, int font, float size)
 {
     const char *cp;
     int i;
     float width = (float) 0.0;
 
-    PDF_TRACE(("PDF_stringwidth2\t(pdf[%p], \"%s\", %d, %d, %f);\n",
-	(void *) p, text, len, font, size));
-
-    if (PDF_SANITY_CHECK_FAILED(p))
-	return width;
-
-    PDF_CHECK_SCOPE(p, "PDF_stringwidth",
-	pdf_state_document | pdf_state_ppt | pdf_state_path);
-
-    /* Check parameters */
-    if (text == NULL || *text == '\0')
-	return width;
-
     if (len == 0)
 	len = (int) strlen(text);
 
-    if (font < 0 || font >= p->fonts_number)
-	pdf_error(p, PDF_ValueError, "Bad font descriptor %d", font);
-	
-    if (size == (float) 0.0)
-	pdf_error(p, PDF_ValueError, "Bad font size 0 in PDF_stringwidth");
-
     /* We cannot handle CID fonts */
-    if (p->fonts[font].encoding == cid)
+    if (p->fonts[font].encoding == cid) {
+	PDF_TRACE((" [%f]\n", width));
 	return width;
+    }
 
     for (i = 0, cp = text; i < len; cp++, i++) {
 	/* take word spacing parameter into account at each blank */
@@ -307,16 +311,80 @@ PDF_stringwidth2(PDF *p, const char *text, int len, int font, float size)
 		 + p->tstate[p->sl].c;
     }
 
-    /* don't take into account the last character's additional spacing */
-    width -= p->tstate[p->sl].c;
-
     /* take current text matrix and horizontal scaling factor into account */
-    width = width * p->tstate[p->sl].m.a * p->tstate[p->sl].h +
-	    p->tstate[p->sl].rise * p->tstate[p->sl].m.c;
+    width = width * p->tstate[p->sl].m.a * p->tstate[p->sl].h;
 
     return width;
 }
 
+PDFLIB_API float PDFLIB_CALL
+PDF_stringwidth(PDF *p, const char *text, int font, float size)
+{
+    static const char fn[] = "PDF_stringwidth";
+    float result;
+
+    PDF_TRACE(("%s\t(pdf[%p], \"%s\", %d, %f);\n",
+	fn, (void *) p, text, font, size));
+
+    if (PDF_SANITY_CHECK_FAILED(p)) {
+	PDF_TRACE((" [%f]\n", (float)0));
+	return 0;
+    }
+
+    PDF_CHECK_SCOPE(p, fn,
+	pdf_state_document | pdf_state_ppt | pdf_state_path);
+
+    /* Check parameters */
+    if (text == NULL || *text == '\0') {
+	PDF_TRACE((" [%f]\n", (float)0));
+	return 0;
+    }
+
+    if (font < 0 || font >= p->fonts_number)
+	pdf_error(p, PDF_ValueError, "Bad font descriptor %d in %s", font, fn);
+	
+    if (size == (float) 0.0)
+	pdf_error(p, PDF_ValueError, "Bad font size 0 in %s", fn);
+
+    result = pdf_str_width(p, text, 0, font, size);
+    PDF_TRACE((" [%f]\n", result));
+    return result;
+}
+
+PDFLIB_API float PDFLIB_CALL
+PDF_stringwidth2(PDF *p, const char *text, int len, int font, float size)
+{
+    static const char fn[] = "PDF_stringwidth2";
+    float result;
+
+    PDF_TRACE(("%s\t(pdf[%p], \"%s\", %d, %d, %f);\n",
+	fn, (void *) p, text, len, font, size));
+
+    if (PDF_SANITY_CHECK_FAILED(p)) {
+	PDF_TRACE((" [%f]\n", 0));
+	return 0;
+    }
+
+    PDF_CHECK_SCOPE(p, fn,
+	pdf_state_document | pdf_state_ppt | pdf_state_path);
+
+    /* Check parameters */
+    if (text == NULL || *text == '\0') {
+	PDF_TRACE((" [%f]\n", 0));
+	return 0;
+    }
+
+    if (font < 0 || font >= p->fonts_number)
+	pdf_error(p, PDF_ValueError, "Bad font descriptor %d in %s", font, fn);
+	
+    if (size == (float) 0.0)
+	pdf_error(p, PDF_ValueError, "Bad font size 0 in %s", fn);
+
+    result = pdf_str_width(p, text, len, font, size);
+    PDF_TRACE((" [%f]\n", result));
+    return result;
+}
+
 /* ------------------------ Text control functions ------------------------ */
 
 static void
@@ -340,7 +408,7 @@ pdf_underline(PDF *p, const char *text, int len, float x, float y)
     /* the font size may be negative, resulting in bad line width */
     linewidth = (float) fabs((double) linewidth);
 
-    length = PDF_stringwidth2(p, text, len, ts->f, ts->fs);
+    length = pdf_str_width(p, text, len, ts->f, ts->fs);
 
     if (length == (float) 0)
 	return;
@@ -383,7 +451,7 @@ pdf_overline(PDF *p, const char *text, int len, float x, float y)
 
     lineheight = ((float) p->fonts[ts->f].ascender/1000) * ts->fs;
 
-    length = PDF_stringwidth2(p, text, len, ts->f, ts->fs);
+    length = pdf_str_width(p, text, len, ts->f, ts->fs);
 
     if (length == (float) 0)
 	return;
@@ -426,7 +494,7 @@ pdf_strikeout(PDF *p, const char *text, int len, float x, float y)
 
     lineheight = ((float) p->fonts[ts->f].ascender/1000) * ts->fs;
 
-    length = PDF_stringwidth2(p, text, len, ts->f, ts->fs);
+    length = pdf_str_width(p, text, len, ts->f, ts->fs);
 
     if (length == (float) 0)
 	return;
@@ -498,6 +566,11 @@ PDF_show2(PDF *p, const char *text, int len)
 
     PDF_CHECK_SCOPE(p, "PDF_show", pdf_state_ppt);
 
+    if (p->tstate[p->sl].f == -1) {			/* no font set */
+        pdf_error(p, PDF_RuntimeError,
+                "Must set font before calling PDF_show");
+    }
+
     if (text == NULL)
 	return;
 
@@ -509,6 +582,8 @@ PDF_show2(PDF *p, const char *text, int len)
 
     if (p->underline)
 	pdf_underline(p, text, len, p->tstate[p->sl].m.e, p->tstate[p->sl].m.f);
+    if (p->overline)
+	pdf_overline(p, text, len, p->tstate[p->sl].m.e, p->tstate[p->sl].m.f);
     if (p->strikeout)
 	pdf_strikeout(p, text, len, p->tstate[p->sl].m.e, p->tstate[p->sl].m.f);
 
@@ -535,7 +610,7 @@ PDF_show2(PDF *p, const char *text, int len)
 	pdf_end_text(p);
 
     p->tstate[p->sl].m.e += 
-    	PDF_stringwidth2(p, text, len, p->tstate[p->sl].f, p->tstate[p->sl].fs);
+    	pdf_str_width(p, text, len, p->tstate[p->sl].f, p->tstate[p->sl].fs);
 }
 
 PDFLIB_API void PDFLIB_CALL
@@ -564,8 +639,16 @@ PDF_show_xy2(PDF *p, const char *text, int len, float x, float y)
 	return;
     }
 
-    if (len == 0)
+    if (len == 0) {
 	len = (int) strlen(text);
+	if (len == 0)
+	    return;
+    }
+
+    if (p->tstate[p->sl].f == -1) {			/* no font set */
+        pdf_error(p, PDF_RuntimeError,
+                "Must set font before calling PDF_show_xy");
+    }
 
     if (p->underline)
 	pdf_underline(p, text, len, x, y);
@@ -602,7 +685,7 @@ PDF_show_xy2(PDF *p, const char *text, int len, float x, float y)
 	pdf_end_text(p);
 
     p->tstate[p->sl].m.e +=
-    	PDF_stringwidth2(p, text, len, p->tstate[p->sl].f, p->tstate[p->sl].fs);
+    	pdf_str_width(p, text, len, p->tstate[p->sl].f, p->tstate[p->sl].fs);
 }
 
 PDFLIB_API void PDFLIB_CALL
@@ -631,6 +714,11 @@ PDF_continue_text2(PDF *p, const char *text, int len)
     if (len == 0)
 	len = (int) strlen(text);
 
+    if (p->tstate[p->sl].f == -1) {			/* no font set */
+        pdf_error(p, PDF_RuntimeError,
+                "Must set font before calling PDF_continue_text");
+    }
+
     if (p->underline)
 	pdf_underline(p, text, len, p->tstate[p->sl].m.e, p->tstate[p->sl].m.f);
     if (p->overline)
@@ -677,7 +765,7 @@ PDF_continue_text2(PDF *p, const char *text, int len)
 	pdf_end_text(p);
 
     p->tstate[p->sl].m.e +=
-    	PDF_stringwidth2(p, text, len, p->tstate[p->sl].f, p->tstate[p->sl].fs);
+    	pdf_str_width(p, text, len, p->tstate[p->sl].f, p->tstate[p->sl].fs);
 }
 
 /* ----------------------- Text formatting routines ------------------------ */
@@ -687,13 +775,21 @@ typedef enum { pdf_align_left, pdf_align_right, pdf_align_center,
 	pdf_align_justify, pdf_align_fulljustify
 } pdf_alignment;
 
+/* this helper function returns the width of the null-terminated string
+** 'text' for the current font and size EXCLUDING the last character's
+** additional charspacing.
+*/
+static float
+pdf_swidth(PDF *p, const char *text)
+{
+    pdf_tstate	*ts = &p->tstate[p->sl];
+
+    return pdf_str_width(p, text, 0, ts->f, ts->fs) - ts->c * ts->m.a * ts->h;
+}
+
 static void
 pdf_show_aligned(PDF *p, const char *text, float x, float y, pdf_alignment mode)
 {
-    pdf_tstate	*ts;
-
-    ts = &p->tstate[p->sl];
-
     switch (mode) {
 	case pdf_align_left:
 	case pdf_align_justify:
@@ -702,12 +798,11 @@ pdf_show_aligned(PDF *p, const char *text, float x, float y, pdf_alignment mode)
 	    break;
 
 	case pdf_align_right:
-	    x -=
-	    	PDF_stringwidth(p, text, ts->f, ts->fs);
+	    x -= pdf_swidth(p, text);
 	    break;
 
 	case pdf_align_center:
-	    x -= PDF_stringwidth(p, text, ts->f, ts->fs) / 2;
+	    x -= pdf_swidth(p, text) / 2;
 	    break;
     }
 
@@ -775,7 +870,7 @@ PDF_show_boxed(
 
     ts = &p->tstate[p->sl];
 
-    if (p->fonts_number == 0 || ts->f == -1) {		/* no font set */
+    if (ts->f == -1) {			/* no font set */
         pdf_error(p, PDF_RuntimeError,
                 "Must set font before calling PDF_show_boxed");
     }
@@ -851,11 +946,11 @@ PDF_show_boxed(
 		
 		/* check whether the line is too long */
 		ts->w = (float) 0.0;
-		textwidth = PDF_stringwidth(p, linebuf, ts->f, ts->fs);
+		textwidth = pdf_swidth(p, linebuf);
 
 		/* the forced break occurs too late for this line */
 		if (textwidth > width) {
-		    if (wordBreakCount == 1) {	/* no blank found */
+		    if (wordBreakCount == 0) {	/* no blank found */
 			prematureexit = pdf_true;
 			break;
 		    }
@@ -865,12 +960,13 @@ PDF_show_boxed(
 		    curTextPos -= (curCharsInLine - lastWordBreak);
 
 		    if (!blind) {
-			textwidth = PDF_stringwidth(p, linebuf, ts->f, ts->fs);
-			if (mode == pdf_align_justify ||
-			    mode == pdf_align_fulljustify) {
+			textwidth = pdf_swidth(p, linebuf);
+			if (wordBreakCount != 1 &&
+				(mode == pdf_align_justify ||
+				mode == pdf_align_fulljustify)) {
 			    pdf_set_word_spacing(p,
 				(width - textwidth) /
-				(wordBreakCount * ts->h * ts->m.a));
+				((wordBreakCount - 1) * ts->h * ts->m.a));
 			}
 			else
 			    pdf_set_word_spacing(p, (float) 0.0);
@@ -879,7 +975,7 @@ PDF_show_boxed(
 
 		} else if (!blind) {
 
-		    if (mode == pdf_align_fulljustify && wordBreakCount > 1) {
+		    if (mode == pdf_align_fulljustify && wordBreakCount > 0) {
 			pdf_set_word_spacing(p,
 			    (width - textwidth) /
 			    (wordBreakCount * ts->h * ts->m.a));
@@ -899,7 +995,7 @@ PDF_show_boxed(
 		ts->w = (float) 0.0;
 
 		/* line too long ==> break at last blank */
-		if (PDF_stringwidth(p, linebuf, ts->f, ts->fs) > width) {
+		if (pdf_swidth(p, linebuf) > width) {
 		    cury -= ts->l;		/* adjust cury by leading */
 
 		    if (cury < bottom) {
@@ -916,22 +1012,25 @@ PDF_show_boxed(
 		    /* LATER: * force break if wordBreakCount == 1,
 		     * i.e., no blank
 		     */
-		    if (wordBreakCount == 1) {
+		    if (wordBreakCount == 0) {
 			prematureexit = pdf_true;
 			break;
 		    }
 
 		    /* adjust word spacing for full justify */
-		    if (mode == pdf_align_justify ||
-			mode == pdf_align_fulljustify) {
+		    if (wordBreakCount != 1 && (mode == pdf_align_justify ||
+					    mode == pdf_align_fulljustify)) {
 			ts->w = (float) 0.0;
-			textwidth = PDF_stringwidth(p, linebuf, ts->f, ts->fs);
+			textwidth = pdf_swidth(p, linebuf);
 			if (!blind) {
 			    pdf_set_word_spacing(p,
 				(width - textwidth) /
 				((wordBreakCount - 1) * ts->h * ts->m.a));
 			}
 		    }
+		    else if (!blind) {
+			pdf_set_word_spacing(p, (float) 0.0);
+		    }
 
 		    lastdone = curTextPos;
 		    if (!blind)
@@ -968,18 +1067,24 @@ PDF_show_boxed(
 	    
 	    /* check if the last line is too long */
 	    ts->w = (float) 0.0;
-	    textwidth = PDF_stringwidth(p, linebuf, ts->f, ts->fs);
+	    textwidth = pdf_swidth(p, linebuf);
 
 	    if (textwidth > width) {
+		if (wordBreakCount == 0)
+		{
+		    prematureexit = pdf_true;
+		    break;
+		}
+
 		linebuf[lastWordBreak] = 0;	/* terminate at last blank */
 		curTextPos -= (curCharsInLine - lastWordBreak - 1);
 
 		/* recalculate the width */
-		textwidth = PDF_stringwidth(p, linebuf, ts->f, ts->fs);
+		textwidth = pdf_swidth(p, linebuf);
 
 		/* adjust word spacing for full justify */
-		if (mode == pdf_align_justify ||
-		    mode == pdf_align_fulljustify) {
+		if (wordBreakCount != 1 && (mode == pdf_align_justify ||
+					    mode == pdf_align_fulljustify)) {
 		    if (!blind) {
 			pdf_set_word_spacing(p,
 			(width - textwidth) /
@@ -1009,6 +1114,10 @@ PDF_show_boxed(
 	pdf_set_word_spacing(p, old_word_spacing);
 
     /* return number of remaining characters */
+
+    while (text[lastdone] == PDF_SPACE)
+	++lastdone;
+
     if ((text[lastdone] == PDF_RETURN ||
 	text[lastdone] == PDF_NEWLINE) && text[lastdone+1] == 0)
 	    ++lastdone;
