@@ -1,7 +1,7 @@
 /*---------------------------------------------------------------------------*
  |              PDFlib - A library for generating PDF on the fly             |
  +---------------------------------------------------------------------------+
- | Copyright (c) 1997-2001 PDFlib GmbH and Thomas Merz. All rights reserved. |
+ | Copyright (c) 1997-2002 PDFlib GmbH and Thomas Merz. All rights reserved. |
  +---------------------------------------------------------------------------+
  |    This software is NOT in the public domain.  It can be used under two   |
  |    substantially different licensing terms:                               |
@@ -27,7 +27,7 @@
  |    however, will receive warranty and support statements in writing.      |
  *---------------------------------------------------------------------------*/
 
-/* $Id: p_tiff.c,v 1.17 2001/04/05 14:17:30 tm Exp $
+/* $Id: p_tiff.c,v 1.17.2.9 2002/05/31 10:28:18 tm Exp $
  *
  * TIFF processing for PDFlib
  *
@@ -230,6 +230,8 @@ pdf_open_TIFF_data(
     pdf_image *image;
     int strip, mask = -1;
     float res_x, res_y;
+    pdf_bool is_mask = pdf_false;
+    pdf_bool colorize = pdf_false;
 
     image = &p->images[imageslot];
 
@@ -247,7 +249,8 @@ pdf_open_TIFF_data(
     if (!image->info.tiff.tif) {
 	if (p->debug['i']) {
 	    pdf_error(p, PDF_NonfatalError,
-		"Unknown tag structure in TIFF file '%s'", filename);
+	    "Couldn't open TIFF file '%s' (file not found or bad structure)",
+	    filename);
 	}
 	return -1;
     }
@@ -447,8 +450,6 @@ pdf_open_TIFF_data(
 		break;
 	    }
 
-	    image->strips = (int) TIFFNumberOfStrips(image->info.tiff.tif);
-
 	    if (extra != 0) {
 		TIFFClose(image->info.tiff.tif);
 		if (p->debug['i']) {
@@ -459,11 +460,8 @@ pdf_open_TIFF_data(
 		return -1;
 	    }
 
-	    if (photometric == PHOTOMETRIC_MINISWHITE)
-		image->invert = !image->invert;
-
-	    image->compression = none;
-	    image->use_raw = pdf_true;
+	    image->use_raw = pdf_false;
+	    image->bpc = 8;
 	    break;
 
 	case COMPRESSION_LZW:
@@ -635,24 +633,56 @@ pdf_open_TIFF_data(
 	    image->bpc = 8;
     }
 
+    /* palette images are automatically converted to RGB by tifflib */
+    if (image->components == 1 && !image->use_raw &&
+	TIFFGetField(image->info.tiff.tif, TIFFTAG_COLORMAP,
+		&rmap, &gmap, &bmap)) {
+	image->components = 3;
+    }
+
     if (stringparam && *stringparam && strcmp(stringparam, "page")) {
-	if (!strcmp(stringparam, "mask")) {
+        if (!strcmp(stringparam, "colorize")) {
+            if (image->components != 1) {
+                TIFFClose(image->info.tiff.tif);
+                pdf_error(p, PDF_ValueError,
+		    "Can't colorize TIFF image with more than 1 component");
+            }
+	    if (intparam >= p->colorspaces_number) {
+                TIFFClose(image->info.tiff.tif);
+                pdf_error(p, PDF_ValueError,
+		    "Invalid color number %d for image %s", intparam, filename);
+	    }
+            image->colorspace = (pdf_colorspace) (intparam + LastCS);
+            colorize = pdf_true;
+
+	} else if (!strcmp(stringparam, "mask")) {
 	    if (image->components != 1 || image->bpc != 1) {
 		TIFFClose(image->info.tiff.tif);
 		pdf_error(p, PDF_ValueError,
-			"Can't handle TIFF image mask with more than 1 bit");
+		    "Can't handle TIFF image mask with more than 1 bit");
 	    }
+/*	    if (image->strips != 1) { */
+/*		TIFFClose(image->info.tiff.tif); */
+/*		pdf_error(p, PDF_ValueError, */
+/*		    "Can't use multistrip TIFF image as mask"); */
+/*	    } */
 	    image->colorspace = ImageMask;
+	    is_mask = pdf_true;
 
 	} else if (!strcmp(stringparam, "masked")) {
 	    mask = intparam;
-	    if (mask >= 0 &&
-		(mask >= p->images_capacity || !p->images[mask].in_use ||
+	    if (mask >= 0 && (mask >= p->images_capacity ||
+		!p->images[mask].in_use || p->images[mask].strips != 1 ||
 		p->images[mask].colorspace != ImageMask)) {
 		    TIFFClose(image->info.tiff.tif);
 		    pdf_error(p, PDF_ValueError,
 		    "Bad image mask (no %d) for image '%s'", mask, filename);
 	    }
+	    if (image->strips != 1) {
+		TIFFClose(image->info.tiff.tif);
+		pdf_error(p, PDF_ValueError,
+		    "Can't mask multistrip TIFF image");
+	    }
 	} else if (!strcmp(stringparam, "invert")) {
 	    image->invert = !image->invert;
 	} else {
@@ -666,15 +696,8 @@ pdf_open_TIFF_data(
 
     switch (image->components) {
 	case 1:
-	    /* if it is a palette image we have to adapt these accordingly */
-	    if (!image->use_raw &&
-		TIFFGetField(image->info.tiff.tif, TIFFTAG_COLORMAP,
-	    		&rmap, &gmap, &bmap)) {
-		image->colorspace	= DeviceRGB;
-		image->components	= 3;
-	    } else
-		/* plain old grayscale image */
-		image->colorspace	= DeviceGray;
+	    if (!colorize && !is_mask)
+		image->colorspace = DeviceGray;
 	    break;
 
 	case 3:
@@ -837,8 +860,8 @@ pdf_open_TIFF_data(
 	    while (linecounter < image->height) {
 		if (TIFFReadScanline(image->info.tiff.tif,
 		    (tdata_t) (image->info.tiff.raster +
-			((int)image->height - linecounter) * (int)image->width),
-			(uint32) linecounter, (tsample_t) 0) == -1) {
+		    ((int)image->height - linecounter - 1) * (int)image->width),
+		    (uint32) linecounter, (tsample_t) 0) == -1) {
 
 		    p->free(p, (void *) image->info.tiff.raster);
 		    TIFFClose(image->info.tiff.tif);
