@@ -1,7 +1,7 @@
 /*---------------------------------------------------------------------------*
  |              PDFlib - A library for generating PDF on the fly             |
  +---------------------------------------------------------------------------+
- | Copyright (c) 1997-2001 PDFlib GmbH and Thomas Merz. All rights reserved. |
+ | Copyright (c) 1997-2002 PDFlib GmbH and Thomas Merz. All rights reserved. |
  +---------------------------------------------------------------------------+
  |    This software is NOT in the public domain.  It can be used under two   |
  |    substantially different licensing terms:                               |
@@ -27,7 +27,7 @@
  |    however, will receive warranty and support statements in writing.      |
  *---------------------------------------------------------------------------*/
 
-/* $Id: p_truetype.c,v 1.33.2.2 2001/05/15 12:41:52 york Exp $
+/* $Id: p_truetype.c,v 1.33.2.44 2002/06/18 11:22:47 tm Exp $
  *
  * PDFlib TrueType handling routines
  *
@@ -45,11 +45,12 @@
 
 #ifdef PDF_TRUETYPE_SUPPORTED
 
-#if defined(WIN32) && defined(PDF_HOSTFONTS_SUPPORTED)
+#if defined(PDF_HOSTFONTS_SUPPORTED) && defined(WIN32)
 #define WIN32_LEAN_AND_MEAN
 #include <windows.h>
 #endif
 
+
 static void pdf_cleanup_tt(PDF *p, tt_file *ttf);
 
 static void
@@ -59,10 +60,10 @@ tt_error(PDF *p, tt_file *ttf, int level, const char *msg)
 
     pdf_cleanup_tt(p, ttf);	/* this will not free the filename */
     if (filename)
-	pdf_error(p, level, "reading TrueType font file '%s': %s",
+	pdf_error(p, level, "Error reading TrueType font file '%s': %s",
 	    filename, msg);
     else
-	pdf_error(p, level, "reading TrueType font: %s", msg);
+	pdf_error(p, level, "Error reading TrueType font: %s", msg);
 } /* tt_error */
 
 #define TT_ASSERT(p, ttf, cond)		\
@@ -73,7 +74,6 @@ tt_error(PDF *p, tt_file *ttf, int level, const char *msg)
 	((cond) ? (void) 0 : tt_error(p, ttf, PDF_IOError, \
 		"Corrupt TrueType font file"))
 
-
 #if	TT_INCORE
 
 static void
@@ -251,23 +251,67 @@ tt_tag2idx(tt_file *ttf, char *tag)
 } /* tt_tag2idx */
 
 static void
-tt_get_cmap0(PDF *p, tt_file *ttf, tt_cmap0 *cm0)
+tt_get_cmap0(PDF *p, tt_file *ttf, tt_cmap0_6 *cm0_6)
 {
-    cm0->length		= tt_get_ushort(p, ttf);
-    cm0->version	= tt_get_ushort(p, ttf);
+    static const char *fn = "tt_get_cmap0";
+    tt_ushort c;
+    tt_byte buf[256];
+
+    cm0_6->length	= tt_get_ushort(p, ttf);
+    cm0_6->language	= tt_get_ushort(p, ttf);
+    cm0_6->glyphIdArray = (tt_ushort *) 0;
+
+    /* These are not used in format 0 */
+    cm0_6->firstCode	= 0;
+    cm0_6->entryCount	= 256;
+
+    cm0_6->glyphIdArray = (tt_ushort *)
+	p->malloc(p, (size_t) (sizeof (tt_ushort) * 256), fn);
+
+    tt_read(p, ttf, buf, 256);
+
+    for (c = 0; c < 256; c++)
+	cm0_6->glyphIdArray[c] = (tt_ushort) buf[c];
 
-    tt_read(p, ttf, cm0->glyphIdArray, 256);
 } /* tt_get_cmap0 */
 
+static void
+tt_get_cmap6(PDF *p, tt_file *ttf, tt_cmap0_6 *cm0_6)
+{
+    static const char *fn = "tt_get_cmap6";
+    tt_ushort c, last;
+
+    cm0_6->length	= tt_get_ushort(p, ttf);
+    cm0_6->language	= tt_get_ushort(p, ttf);
+    cm0_6->firstCode	= tt_get_ushort(p, ttf);
+    cm0_6->entryCount	= tt_get_ushort(p, ttf);
+    cm0_6->glyphIdArray = (tt_ushort *) 0;
+
+    last = (tt_ushort) (cm0_6->firstCode + cm0_6->entryCount);
+
+    cm0_6->glyphIdArray = (tt_ushort *)
+	p->malloc(p, (size_t) (sizeof (tt_ushort) * last), fn);
+
+    /* default for codes outside the range specified in this table */
+    for (c = 0; c < last; c++)
+	cm0_6->glyphIdArray[c] = (tt_ushort) 0;
+
+    for (c = cm0_6->firstCode; c < last; c++)
+	cm0_6->glyphIdArray[c] = tt_get_ushort(p, ttf);
+
+} /* tt_get_cmap6 */
+
 static int
-tt_code2gidx0(PDF *p, tt_file *ttf, int code)
+tt_code2gidx0_6(PDF *p, tt_file *ttf, int code)
 {
     TT_ASSERT(p, ttf, ttf->tab_cmap != (tt_tab_cmap *) 0);
-    TT_ASSERT(p, ttf, ttf->tab_cmap->mac != (tt_cmap0 *) 0);
-    TT_ASSERT(p, ttf, 0 <= code && code <= 255);
+    TT_ASSERT(p, ttf, ttf->tab_cmap->mac != (tt_cmap0_6 *) 0);
+
+    if (ttf->tab_cmap->mac->format == 0)
+	TT_ASSERT(p, ttf, 0 <= code && code <= 255);
 
     return ttf->tab_cmap->mac->glyphIdArray[code];
-} /* tt_code2gidx0 */
+} /* tt_code2gidx0_6 */
 
 static void
 tt_get_cmap4(PDF *p, tt_file *ttf, tt_cmap4 *cm4, tt_ushort encID)
@@ -307,7 +351,7 @@ tt_get_cmap4(PDF *p, tt_file *ttf, tt_cmap4 *cm4, tt_ushort encID)
 	(tt_short *)  p->malloc(p, (size_t) (sizeof (tt_ushort) * segs), fn);
     cm4->idRangeOffs =
 	(tt_ushort *) p->malloc(p, (size_t) (sizeof (tt_ushort) * segs), fn);
-    
+
     if (cm4->numGlyphIds) {
 	cm4->glyphIdArray = (tt_ushort *)
 	    p->malloc(p, (size_t) (sizeof (tt_ushort) * cm4->numGlyphIds), fn);
@@ -376,11 +420,12 @@ tt_get_tab_cmap(PDF *p, tt_file *ttf)
     int		i;
     tt_ulong	offset;
     tt_ushort	numEncTabs;
+    tt_ushort	tableFormat;
 
     /* the instruction order is critical for cleanup after exceptions!
     */
     tp->win = (tt_cmap4 *) 0;
-    tp->mac = (tt_cmap0 *) 0;
+    tp->mac = (tt_cmap0_6 *) 0;
     ttf->tab_cmap = tp;
 
     TT_IOCHECK(p, ttf, idx != -1);
@@ -401,16 +446,24 @@ tt_get_tab_cmap(PDF *p, tt_file *ttf)
 
 	if (platformID == tt_pfid_mac && encodingID == 0)
 	{
+	    tableFormat = tt_get_ushort(p, ttf);
+
 	    /* we currently do not support cmaps
-	    ** other than format 0 on mac.
+	    ** other than format 0 and 6 for Macintosh cmaps.
 	    */
-	    if (tt_get_ushort(p, ttf) == 0)
+
+	    if (tableFormat == 0 && tp->mac == (tt_cmap0_6 *) 0)
 	    {
-		TT_IOCHECK(p, ttf, tp->mac == (tt_cmap0 *) 0);
-		tp->mac = (tt_cmap0 *) p->malloc(p, sizeof (tt_cmap0), fn);
+		tp->mac = (tt_cmap0_6 *) p->malloc(p, sizeof (tt_cmap0_6), fn);
 		tp->mac->format = 0;
 		tt_get_cmap0(p, ttf, tp->mac);
 	    }
+	    else if (tableFormat == 6 && tp->mac == (tt_cmap0_6 *) 0)
+	    {
+		tp->mac = (tt_cmap0_6 *) p->malloc(p, sizeof (tt_cmap0_6), fn);
+		tp->mac->format = 6;
+		tt_get_cmap6(p, ttf, tp->mac);
+	    }
 	}
 	else if (platformID == tt_pfid_win &&
 		(encodingID == tt_wenc_symbol || encodingID == tt_wenc_text))
@@ -423,7 +476,7 @@ tt_get_tab_cmap(PDF *p, tt_file *ttf)
 	tt_seek(p, ttf, pos);
     } /* for */
 } /* tt_get_tab_cmap */
- 
+
 static void
 tt_get_tab_head(PDF *p, tt_file *ttf)
 {
@@ -531,13 +584,20 @@ tt_get_tab_hmtx(PDF *p, tt_file *ttf)
     }
 } /* tt_get_tab_hmtx */
 
-static int
-tt_get_tab_CFF(PDF *p, tt_file *ttf)
+static void
+tt_get_tab_CFF_(PDF *p, tt_file *ttf)
 {
-    (void) p;
-    
-    return tt_tag2idx(ttf, pdf_str_CFF_);
-} /* tt_get_tab_CFF */
+    static const char *fn = "tt_get_tab_CFF_";
+    int		idx = tt_tag2idx(ttf, pdf_str_CFF_);
+
+    if (idx != -1) {
+	/* CFF table found */
+	ttf->tab_CFF_ = (tt_tab_CFF_ *) p->malloc(p, sizeof (tt_tab_CFF_), fn);
+	ttf->tab_CFF_->offset = ttf->dir[idx].offset;
+	ttf->tab_CFF_->length = ttf->dir[idx].length;
+    }
+
+} /* tt_get_tab_CFF_ */
 
 static int
 tt_gidx2width(PDF *p, tt_file *ttf, int gidx)
@@ -557,7 +617,7 @@ tt_gidx2width(PDF *p, tt_file *ttf, int gidx)
 	return (int) ROUND(
 	(ttf->tab_hmtx->metrics[gidx].advanceWidth * 1000.0f) / unitsPerEm);
 */
-	return (int) 
+	return (int)
 	((ttf->tab_hmtx->metrics[gidx].advanceWidth * 1000L) / unitsPerEm);
     }
 } /* tt_gidx2width */
@@ -621,7 +681,13 @@ tt_get_tab_name(PDF *p, tt_file *ttf)
     tt_seek(p, ttf, (long) ttf->dir[idx].offset);
 
     tp->format = tt_get_ushort(p, ttf);
-    TT_IOCHECK(p, ttf, tp->format == 0);
+
+    /* Format 0 is the only document one, but some Apple fonts use 65535.
+     * This is very consequent since it follows Microsoft's lead in
+     * disregarding one's own published specifications.
+     */
+    TT_IOCHECK(p, ttf, (tp->format == 0 || tp->format == 65535));
+
     tp->numNameRecords = tt_get_ushort(p, ttf);
     tp->offsetStrings = tt_get_ushort(p, ttf);
     offs = ttf->dir[idx].offset + tp->offsetStrings;
@@ -679,22 +745,27 @@ tt_get_tab_name(PDF *p, tt_file *ttf)
     } /* for i */
 
     /* If these checks ever fail, we can consider constructing the
-    ** missing names 4/6 from 1/2.
+    ** missing name 4 from 1+2.
     */
     TT_IOCHECK(p, ttf, tp->names[4] != (char *) 0);
-    TT_IOCHECK(p, ttf, tp->names[6] != (char *) 0);
-
 } /* tt_get_tab_name */
- 
+
 static void
 tt_get_tab_OS_2(PDF *p, tt_file *ttf)
 {
     static const char *fn = "tt_get_tab_OS_2";
-    tt_tab_OS_2 *tp = (tt_tab_OS_2 *) p->malloc(p, sizeof (tt_tab_OS_2), fn);
-    int		idx = tt_tag2idx(ttf, pdf_str_OS_2);
+    tt_tab_OS_2 *tp;
+    int idx;
 
+    idx = tt_tag2idx(ttf, pdf_str_OS_2);
+
+    /* Some Apple fonts don't have an OS/2 table */
+    if (idx == -1)
+	return;
+
+    tp = (tt_tab_OS_2 *) p->malloc(p, sizeof (tt_tab_OS_2), fn);
     ttf->tab_OS_2 = tp;
-    TT_IOCHECK(p, ttf, idx != -1);
+
     tt_seek(p, ttf, (long) ttf->dir[idx].offset);
 
     tp->version = tt_get_ushort(p, ttf);
@@ -786,7 +857,6 @@ tt_get_tab_post(PDF *p, tt_file *ttf)
 static void
 pdf_cleanup_tt(PDF *p, tt_file *ttf)
 {
-    /* Note: do not clean up ttf->img since the data belongs to font->img */
 
     if (ttf->fp != (FILE *) 0)
 	fclose(ttf->fp);
@@ -797,12 +867,16 @@ pdf_cleanup_tt(PDF *p, tt_file *ttf)
     if (ttf->tab_hhea != (tt_tab_hhea *) 0)	p->free(p, ttf->tab_hhea);
     if (ttf->tab_maxp != (tt_tab_maxp *) 0)	p->free(p, ttf->tab_maxp);
     if (ttf->tab_OS_2 != (tt_tab_OS_2 *) 0)	p->free(p, ttf->tab_OS_2);
+    if (ttf->tab_CFF_ != (tt_tab_CFF_ *) 0)	p->free(p, ttf->tab_CFF_);
     if (ttf->tab_post != (tt_tab_post *) 0)	p->free(p, ttf->tab_post);
 
     if (ttf->tab_cmap != (tt_tab_cmap *) 0)
     {
-	if (ttf->tab_cmap->mac != (tt_cmap0 *) 0)
+	if (ttf->tab_cmap->mac != (tt_cmap0_6 *) 0) {
+	    if (ttf->tab_cmap->mac->glyphIdArray)
+		p->free(p, ttf->tab_cmap->mac->glyphIdArray);
 	    p->free(p, ttf->tab_cmap->mac);
+	}
 
 	if (ttf->tab_cmap->win != (tt_cmap4 *) 0)
 	{
@@ -839,6 +913,9 @@ pdf_cleanup_tt(PDF *p, tt_file *ttf)
 	p->free(p, ttf->tab_name);
     }
 
+    /* Note: do not clean up ttf->img since the data belongs to font->img
+    */
+
     p->free(p, ttf);
 
 } /* pdf_cleanup_tt */
@@ -858,6 +935,7 @@ pdf_init_tt(PDF *p, tt_file *ttf)
     ttf->tab_name = (tt_tab_name *) 0;
     ttf->tab_post = (tt_tab_post *) 0;
     ttf->tab_OS_2 = (tt_tab_OS_2 *) 0;
+    ttf->tab_CFF_ = (tt_tab_CFF_ *) 0;
 
     (void) tt_get_ulong(p, ttf);		/* version */
 
@@ -871,13 +949,11 @@ pdf_init_tt(PDF *p, tt_file *ttf)
     for (i = 0; i < ttf->n_tables; ++i)
 	tt_get_dirent(p, ttf->dir + i, ttf);
 
-    /* check if all the tables required by Acrobat are contained in the font.
-    */
-/*  TT_IOCHECK(p, ttf, tt_tag2idx(ttf, pdf_str_cvt_) != -1);  c0d0gb12 */
-/*  TT_IOCHECK(p, ttf, tt_tag2idx(ttf, pdf_str_fpgm) != -1);  c0d0gb12 */
-    TT_IOCHECK(p, ttf, tt_tag2idx(ttf, pdf_str_glyf) != -1);
-    TT_IOCHECK(p, ttf, tt_tag2idx(ttf, pdf_str_loca) != -1);
-/*  TT_IOCHECK(p, ttf, tt_tag2idx(ttf, pdf_str_prep) != -1);  8940hg-symbol */
+    /* make sure this isn't a bitmap-only Mac font */
+    if (tt_tag2idx(ttf, pdf_str_bhed) != -1) {
+	pdf_error(p, PDF_RuntimeError,
+	    "TrueType bitmap fonts are not supported");
+    }
 
     /* These are all required TrueType tables; optional tables are not read.
     **
@@ -893,7 +969,10 @@ pdf_init_tt(PDF *p, tt_file *ttf)
     tt_get_tab_hmtx(p, ttf);	/* MUST be read AFTER hhea & maxp! */
     tt_get_tab_name(p, ttf);
     tt_get_tab_post(p, ttf);
-    tt_get_tab_OS_2(p, ttf);
+    tt_get_tab_OS_2(p, ttf);	/* may be missing from some Apple fonts */
+
+    /* this is an optional table, present only in OpenType fonts */
+    tt_get_tab_CFF_(p, ttf);
 } /* pdf_init_tt */
 
 #ifdef PDF_UNUSED
@@ -969,7 +1048,7 @@ tt_supports_cp(char *name, unsigned *cp_range)
 #endif	/* PDF_UNUSED */
 
 
-#if defined(WIN32) && defined(PDF_HOSTFONTS_SUPPORTED)
+#if defined(PDF_HOSTFONTS_SUPPORTED) && defined(WIN32)
 
 static pdf_bool
 pdf_get_hostfont(PDF *p, const char *fontname, pdf_font *font)
@@ -1002,7 +1081,7 @@ pdf_get_hostfont(PDF *p, const char *fontname, pdf_font *font)
 	}
     }
 
-    strcpy(lf.lfFaceName, fontname);
+    strcpy((char *)lf.lfFaceName, fontname);
 
     if ((hf = CreateFontIndirect(&lf)) == NULL)
 	return pdf_false;
@@ -1043,19 +1122,24 @@ pdf_get_hostfont(PDF *p, const char *fontname, pdf_font *font)
 	if (font->img[0] == 0x80 && font->img[1] == 0x01 &&
 	    strncmp(PS_START, img + 6, 4) == 0)
 	    return pdf_true;
-#endif 
-	/* The TrueType (including OpenType) "version" is always 0x00010000
+#endif
+	/* The TrueType (including OpenType/TT) "version" is always 0x00010000
 	*/
 	if (font->img[0] == 0x00 && font->img[1] == 0x01 &&
 	    font->img[2] == 0x00 && font->img[3] == 0x00)
 	    return pdf_true;
+
+	/* The OpenType/CFF version is always 'OTTO' */
+	if (font->img[0] == PDF_O && font->img[1] == PDF_T &&
+	    font->img[2] == PDF_T && font->img[3] == PDF_O)
+	    return pdf_true;
     }
 
     p->free(p, font->img);
     return pdf_false;
-} /* pdf_get_hostfont */
+} /* pdf_get_hostfont - WIN32 */
 
-#endif /* WIN32 && PDF_HOSTFONTS_SUPPORTED */
+#endif /* PDF_HOSTFONTS_SUPPORTED && WIN32 */
 
 
 static pdf_bool
@@ -1071,10 +1155,10 @@ pdf_parse_tt(
     int		i;
     float	w;
     tt_cmap4	*win;
-    tt_cmap0	*mac;
+    tt_cmap0_6	*mac;
     tt_file	*ttf;
 
-#if !(defined(WIN32) && defined(PDF_HOSTFONTS_SUPPORTED))
+#if !defined(PDF_HOSTFONTS_SUPPORTED)
     if(!filename) 			/* host font requested... */
 	return pdf_false;		/* ...but no support for host fonts */
 #endif
@@ -1099,12 +1183,16 @@ pdf_parse_tt(
 		return pdf_false;
 	}
 
+#if defined(MVS) && defined(I370)
+	font->filelen = pdf_ftell(ttf->fp);
+#else
 	fseek(ttf->fp, 0L, SEEK_END);
 	font->filelen = (size_t) ftell(ttf->fp);
+#endif
 	fseek(ttf->fp, 0L, SEEK_SET);
 
 #if TT_INCORE
-	ttf->img = (tt_byte *) p->malloc(p, font->filelen, fn);
+	font->img = ttf->img = (tt_byte *) p->malloc(p, font->filelen, fn);
 	ttf->pos = ttf->img;
 	ttf->end = ttf->img + font->filelen;
 
@@ -1126,7 +1214,7 @@ pdf_parse_tt(
 #endif
     }
 
-#if defined(WIN32) && defined(PDF_HOSTFONTS_SUPPORTED)
+#if defined(PDF_HOSTFONTS_SUPPORTED)
     else {				/* check host fonts */
 	if (!pdf_get_hostfont(p, fontname, font)) {
 	    p->free(p, ttf);
@@ -1137,24 +1225,30 @@ pdf_parse_tt(
 	ttf->pos = ttf->img;
 	ttf->end = ttf->img + font->filelen;
     }
-#endif /* WIN32 && PDF_HOSTFONTS_SUPPORTED */
-
+#endif /* PDF_HOSTFONTS_SUPPORTED */
 
     pdf_init_tt(p, ttf);		/* read font file */
 
-    if (tt_get_tab_CFF(p, ttf) != -1) {
-	filename = ttf->filename;
-	pdf_cleanup_tt(p, ttf);	/* this will not free the filename */
-	if (p->debug['F']) {
-	    pdf_error(p, PDF_IOError,
-		"OpenType font '%s' with PostScript outlines not supported",
-		filename);
-	} else
-	    return pdf_false;
+    if (ttf->tab_CFF_) {
+	font->type = Type1C;
+	font->cff_offset = (long) ttf->tab_CFF_->offset;
+	font->cff_length = ttf->tab_CFF_->length;
+    } else {
+	font->type = TrueType;
+    }
+
+    /* check if all the tables required by Acrobat are contained in the font.
+    */
+/*  TT_IOCHECK(p, ttf, tt_tag2idx(ttf, pdf_str_cvt_) != -1);  c0d0gb12 */
+/*  TT_IOCHECK(p, ttf, tt_tag2idx(ttf, pdf_str_fpgm) != -1);  c0d0gb12 */
+/*  TT_IOCHECK(p, ttf, tt_tag2idx(ttf, pdf_str_prep) != -1);  8940hg-symbol */
+
+    /* These tables are not present in OT fonts */
+    if (font->type != Type1C) {
+	TT_IOCHECK(p, ttf, tt_tag2idx(ttf, pdf_str_glyf) != -1);
+	TT_IOCHECK(p, ttf, tt_tag2idx(ttf, pdf_str_loca) != -1);
     }
-	
 
-    font->type		= TrueType;
     font->numOfChars	= 256;
 
     /* check whether our font is compatible with the requested encoding */
@@ -1252,7 +1346,10 @@ pdf_parse_tt(
 
 #define PDF_RESTRICTED_TT_EMBEDDING	0x02
     /* check embedding flags */
-    if (embed && ttf->tab_OS_2->fsType == PDF_RESTRICTED_TT_EMBEDDING) {
+    if (embed && ttf->tab_OS_2 &&
+	ttf->tab_OS_2->fsType == PDF_RESTRICTED_TT_EMBEDDING) {
+	if (ttf->img)
+	    p->free(p, ttf->img);
 	pdf_cleanup_tt(p, ttf);
 	if (p->debug['F']) {
 	    pdf_error(p, PDF_RuntimeError,
@@ -1264,9 +1361,9 @@ pdf_parse_tt(
 
     font->encoding	= enc;
 
-    /* The name which is exposed in Windows and at the PDFlib API... */
+    /* The name which is exposed in Windows/Mac OS and at the PDFlib API... */
 
-#if defined(WIN32) && defined(PDF_HOSTFONTS_SUPPORTED)
+#if defined(PDF_HOSTFONTS_SUPPORTED)
 
     /* we take the user-supplied name (which may be localized) for host fonts */
     if (!filename) {
@@ -1280,17 +1377,25 @@ pdf_parse_tt(
 
     font->name	= pdf_strdup(p, ttf->tab_name->names[4]);
 
-#endif
+#endif	/* !PDF_HOSTFONTS_SUPPORTED */
 
-    font->ttname	= pdf_strdup(p, ttf->tab_name->names[6]);
+    if (ttf->tab_name->names[6] == (char *) 0)
+	font->ttname	= pdf_strdup(p, ttf->tab_name->names[4]);
+    else
+	font->ttname	= pdf_strdup(p, ttf->tab_name->names[6]);
 
-#ifdef PDFLIB_EBCDIC
-    pdf_make_ebcdic(font->name);
-    pdf_make_ebcdic(font->ttname);
-#endif
 
-    w			= ttf->tab_OS_2->usWeightClass / (float) 65;
-    font->StdVW		= (int) (PDF_STEMV_MIN + w * w);
+#define PDF_MACSTYLE_BOLD	0x01
+
+    if (ttf->tab_OS_2) {
+	w			= ttf->tab_OS_2->usWeightClass / (float) 65;
+	font->StdVW		= (int) (PDF_STEMV_MIN + w * w);
+    } else {
+	if (ttf->tab_head->macStyle & PDF_MACSTYLE_BOLD)
+	    font->StdVW		= PDF_STEMV_BOLD;
+	else
+	    font->StdVW		= PDF_STEMV_NORMAL;
+    }
 
     font->italicAngle	= (float) ttf->tab_post->italicAngle;
     font->isFixedPitch	= (pdf_bool) ttf->tab_post->isFixedPitch;
@@ -1305,15 +1410,40 @@ pdf_parse_tt(
     font->underlinePosition	= PDF_TT2PDF(ttf->tab_post->underlinePosition);
     font->underlineThickness	= PDF_TT2PDF(ttf->tab_post->underlineThickness);
 
-    if (ttf->tab_OS_2->sCapHeight) {
-	font->capHeight	= PDF_TT2PDF(ttf->tab_OS_2->sCapHeight);
-    } else {
-	font->capHeight	= 666;	/* an educated guess */
-    }
+    if (ttf->tab_OS_2) {
+	/* some Apple fonts have zero values in the OS/2 table  */
+	if (ttf->tab_OS_2->sTypoAscender)
+	    font->ascender	= PDF_TT2PDF(ttf->tab_OS_2->sTypoAscender);
+	else
+	    font->ascender	= PDF_TT2PDF(ttf->tab_hhea->ascender);
 
-    font->xHeight	= PDF_TT2PDF(ttf->tab_OS_2->sxHeight);
-    font->ascender	= PDF_TT2PDF(ttf->tab_OS_2->sTypoAscender);
-    font->descender	= PDF_TT2PDF(ttf->tab_OS_2->sTypoDescender);
+	if (ttf->tab_OS_2->sTypoDescender)
+	    font->descender	= PDF_TT2PDF(ttf->tab_OS_2->sTypoDescender);
+	else
+	    font->descender	= PDF_TT2PDF(ttf->tab_hhea->descender);
+
+	/* sCapHeight and sxHeight are only valid if version >= 2 */
+	if (ttf->tab_OS_2->version >= 2 && ttf->tab_OS_2->sCapHeight) {
+	    font->capHeight	= PDF_TT2PDF(ttf->tab_OS_2->sCapHeight);
+	} else {
+	    font->capHeight	= font->ascender;	/* an educated guess */
+	}
+
+	if (ttf->tab_OS_2->version >= 2 && ttf->tab_OS_2->sxHeight) {
+	    font->xHeight	= PDF_TT2PDF(ttf->tab_OS_2->sxHeight);
+	} else {
+	    							/* guess */
+	    font->xHeight	= (int) ((float) 0.66 * font->ascender);
+	}
+    } else {
+	/* The OS/2 table is missing from some Apple fonts */
+	font->ascender	= PDF_TT2PDF(ttf->tab_hhea->ascender);
+	font->descender	= PDF_TT2PDF(ttf->tab_hhea->descender);
+
+	/* estimate the remaining values from the known ones */
+	font->xHeight	= (int)((float) 0.66 * font->ascender);
+	font->capHeight	= font->ascender;
+    }
 
     if (ttf->tab_cmap->win && ttf->tab_cmap->win->encodingID == tt_wenc_text)
 	font->encodingScheme = pdf_strdup(p, "AdobeStandardEncoding");
@@ -1341,10 +1471,23 @@ pdf_parse_tt(
 	int gidx, uv;
 
 	if (enc == macroman)
-	    gidx = tt_code2gidx0(p, ttf, i);
+	    gidx = tt_code2gidx0_6(p, ttf, i);
 
     	else if (font->flags & SYMBOL)
 	{
+	    /* This would be an Apple font with an encoding different
+	     * from macroman.
+	     */
+	    if (!ttf->tab_OS_2) {
+		pdf_cleanup_tt(p, ttf);
+		if (p->debug['F']) {
+		    pdf_error(p, PDF_RuntimeError,
+			"Symbol font '%s' doesn't contain OS/2 table",
+			font->ttname);
+		} else
+		    return pdf_false;
+	    }
+
 	    /* e.g. WingDings has usFirstChar = 0xF020, but we must start
 	    ** at 0xF000. I haven't seen non-symbol fonts with a usFirstChar
 	    ** like that; perhaps we have to apply similar tricks then...
@@ -1405,7 +1548,7 @@ pdf_get_metrics_tt(
 	}
     }
 
-#ifdef NYI
+#ifdef DONTUSE
     /* validate the font name if read from file */
     if (filename && strcmp(font->name, fontname)) {
 	if (p->debug['F']) {
@@ -1427,7 +1570,7 @@ pdf_put_ttfont(PDF *p, int slot)
 {
     int		i, j;
     id		encoding_id = BAD_ID, fontdescriptor_id, fontfile_id;
-    id		length_id, length1_id;
+    id		length_id, length1_id = BAD_ID;
     long	length, stream_start;
     pdf_font	*font;
 
@@ -1436,7 +1579,12 @@ pdf_put_ttfont(PDF *p, int slot)
     pdf_begin_obj(p, font->obj_id);		/* font attributes	*/
     pdf_begin_dict(p);
     pdf_puts(p, "/Type/Font\n");
-    pdf_puts(p, "/Subtype/TrueType\n");
+
+    if (font->type == Type1C)
+	pdf_puts(p, "/Subtype/Type1\n");	/* NOT /Type1C !!! */
+    else
+	pdf_puts(p, "/Subtype/TrueType\n");
+
     pdf_puts(p, "/FirstChar 0\n");
     pdf_puts(p, "/LastChar 255\n");
 
@@ -1461,7 +1609,15 @@ pdf_put_ttfont(PDF *p, int slot)
     }
 
     /* TrueType fonts use a different name at the API level than in the PDF */
-    pdf_printf(p, "/BaseFont/%s\n", font->ttname ? font->ttname : font->name);
+    pdf_puts(p, "/BaseFont/");
+#ifndef PDFLIB_EBCDIC
+    pdf_quote_name(p, font->ttname, strlen(font->ttname));
+#else
+    afontname = pdf_strdup(p, font->ttname);
+    pdf_make_ascii(afontname);
+    pdf_quote_name(p, afontname, strlen(afontname));
+#endif
+    pdf_puts(p, "\n");
 
     fontdescriptor_id = pdf_alloc_id(p);
     pdf_printf(p, "/FontDescriptor %ld 0 R\n", fontdescriptor_id);
@@ -1487,9 +1643,15 @@ pdf_put_ttfont(PDF *p, int slot)
 	pdf_begin_dict(p);
 
 	length_id = pdf_alloc_id(p);
-	length1_id = pdf_alloc_id(p);
 	pdf_printf(p, "/Length %ld 0 R\n", length_id);
-	pdf_printf(p, "/Length1 %ld 0 R\n", length1_id);
+
+	if (font->type == Type1C)
+	    pdf_puts(p, "/Subtype/Type1C\n");
+
+	if (font->type == TrueType) {
+	    length1_id = pdf_alloc_id(p);
+	    pdf_printf(p, "/Length1 %ld 0 R\n", length1_id);
+	}
 
 	if (p->compresslevel && font->filelen != 0L)
 	    pdf_puts(p, "/Filter/FlateDecode\n");
@@ -1508,6 +1670,16 @@ pdf_put_ttfont(PDF *p, int slot)
 	    src.fill        = pdf_data_source_file_fill;
 	    src.terminate   = pdf_data_source_file_terminate;
 
+	    if (font->type == TrueType) {
+		src.offset      = (long) 0;
+		src.length      = (long) 0;	/* the complete file */
+
+	    } else {	/* font->type == Type1C */
+
+		src.offset      = font->cff_offset;
+		src.length      = (long) font->cff_length;
+	    }
+
 	} else {
 
 	    /* read the font from memory */
@@ -1515,8 +1687,15 @@ pdf_put_ttfont(PDF *p, int slot)
 	    src.fill        = pdf_data_source_buf_fill;
 	    src.terminate   = pdf_noop;
 
-	    src.buffer_start = font->img;
-	    src.buffer_length = font->filelen;
+	    if (font->type == TrueType) {
+		src.buffer_start = font->img;
+		src.buffer_length = font->filelen;
+
+	    } else {	/* font->type == Type1C */
+
+		src.buffer_start = font->img + font->cff_offset;
+		src.buffer_length = font->cff_length;
+	    }
 
 	    src.bytes_available = 0;
 	    src.next_byte	= NULL;
@@ -1535,9 +1714,12 @@ pdf_put_ttfont(PDF *p, int slot)
 	pdf_printf(p, "%ld\n", length);
 	pdf_end_obj(p);
 
-	pdf_begin_obj(p, length1_id);		/* Length1 object */
-	pdf_printf(p, "%ld\n", (font->filelen != 0) ? font->filelen : length);
-	pdf_end_obj(p);
+	if (font->type == TrueType) {	/* not for Type1C fonts */
+	    pdf_begin_obj(p, length1_id);		/* Length1 object */
+	    pdf_printf(p, "%ld\n",
+		(font->filelen != 0) ? font->filelen : length);
+	    pdf_end_obj(p);
+	}
     }
 
     if (p->flush & PDF_FLUSH_CONTENT)
