@@ -1,7 +1,7 @@
 /*---------------------------------------------------------------------------*
  |              PDFlib - A library for generating PDF on the fly             |
  +---------------------------------------------------------------------------+
- | Copyright (c) 1997-2001 PDFlib GmbH and Thomas Merz. All rights reserved. |
+ | Copyright (c) 1997-2002 PDFlib GmbH and Thomas Merz. All rights reserved. |
  +---------------------------------------------------------------------------+
  |    This software is NOT in the public domain.  It can be used under two   |
  |    substantially different licensing terms:                               |
@@ -27,7 +27,7 @@
  |    however, will receive warranty and support statements in writing.      |
  *---------------------------------------------------------------------------*/
 
-/* $Id: p_util.c,v 1.26.10.4 2001/05/11 07:03:34 tm Exp $
+/* $Id: p_util.c,v 1.26.10.20 2002/05/28 14:35:22 tm Exp $
  *
  * PDFlib utility routines
  *
@@ -105,6 +105,34 @@ pdf_strlen(const char *text)
 	return strlen(text) + 1;
 }
 
+#if defined(MVS) && defined(I370)
+size_t
+pdf_ftell(FILE *fp)
+{
+#define PDF_FTELL_BUSIZE	32000
+
+    char buf[PDF_FTELL_BUSIZE];
+    size_t filelen = 0;
+
+    while (!feof(fp))
+	filelen += fread((void *) buf, 1, PDF_FTELL_BUSIZE, fp);
+
+    return filelen;
+}
+#endif
+
+char *
+pdf_getenv(const char *name)
+{
+#ifdef HAVE_ENVVARS
+    return getenv(name);
+#else
+    (void) name;
+
+    return (char *) 0;
+#endif
+}
+
 /* Allocate a PDFlib-local buffer and copy the string including
  * the terminating sentinel. If the string starts with the Unicode BOM
  * it is considered a Unicode string, and must be terminated by
@@ -172,6 +200,12 @@ pdf_quote_string(PDF *p, const char *text)
     pdf_quote_string2(p, text, len);
 }
 
+/* characters illegal in PDF names: "()<>[]{}/%#" */
+#define PDF_ILL_IN_NAMES "\050\051\074\076\133\135\173\175\057\045\043"
+
+#define PDF_NEEDS_QUOTE(c) \
+	((c) < 33 || (c) > 126 || strchr(PDF_ILL_IN_NAMES, (c)) != (char *) 0)
+
 void
 pdf_quote_name(PDF *p, const char *text, size_t len)
 {
@@ -181,7 +215,7 @@ pdf_quote_name(PDF *p, const char *text, size_t len)
     goal = (const unsigned char *) text + len;
 
     for (s = (const unsigned char *) text; s < goal; s++) {
-	if (*s < 33 || *s > 126) {
+	if (PDF_NEEDS_QUOTE(*s)) {
 	    pdf_putc(p, PDF_HASH);
 	    pdf_putc(p, BinToHex[*s >> 4]);	/* first nibble  */
 	    pdf_putc(p, BinToHex[*s & 0x0F]);	/* second nibble  */
@@ -202,7 +236,7 @@ pdf_make_quoted_name(PDF *p, const char *text, size_t len, char *buf)
     goal = (const unsigned char *) text + len;
 
     for (s = (const unsigned char *) text; s < goal; s++) {
-	if (*s < 33 || *s > 126) {
+	if (PDF_NEEDS_QUOTE(*s)) {
 	    *cur++ = PDF_HASH;
 	    *cur++ = BinToHex[*s >> 4];		/* first nibble */
 	    *cur++ = BinToHex[*s & 0x0F];	/* second nibble */
@@ -410,6 +444,10 @@ pdf_add_resource(PDF *p, const char *category, const char *resource,
 	    prefix += 2;
     }
 
+#ifdef  PDF_OS390_MVS_RESOURCE
+    prefix--; /* back up 1 to catch the leading "'" */
+#endif
+
     /* Go to the end of the resource list */
     for (res = cat->kids; res != (pdf_res *) NULL; res = res->next)
 	lastres = res;
@@ -424,7 +462,7 @@ pdf_add_resource(PDF *p, const char *category, const char *resource,
     res->next = NULL;
     res->name = pdf_strdup(p, resource);
 
-    /* we need two extra bytes for the OS/390's closing ")'" */
+    /* we need two extra bytes for MVS's closing ")'" */
     res->filename = (char *) p->malloc(p, strlen(filename) + strlen(prefix) + 4,
 		"pdf_add_resource");
 
@@ -432,9 +470,13 @@ pdf_add_resource(PDF *p, const char *category, const char *resource,
 	strcpy (res->filename, prefix);
 	strcat (res->filename, PATHSEP);
 	strcat (res->filename, filename);
-#ifdef  PDF_OS390_MVS_RESOURCE
-        strcat (res->filename, ")'");
-#endif  /* PDF_OS390_MVS_RESOURCE */
+#ifdef MVS
+#ifdef I370
+        strcat(res->filename, ")");
+#else
+	strcat(res->filename, ")'");
+#endif	/* I370 */
+#endif	/* MVS */
     } else
 	strcpy (res->filename, filename);
 }
@@ -481,31 +523,31 @@ pdf_init_resources(PDF *p)
     else
 	p->resourcefile_loaded = pdf_true;
 
-#ifdef HAVE_ENVVARS
     /* user-supplied upr file */
-    uprfilename = getenv(RESOURCEFILE);
-#else
-    uprfilename = NULL;
-#endif
+    uprfilename = pdf_getenv(RESOURCEFILE);
+
     /* client-supplied res file */
     if (uprfilename == NULL || *uprfilename == '\0')
 	uprfilename = p->resourcefilename;
 
     /* default resource file */
+    if (uprfilename == NULL || *uprfilename == '\0') {
 #ifndef AS400
-    if (uprfilename == NULL || *uprfilename == '\0')
-	uprfilename = pdf_strdup(p, DEFAULTRESOURCEFILE);
+	uprfilename = DEFAULTRESOURCEFILE;
 #else
-        strcpy (uprpath, RESOURCEROOT);             /* /pdflib	*/
-        strcat (uprpath, PATHSEP);                  /* /	*/
-        strcat (uprpath, PDFLIB_VERSIONSTRING);     /* <version>*/
-        strcat (uprpath, RESOURCEDIR);              /* /fonts	*/
-        strcat (uprpath, PATHSEP);                  /* /	*/
-        strcat (uprpath, DEFAULTRESOURCEFILE);
-        uprfilename = pdf_strdup(p, uprpath);
+	strcpy (uprpath, RESOURCEROOT);             /* /pdflib	*/
+	strcat (uprpath, PATHSEP);                  /* /	*/
+	strcat (uprpath, PDFLIB_VERSIONSTRING);     /* <version>*/
+	strcat (uprpath, RESOURCEDIR);              /* /fonts	*/
+	strcat (uprpath, PATHSEP);                  /* /	*/
+	strcat (uprpath, DEFAULTRESOURCEFILE);
+	uprfilename = uprpath;
 #endif	/* AS400 */
+    }
 
-    if ((fp = fopen(uprfilename, "r")) == NULL) {
+    fp = fopen(uprfilename, "r");
+
+    if (fp == NULL) {
 	if (p->debug['F'] == pdf_true)
 	    pdf_error(p, PDF_IOError,
 	    	"Resource configuration file '%s' not found", uprfilename);
@@ -625,7 +667,7 @@ pdf_load_encoding(PDF *p, const char *filename, const char *encoding)
 
 
     if ((fp = fopen(filename, "r")) == NULL) {
-	p->free(p, enc);
+	pdf_cleanup_enc(p, enc);
 	if (p->debug['F'] == pdf_true)
 	    pdf_error(p, PDF_IOError,
 		"Couldn't open encoding file '%s'", filename);
