@@ -13,7 +13,7 @@
  * and png_info.
  */
 
-/* $Id: pngconf.h,v 1.10 2001/04/02 13:11:31 rjs Exp $ */
+/* $Id: pngconf.h,v 1.10.2.4 2002/03/25 16:06:09 tm Exp $ */
 
 #ifndef PNGCONF_H
 #define PNGCONF_H
@@ -261,6 +261,23 @@
 #define png_get_error_ptr pdf_png_get_error_ptr
 #define png_set_error_fn pdf_png_set_error_fn
 #define png_warning pdf_png_warning
+
+/* the following added 2002-02-26 tm for libpng-1.2.1 compatibility*/
+
+#define png_create_read_struct_2 pdf_png_create_read_struct_2
+#define png_create_struct_2 pdf_png_create_struct_2
+#define png_default_error pdf_png_default_error
+#define png_default_read_data pdf_png_default_read_data
+#define png_default_warning pdf_png_default_warning
+#define png_destroy_struct_2 pdf_png_destroy_struct_2
+#define png_digit pdf_png_digit
+#define png_format_buffer pdf_png_format_buffer
+#define png_free_default pdf_png_free_default
+#define png_gamma_shift pdf_png_gamma_shift
+#define png_get_mem_ptr pdf_png_get_mem_ptr
+#define png_malloc_default pdf_png_malloc_default
+#define png_set_mem_fn pdf_png_set_mem_fn
+
 #endif
 
 /* This is the size of the compression buffer, and thus the size of
@@ -449,10 +466,6 @@
 #      define PNG_SAVE_BSD_SOURCE
 #      undef _BSD_SOURCE
 #    endif
-#    ifdef _SETJMP_H
-      __png.h__ already includes setjmp.h;
-      __dont__ include it again.;
-#    endif
 #endif /* __linux__ */
 
 /* include setjmp.h for error handling */
@@ -1215,11 +1228,7 @@ typedef png_int_16      FAR * png_int_16p;
 typedef PNG_CONST char  FAR * png_const_charp;
 typedef char            FAR * png_charp;
 typedef png_fixed_point FAR * png_fixed_point_p;
-#if defined(_WIN32_WCE)
-typedef HANDLE                png_FILE_p;
-#else
 typedef FILE                * png_FILE_p;
-#endif
 #ifdef PNG_FLOATING_POINT_SUPPORTED
 typedef double          FAR * png_doublep;
 #endif
@@ -1326,11 +1335,13 @@ typedef z_stream FAR *  png_zstreamp;
 #  ifdef __GNUC__
 #    define PNGAPI __cdecl
 #  else
-#if !defined(OS390)
-#    define PNGAPI __cdecl
-#else
-#    define PNGAPI /* */
-#endif
+#    if defined(_WIN32_WCE)
+#      define PNGAPI /* */
+#    elif !defined(OS390) && !defined(MVS)
+#      define PNGAPI __cdecl
+#    else
+#      define PNGAPI /* */
+#    endif
 #  endif
 
 #  if !defined(PNG_IMPEXP) && (!defined(PNG_DLL) || \
